@@ -1,12 +1,12 @@
 /*
-	Editor engine
-	Copyright
-		(C) 1992 Joseph H. Allen
-
-	This file is part of JOE (Joe's Own Editor)
-*/
-
+ *	Editor engine
+ *	Copyright
+ *		(C) 1992 Joseph H. Allen
+ *
+ *	This file is part of JOE (Joe's Own Editor)
+ */
 #include "config.h"
+#include "types.h"
 
 #include <unistd.h>
 #include <stdio.h>
@@ -24,20 +24,20 @@
 #include <time.h>
 #endif
 
-#include "bw.h"
-#include "blocks.h"
-#include "undo.h"
-#include "vs.h"
-#include "va.h"
-#include "utils.h"
-#include "path.h"
-#include "w.h"
-#include "tty.h"
-#include "scrn.h"
-#include "main.h"
-#include "uerror.h"
-
 #include "b.h"
+#include "blocks.h"
+#include "main.h"
+#include "path.h"
+#include "queue.h"
+#include "rc.h"
+#include "scrn.h"
+#include "uerror.h"
+#include "undo.h"
+#include "utils.h"
+#include "va.h"
+#include "vfile.h"
+#include "vs.h"
+#include "w.h"
 
 char stdbuf[stdsiz];
 
@@ -56,54 +56,57 @@ char *msgs[] = {
 };
 
 /* Get size of gap (amount of free space) */
-
-#define GGAPSZ(hdr) ((hdr)->ehole-(hdr)->hole)
+#define GGAPSZ(hdr) ((hdr)->ehole - (hdr)->hole)
 
 /* Get number of characters in gap buffer */
+#define GSIZE(hdr) (SEGSIZ - GGAPSZ(hdr))
 
-#define GSIZE(hdr) (SEGSIZ-GGAPSZ(hdr))
+/* Get char from buffer (with jumping around the gap) */
+#define GCHAR(p) ((p)->ofst >= (p)->hdr->hole ? (p)->ptr[(p)->ofst + GGAPSZ((p)->hdr)] \
+					      : (p)->ptr[(p)->ofst])
 
 /* Set position of gap */
-
-static void gstgap(H * hdr, char *ptr, int ofst)
+static void gstgap(H *hdr, char *ptr, int ofst)
 {
-	if (ofst > hdr->hole)
-		mfwrd(ptr + hdr->hole, ptr + hdr->ehole, ofst - hdr->hole), vchanged(ptr);
-	else if (ofst < hdr->hole)
-		mbkwd(ptr + hdr->ehole - (hdr->hole - ofst), ptr + ofst, hdr->hole - ofst), vchanged(ptr);
+	if (ofst > hdr->hole) {
+		mmove(ptr + hdr->hole, ptr + hdr->ehole, ofst - hdr->hole);
+		vchanged(ptr);
+	} else if (ofst < hdr->hole) {
+		mmove(ptr + hdr->ehole - (hdr->hole - ofst), ptr + ofst, hdr->hole - ofst);
+		vchanged(ptr);
+	}
 	hdr->ehole = ofst + hdr->ehole - hdr->hole;
 	hdr->hole = ofst;
 }
 
 /* Insert a block */
-
-static void ginsm(H * hdr, char *ptr, int ofst, char *blk, int size)
+static void ginsm(H *hdr, char *ptr, int ofst, char *blk, int size)
 {
 	if (ofst != hdr->hole)
 		gstgap(hdr, ptr, ofst);
-	mcpy(ptr + hdr->hole, blk, size);
+	mmove(ptr + hdr->hole, blk, size);
 	hdr->hole += size;
 	vchanged(ptr);
 }
 
 /* Read block */
-
-static void grmem(H * hdr, char *ptr, int ofst, char *blk, int size)
+static void grmem(H *hdr, char *ptr, int ofst, char *blk, int size)
 {
 	if (ofst < hdr->hole)
-		if (size > hdr->hole - ofst)
-			mcpy(blk, ptr + ofst, hdr->hole - ofst), mcpy(blk + hdr->hole - ofst, ptr + hdr->ehole, size - (hdr->hole - ofst));
-		else
-			mcpy(blk, ptr + ofst, size);
+		if (size > hdr->hole - ofst) {
+			mmove(blk, ptr + ofst, hdr->hole - ofst);
+			mmove(blk + hdr->hole - ofst, ptr + hdr->ehole, size - (hdr->hole - ofst));
+		} else
+			mmove(blk, ptr + ofst, size);
 	else
-		mcpy(blk, ptr + ofst + hdr->ehole - hdr->hole, size);
+		mmove(blk, ptr + ofst + hdr->ehole - hdr->hole, size);
 }
 
-/* Header allocation */
 
 static H nhdrs = { {&nhdrs, &nhdrs} };
 static H ohdrs = { {&ohdrs, &ohdrs} };
 
+/* Header allocation */
 static H *halloc(void)
 {
 	H *h;
@@ -120,32 +123,31 @@ static H *halloc(void)
 	return h;
 }
 
-static void hfree(H * h)
+static void hfree(H *h)
 {
 	enquef(H, link, &ohdrs, h);
 }
 
-static void hfreechn(H * h)
+static void hfreechn(H *h)
 {
 	splicef(H, link, &ohdrs, h);
 }
 
-/* Pointer allocation */
 
 static P frptrs = { {&frptrs, &frptrs} };
 
+/* Pointer allocation */
 static P *palloc(void)
 {
 	return alitem(&frptrs, sizeof(P));
 }
 
-static void pfree(P * p)
+static void pfree(P *p)
 {
 	enquef(P, link, &frptrs, p);
 }
 
 /* Doubly linked list of buffers and free buffer structures */
-
 static B bufs = { {&bufs, &bufs} };
 static B frebufs = { {&frebufs, &frebufs} };
 
@@ -157,8 +159,7 @@ B *bnext(void)
 		b = bufs.link.prev;
 		deque(B, link, &bufs);
 		enqueb(B, link, b, &bufs);
-	}
-	while (b->internal);
+	} while (b->internal);
 	return b;
 }
 
@@ -170,14 +171,12 @@ B *bprev(void)
 		b = bufs.link.next;
 		deque(B, link, &bufs);
 		enquef(B, link, b, &bufs);
-	}
-	while (b->internal);
+	} while (b->internal);
 	return b;
 }
 
 /* Make a buffer out of a chain */
-
-static B *bmkchn(H * chn, B * prop, long amnt, long nlines)
+static B *bmkchn(H *chn, B *prop, long amnt, long nlines)
 {
 	B *b = alitem(&frebufs, sizeof(B));
 
@@ -189,19 +188,19 @@ static B *bmkchn(H * chn, B * prop, long amnt, long nlines)
 	mset(b->marks, 0, sizeof(b->marks));
 	b->rdonly = 0;
 	b->orphan = 0;
-	b->oldcur = 0;
-	b->oldtop = 0;
+	b->oldcur = NULL;
+	b->oldtop = NULL;
 	b->backup = 1;
 	b->internal = 1;
 	b->changed = 0;
 	b->count = 1;
-	b->name = 0;
+	b->name = NULL;
 	b->er = -3;
 	b->bof = palloc();
 	izque(P, link, b->bof);
 	b->bof->end = 0;
 	b->bof->b = b;
-	b->bof->owner = 0;
+	b->bof->owner = NULL;
 	b->bof->hdr = chn;
 	b->bof->ptr = vlock(vmem, b->bof->hdr->seg);
 	b->bof->ofst = 0;
@@ -226,23 +225,22 @@ static B *bmkchn(H * chn, B * prop, long amnt, long nlines)
 }
 
 /* Create an empty buffer */
-
-B *bmk(B * prop)
+B *bmk(B *prop)
 {
 	return bmkchn(halloc(), prop, 0L, 0L);
 }
 
-/* Eliminate a buffer */
 
 extern B *errbuf;
 
-void brm(B * b)
+/* Eliminate a buffer */
+void brm(B *b)
 {
 	if (b && !--b->count) {
 		if (b->changed)
 			abrerr(b->name);
 		if (b == errbuf)
-			errbuf = 0;
+			errbuf = NULL;
 		if (b->undo)
 			undorm(b->undo);
 		hfreechn(b->eof->hdr);
@@ -250,64 +248,64 @@ void brm(B * b)
 			prm(b->bof->link.next);
 		prm(b->bof);
 		if (b->name)
-			free(b->name);
+			joe_free(b->name);
 		demote(B, link, &frebufs, b);
 	}
 }
 
-P *poffline(P * p)
+P *poffline(P *p)
 {
 	if (p->ptr) {
 		vunlock(p->ptr);
-		p->ptr = 0;
+		p->ptr = NULL;
 	}
 	return p;
 }
 
-P *ponline(P * p)
+P *ponline(P *p)
 {
 	if (!p->ptr)
 		p->ptr = vlock(vmem, p->hdr->seg);
 	return p;
 }
 
-B *boffline(B * b)
+B *boffline(B *b)
 {
 	P *p = b->bof;
 
-	do
+	do {
 		poffline(p);
-	while ((p = p->link.next) != b->bof);
+	} while ((p = p->link.next) != b->bof);
 	return b;
 }
 
-B *bonline(B * b)
+B *bonline(B *b)
 {
 	P *p = b->bof;
 
-	do
+	do {
 		ponline(p);
-	while ((p = p->link.next) != b->bof);
+	} while ((p = p->link.next) != b->bof);
 	return b;
 }
 
-P *pdup(P * p)
+P *pdup(P *p)
 {
 	P *n = palloc();
 
 	n->end = 0;
-	n->ptr = 0;
-	n->owner = 0;
+	n->ptr = NULL;
+	n->owner = NULL;
 	enquef(P, link, p, n);
 	return pset(n, p);
 }
 
-P *pdupown(P * p, P ** o)
+P *pdupown(P *p, P **o)
 {
 	P *n = palloc();
 
 	n->end = 0;
-	n->ptr = 0;
+	n->ptr = NULL;
 	n->owner = o;
 	enquef(P, link, p, n);
 	pset(n, p);
@@ -317,18 +315,18 @@ P *pdupown(P * p, P ** o)
 	return n;
 }
 
-void prm(P * p)
+void prm(P *p)
 {
 	if (!p)
 		return;
 	if (p->owner)
-		*p->owner = 0;
+		*p->owner = NULL;
 	if (p->ptr)
 		vunlock(p->ptr);
 	pfree(deque_f(P, link, p));
 }
 
-P *pset(P * n, P * p)
+P *pset(P *n, P *p)
 {
 	if (n != p) {
 		n->b = p->b;
@@ -349,27 +347,30 @@ P *pset(P * n, P * p)
 	return n;
 }
 
-P *p_goto_bof(P * p)
+P *p_goto_bof(P *p)
 {
 	return pset(p, p->b->bof);
 }
 
-P *p_goto_eof(P * p)
+P *p_goto_eof(P *p)
 {
 	return pset(p, p->b->eof);
 }
 
-int pisbof(P * p)
+/* is p at the beginning of file? */
+int pisbof(P *p)
 {
 	return p->hdr == p->b->bof->hdr && !p->ofst;
 }
 
-int piseof(P * p)
+/* is p at the end of file? */
+int piseof(P *p)
 {
 	return p->ofst == GSIZE(p->hdr);
 }
 
-int piseol(P * p)
+/* is p at the end of line? */
+int piseol(P *p)
 {
 	int c;
 
@@ -392,7 +393,8 @@ int piseol(P * p)
 	return 0;
 }
 
-int pisbol(P * p)
+/* is p at the beginning of line? */
+int pisbol(P *p)
 {
 	char c;
 
@@ -403,33 +405,36 @@ int pisbol(P * p)
 	return c == '\n';
 }
 
-int pisbow(P * p)
+/* is p at the beginning of word? */
+int pisbow(P *p)
 {
 	P *q = pdup(p);
 	int c = brc(p);
 	int d = prgetc(q);
 
 	prm(q);
-	if (isalnum_(c) && !isalnum_(d))
+	if (isalnum_(c) && (!isalnum_(d) || pisbof(p)))
 		return 1;
 	else
 		return 0;
 }
 
-int piseow(P * p)
+/* is p at the end of word? */
+int piseow(P *p)
 {
 	P *q = pdup(p);
 	int d = brc(q);
 	int c = prgetc(q);
 
 	prm(q);
-	if (isalnum_(c) && !isalnum_(d))
+	if (isalnum_(c) && (!isalnum_(d) || piseof(p)))
 		return 1;
 	else
 		return 0;
 }
 
-int pisblank(P * p)
+/* is p on the blank line (ie. full of spaces/tabs)? */
+int pisblank(P *p)
 {
 	P *q = pdup(p);
 
@@ -445,7 +450,8 @@ int pisblank(P * p)
 	}
 }
 
-long pisindent(P * p)
+/* return column of first nonblank character */
+long pisindent(P *p)
 {
 	P *q = pdup(p);
 	long col;
@@ -458,7 +464,7 @@ long pisindent(P * p)
 	return col;
 }
 
-int pnext(P * p)
+int pnext(P *p)
 {
 	if (p->hdr == p->b->eof->hdr) {
 		p->ofst = GSIZE(p->hdr);
@@ -471,7 +477,7 @@ int pnext(P * p)
 	return 1;
 }
 
-int pprev(P * p)
+int pprev(P *p)
 {
 	if (p->hdr == p->b->bof->hdr) {
 		p->ofst = 0;
@@ -484,16 +490,14 @@ int pprev(P * p)
 	return 1;
 }
 
-int pgetc(P * p)
+/* return current character and move p to the next character */
+int pgetc(P *p)
 {
 	unsigned char c;
 
 	if (p->ofst == GSIZE(p->hdr))
-		return MAXINT;
-	if (p->ofst >= p->hdr->hole)
-		c = p->ptr[p->ofst + p->hdr->ehole - p->hdr->hole];
-	else
-		c = p->ptr[p->ofst];
+		return NO_MORE_DATA;
+	c = GCHAR(p);
 	if (++p->ofst == GSIZE(p->hdr))
 		pnext(p);
 	++p->byte;
@@ -516,7 +520,8 @@ int pgetc(P * p)
 	return c;
 }
 
-P *pfwrd(P * p, long n)
+/* move p n characters forward */
+P *pfwrd(P *p, long n)
 {
 	if (!n)
 		return p;
@@ -524,21 +529,19 @@ P *pfwrd(P * p, long n)
 	do {
 		if (p->ofst == GSIZE(p->hdr))
 			do {
-				if (!p->ofst)
-					p->byte += GSIZE(p->hdr), n -= GSIZE(p->hdr), p->line += p->hdr->nlines;
+				if (!p->ofst) {
+					p->byte += GSIZE(p->hdr);
+					n -= GSIZE(p->hdr);
+					p->line += p->hdr->nlines;
+				}
 				if (!pnext(p))
-					return 0;
-			}
-			while (n > GSIZE(p->hdr));
-		if (p->ofst >= p->hdr->hole) {
-			if (p->ptr[p->ofst + p->hdr->ehole - p->hdr->hole] == '\n')
-				++p->line;
-		} else if (p->ptr[p->ofst] == '\n')
+					return NULL;
+			} while (n > GSIZE(p->hdr));
+		if (GCHAR(p) == '\n')
 			++p->line;
 		++p->byte;
 		++p->ofst;
-	}
-	while (--n);
+	} while (--n);
 	if (p->ofst == GSIZE(p->hdr))
 		pnext(p);
 	return p;
@@ -550,16 +553,14 @@ static int prgetc1(P *p)
 
 	if (!p->ofst)
 		if (!pprev(p))
-			return MAXINT;
+			return NO_MORE_DATA;
 	--p->ofst;
-	if (p->ofst >= p->hdr->hole)
-		c = p->ptr[p->ofst + p->hdr->ehole - p->hdr->hole];
-	else
-		c = p->ptr[p->ofst];
+	c = GCHAR(p);
 	--p->byte;
-	if (c == '\n')
-		--p->line, p->valcol = 0;
-	else {
+	if (c == '\n') {
+		--p->line;
+		p->valcol = 0;
+	} else {
 		if (c == '\t')
 			p->valcol = 0;
 		--p->col;
@@ -567,7 +568,8 @@ static int prgetc1(P *p)
 	return c;
 }
 
-int prgetc(P * p)
+/* move p to the previous character */
+int prgetc(P *p)
 {
 	int c = prgetc1(p);
 
@@ -575,14 +577,15 @@ int prgetc(P * p)
 		c = prgetc1(p);
 		if (c == '\r')
 			return '\n';
-		if (c != MAXINT)
+		if (c != NO_MORE_DATA)
 			pgetc(p);
 		c = '\n';
 	}
 	return c;
 }
 
-P *pbkwd(P * p, long n)
+/* move p n characters backwards */
+P *pbkwd(P *p, long n)
 {
 	if (!n)
 		return p;
@@ -590,25 +593,24 @@ P *pbkwd(P * p, long n)
 	do {
 		if (!p->ofst)
 			do {
-				if (p->ofst)
-					p->byte -= p->ofst, n -= p->ofst, p->line -= p->hdr->nlines;
+				if (p->ofst) {
+					p->byte -= p->ofst;
+					n -= p->ofst;
+					p->line -= p->hdr->nlines;
+				}
 				if (!pprev(p))
-					return 0;
-			}
-			while (n > GSIZE(p->hdr));
+					return NULL;
+			} while (n > GSIZE(p->hdr));
 		--p->ofst;
 		--p->byte;
-		if (p->ofst >= p->hdr->hole) {
-			if (p->ptr[p->ofst + p->hdr->ehole - p->hdr->hole] == '\n')
-				--p->line;
-		} else if (p->ptr[p->ofst] == '\n')
+		if (GCHAR(p) == '\n')
 			--p->line;
-	}
-	while (--n);
+	} while (--n);
 	return p;
 }
 
-P *pgoto(P * p, long loc)
+/* move p n characters forwards/backwards according to loc */
+P *pgoto(P *p, long loc)
 {
 	if (loc > p->byte)
 		pfwrd(p, loc - p->byte);
@@ -617,7 +619,8 @@ P *pgoto(P * p, long loc)
 	return p;
 }
 
-P *pfcol(P * p)
+/* make p->col valid */
+P *pfcol(P *p)
 {
 	H *hdr = p->hdr;
 	int ofst = p->ofst;
@@ -628,7 +631,8 @@ P *pfcol(P * p)
 	return p;
 }
 
-P *p_goto_bol(P * p)
+/* move p to the beginning of line */
+P *p_goto_bol(P *p)
 {
 	if (pprevl(p))
 		pgetc(p);
@@ -637,7 +641,8 @@ P *p_goto_bol(P * p)
 	return p;
 }
 
-P *p_goto_eol(P * p)
+/* move p to the end of line */
+P *p_goto_eol(P *p)
 {
 	if (p->b->o.crlf)
 		while (!piseol(p))
@@ -646,10 +651,7 @@ P *p_goto_eol(P * p)
 		while (p->ofst != GSIZE(p->hdr)) {
 			unsigned char c;
 
-			if (p->ofst >= p->hdr->hole)
-				c = p->ptr[p->ofst + p->hdr->ehole - p->hdr->hole];
-			else
-				c = p->ptr[p->ofst];
+			c = GCHAR(p);
 			if (c == '\n')
 				break;
 			else {
@@ -666,7 +668,8 @@ P *p_goto_eol(P * p)
 	return p;
 }
 
-P *pnextl(P * p)
+/* move p to the beginning of next line */
+P *pnextl(P *p)
 {
 	char c;
 
@@ -675,17 +678,12 @@ P *pnextl(P * p)
 			do {
 				p->byte += GSIZE(p->hdr) - p->ofst;
 				if (!pnext(p))
-					return 0;
-			}
-			while (!p->hdr->nlines);
-		if (p->ofst >= p->hdr->hole)
-			c = p->ptr[p->ofst + p->hdr->ehole - p->hdr->hole];
-		else
-			c = p->ptr[p->ofst];
+					return NULL;
+			} while (!p->hdr->nlines);
+		c = GCHAR(p);
 		++p->byte;
 		++p->ofst;
-	}
-	while (c != '\n');
+	} while (c != '\n');
 	++p->line;
 	p->col = 0;
 	p->valcol = 1;
@@ -694,7 +692,8 @@ P *pnextl(P * p)
 	return p;
 }
 
-P *pprevl(P * p)
+/* move p to the end of previous line */
+P *pprevl(P *p)
 {
 	char c;
 
@@ -704,28 +703,24 @@ P *pprevl(P * p)
 			do {
 				p->byte -= p->ofst;
 				if (!pprev(p))
-					return 0;
-			}
-			while (!p->hdr->nlines);
+					return NULL;
+			} while (!p->hdr->nlines);
 		--p->ofst;
 		--p->byte;
-		if (p->ofst >= p->hdr->hole)
-			c = p->ptr[p->ofst + p->hdr->ehole - p->hdr->hole];
-		else
-			c = p->ptr[p->ofst];
-	}
-	while (c != '\n');
+		c = GCHAR(p);
+	} while (c != '\n');
 	--p->line;
 	if (p->b->o.crlf && c == '\n') {
 		int k = prgetc1(p);
 
-		if (k != '\r' && k != MAXINT)
+		if (k != '\r' && k != NO_MORE_DATA)
 			pgetc(p);
 	}
 	return p;
 }
 
-P *pline(P * p, long line)
+/* move p to the given 'line' line */
+P *pline(P *p, long line)
 {
 	if (line > p->b->eof->line) {
 		pset(p, p->b->eof);
@@ -749,7 +744,8 @@ P *pline(P * p, long line)
 	return p;
 }
 
-P *pcol(P * p, long goalcol)
+/* move p to the given 'goalcol' column */
+P *pcol(P *p, long goalcol)
 {
 	p_goto_bol(p);
 	do {
@@ -758,10 +754,7 @@ P *pcol(P * p, long goalcol)
 
 		if (p->ofst == GSIZE(p->hdr))
 			break;
-		if (p->ofst >= p->hdr->hole)
-			c = p->ptr[p->ofst + p->hdr->ehole - p->hdr->hole];
-		else
-			c = p->ptr[p->ofst];
+		c = GCHAR(p);
 		if (c == '\n')
 			break;
 		if (p->b->o.crlf && c == '\r' && piseol(p))
@@ -776,25 +769,26 @@ P *pcol(P * p, long goalcol)
 			pnext(p);
 		++p->byte;
 		p->col += wid;
-	}
-	while (p->col != goalcol);
+	} while (p->col != goalcol);
 	return p;
 }
 
-P *pcolwse(P * p, long goalcol)
+/* move p into given given 'goalcol' column (even within whitespaces) */
+P *pcolwse(P *p, long goalcol)
 {
 	int c;
 
 	pcol(p, goalcol);
-	do
+	do {
 		c = prgetc(p);
-	while (c == ' ' || c == '\t');
-	if (c != MAXINT)
+	} while (c == ' ' || c == '\t');
+	if (c != NO_MORE_DATA)
 		pgetc(p);
 	return p;
 }
 
-P *pcoli(P * p, long goalcol)
+/* FIXME: whats the differnce between pcol() and pcoli() ??? */
+P *pcoli(P *p, long goalcol)
 {
 	p_goto_bol(p);
 	while (p->col < goalcol) {
@@ -802,10 +796,7 @@ P *pcoli(P * p, long goalcol)
 
 		if (p->ofst == GSIZE(p->hdr))
 			break;
-		if (p->ofst >= p->hdr->hole)
-			c = p->ptr[p->ofst + p->hdr->ehole - p->hdr->hole];
-		else
-			c = p->ptr[p->ofst];
+		c = GCHAR(p);
 		if (c == '\n')
 			break;
 #ifdef __MSDOS
@@ -823,46 +814,50 @@ P *pcoli(P * p, long goalcol)
 	return p;
 }
 
-void pfill(P * p, long to, int usetabs)
+/* fill space between curent column and 'to' column with tabs/spaces */
+void pfill(P *p, long to, int usetabs)
 {
 	piscol(p);
 	if (usetabs)
 		while (p->col < to)
-			if (p->col + p->b->o.tab - p->col % p->b->o.tab <= to)
-				binsc(p, '\t'), pgetc(p);
-			else
-				binsc(p, ' '), pgetc(p);
+			if (p->col + p->b->o.tab - p->col % p->b->o.tab <= to) {
+				binsc(p, '\t');
+				pgetc(p);
+			} else {
+				binsc(p, ' ');
+				pgetc(p);
+			}
 	else
-		while (p->col < to)
-			binsc(p, ' '), pgetc(p);
+		while (p->col < to) {
+			binsc(p, ' ');
+			pgetc(p);
+		}
 }
 
-void pbackws(P * p)
+/* delete sequence of whitespaces - backwards */
+void pbackws(P *p)
 {
 	int c;
 	P *q = pdup(p);
 
-	do
+	do {
 		c = prgetc(q);
-	while (c == ' ' || c == '\t');
-	if (c != MAXINT)
+	} while (c == ' ' || c == '\t');
+	if (c != NO_MORE_DATA)
 		pgetc(q);
 	bdel(q, p);
 	prm(q);
 }
 
-static char frgetc(P * p)
+static char frgetc(P *p)
 {
 	if (!p->ofst)
 		pprev(p);
 	--p->ofst;
-	if (p->ofst >= p->hdr->hole)
-		return p->ptr[p->ofst + p->hdr->ehole - p->hdr->hole];
-	else
-		return p->ptr[p->ofst];
+	return GCHAR(p);
 }
 
-static void ffwrd(P * p, int n)
+static void ffwrd(P *p, int n)
 {
 	while (n > GSIZE(p->hdr) - p->ofst) {
 		n -= GSIZE(p->hdr) - p->ofst;
@@ -873,14 +868,15 @@ static void ffwrd(P * p, int n)
 		pnext(p);
 }
 
-static P *ffind(P * p, unsigned char *s, int len)
+/* forward find pattern 's' in text pointed by 'p' (Boyer-Moore algorithm) */
+static P *ffind(P *p, unsigned char *s, int len)
 {
 	long amnt = p->b->eof->byte - p->byte;
 	int x;
 	unsigned char table[256], c;
 
 	if (len > amnt)
-		return 0;
+		return NULL;
 	if (!len)
 		return p;
 	p->valcol = 0;
@@ -890,31 +886,36 @@ static P *ffind(P * p, unsigned char *s, int len)
 	ffwrd(p, len);
 	amnt -= len;
 	x = len;
-	do
+	do {
 		if ((c = frgetc(p)) != s[--x]) {
-			if (table[c] == 255)
-				ffwrd(p, len + 1), amnt -= x + 1;
-			else if (x <= table[c])
-				ffwrd(p, len - x + 1), --amnt;
-			else
-				ffwrd(p, len - table[c]), amnt -= x - table[c];
+			if (table[c] == 255) {
+				ffwrd(p, len + 1);
+				amnt -= x + 1;
+			} else if (x <= table[c]) {
+				ffwrd(p, len - x + 1);
+				--amnt;
+			} else {
+				ffwrd(p, len - table[c]);
+				amnt -= x - table[c];
+			}
 			if (amnt < 0)
-				return 0;
+				return NULL;
 			else
 				x = len;
 		}
-	while (x) ;
+	} while (x);
 	return p;
 }
 
-static P *fifind(P * p, unsigned char *s, int len)
+/* forward find (case insensitive) pattern 's' in text pointed by 'p' (Boyer-Moore algorithm) */
+static P *fifind(P *p, unsigned char *s, int len)
 {
 	long amnt = p->b->eof->byte - p->byte;
 	int x;
 	unsigned char table[256], c;
 
 	if (len > amnt)
-		return 0;
+		return NULL;
 	if (!len)
 		return p;
 	p->valcol = 0;
@@ -924,44 +925,49 @@ static P *fifind(P * p, unsigned char *s, int len)
 	ffwrd(p, len);
 	amnt -= len;
 	x = len;
-	do
+	do {
 		if ((c = toupper(frgetc(p))) != s[--x]) {
-			if (table[c] == 255)
-				ffwrd(p, len + 1), amnt -= x + 1;
-			else if (x <= table[c])
-				ffwrd(p, len - x + 1), --amnt;
-			else
-				ffwrd(p, len - table[c]), amnt -= x - table[c];
+			if (table[c] == 255) {
+				ffwrd(p, len + 1);
+				amnt -= x + 1;
+			} else if (x <= table[c]) {
+				ffwrd(p, len - x + 1);
+				--amnt;
+			} else {
+				ffwrd(p, len - table[c]);
+				amnt -= x - table[c];
+			}
 			if (amnt < 0)
-				return 0;
+				return NULL;
 			else
 				x = len;
 		}
-	while (x) ;
+	} while (x);
 	return p;
 }
 
-static P *getto(P * p, P * q)
+/* move cursor p to q's position and set p's col, line, ofst, byte etc. accordingly */
+/* same as rgetto() but p is before q */
+static P *getto(P *p, P *q)
 {
 	while (p->hdr != q->hdr || p->ofst != q->ofst) {
-		if (p->ofst >= p->hdr->hole) {
-			if (p->ptr[p->ofst + p->hdr->ehole - p->hdr->hole] == '\n')
-				++p->line;
-		} else if (p->ptr[p->ofst] == '\n')
+		if (GCHAR(p) == '\n')
 			++p->line;
 		++p->byte;
 		++p->ofst;
 		if (p->ofst == GSIZE(p->hdr))
 			pnext(p);
 		while (!p->ofst && p->hdr != q->hdr) {
-			p->byte += GSIZE(p->hdr), p->line += p->hdr->nlines;
+			p->byte += GSIZE(p->hdr);
+			p->line += p->hdr->nlines;
 			pnext(p);
 		}
 	}
 	return p;
 }
 
-P *pfind(P * p, char *s, int len)
+/* find forward substring s in text pointed by p and set p after found substring */
+P *pfind(P *p, unsigned char *s, int len)
 {
 	P *q = pdup(p);
 
@@ -971,11 +977,12 @@ P *pfind(P * p, char *s, int len)
 		return p;
 	} else {
 		prm(q);
-		return 0;
+		return NULL;
 	}
 }
 
-P *pifind(P * p, char *s, int len)
+/* same as pfind() but case insensitive */
+P *pifind(P *p, unsigned char *s, int len)
 {
 	P *q = pdup(p);
 
@@ -985,11 +992,11 @@ P *pifind(P * p, char *s, int len)
 		return p;
 	} else {
 		prm(q);
-		return 0;
+		return NULL;
 	}
 }
 
-static void fbkwd(P * p, int n)
+static void fbkwd(P *p, int n)
 {
 	while (n > p->ofst) {
 		n -= p->ofst;
@@ -1002,22 +1009,20 @@ static void fbkwd(P * p, int n)
 		p->ofst = 0;
 }
 
-static int fpgetc(P * p)
+static int fpgetc(P *p)
 {
 	char c;
 
 	if (p->ofst == GSIZE(p->hdr))
-		return MAXINT;
-	if (p->ofst >= p->hdr->hole)
-		c = p->ptr[p->ofst + p->hdr->ehole - p->hdr->hole];
-	else
-		c = p->ptr[p->ofst];
+		return NO_MORE_DATA;
+	c = GCHAR(p);
 	if (++p->ofst == GSIZE(p->hdr))
 		pnext(p);
 	return c;
 }
 
-static P *frfind(P * p, unsigned char *s, int len)
+/* backward find pattern 's' in text pointed by 'p' (Boyer-Moore algorithm) */
+static P *frfind(P *p, unsigned char *s, int len)
 {
 	long amnt = p->byte;
 	int x;
@@ -1026,7 +1031,7 @@ static P *frfind(P * p, unsigned char *s, int len)
 	if (len > p->b->eof->byte - p->byte) {
 		x = len - (p->b->eof->byte - p->byte);
 		if (amnt < x)
-			return 0;
+			return NULL;
 		amnt -= x;
 		fbkwd(p, x);
 	}
@@ -1036,25 +1041,30 @@ static P *frfind(P * p, unsigned char *s, int len)
 	mset(table, 255, 256);
 	for (x = len; --x; table[s[x]] = len - x - 1) ;
 	x = 0;
-	do
+	do {
 		if ((c = fpgetc(p)) != s[x++]) {
-			if (table[c] == 255)
-				fbkwd(p, len + 1), amnt -= len - x + 1;
-			else if (len - table[c] <= x)
-				fbkwd(p, x + 1), --amnt;
-			else
-				fbkwd(p, len - table[c]), amnt -= len - table[c] - x;
+			if (table[c] == 255) {
+				fbkwd(p, len + 1);
+				amnt -= len - x + 1;
+			} else if (len - table[c] <= x) {
+				fbkwd(p, x + 1);
+				--amnt;
+			} else {
+				fbkwd(p, len - table[c]);
+				amnt -= len - table[c] - x;
+			}
 			if (amnt < 0)
-				return 0;
+				return NULL;
 			else
 				x = 0;
 		}
-	while (x != len) ;
+	} while (x != len);
 	fbkwd(p, len);
 	return p;
 }
 
-static P *frifind(P * p, unsigned char *s, int len)
+/* backward find (case insensitive) pattern 's' in text pointed by 'p' (Boyer-Moore algorithm) */
+static P *frifind(P *p, unsigned char *s, int len)
 {
 	long amnt = p->byte;
 	int x;
@@ -1063,7 +1073,7 @@ static P *frifind(P * p, unsigned char *s, int len)
 	if (len > p->b->eof->byte - p->byte) {
 		x = len - (p->b->eof->byte - p->byte);
 		if (amnt < x)
-			return 0;
+			return NULL;
 		amnt -= x;
 		fbkwd(p, x);
 	}
@@ -1073,46 +1083,51 @@ static P *frifind(P * p, unsigned char *s, int len)
 	mset(table, 255, 256);
 	for (x = len; --x; table[s[x]] = len - x - 1) ;
 	x = 0;
-	do
+	do {
 		if ((c = toupper(fpgetc(p))) != s[x++]) {
-			if (table[c] == 255)
-				fbkwd(p, len + 1), amnt -= len - x + 1;
-			else if (len - table[c] <= x)
-				fbkwd(p, x + 1), --amnt;
-			else
-				fbkwd(p, len - table[c]), amnt -= len - table[c] - x;
+			if (table[c] == 255) {
+				fbkwd(p, len + 1);
+				amnt -= len - x + 1;
+			} else if (len - table[c] <= x) {
+				fbkwd(p, x + 1);
+				--amnt;
+			} else {
+				fbkwd(p, len - table[c]);
+				amnt -= len - table[c] - x;
+			}
 			if (amnt < 0)
-				return 0;
+				return NULL;
 			else
 				x = 0;
 		}
-	while (x != len) ;
+	} while (x != len);
 	fbkwd(p, len);
 	return p;
 }
 
-static P *rgetto(P * p, P * q)
+/* move cursor p to q's position and set p's col, line, ofst, byte etc. accordingly */
+/* same as getto() but q is before p */
+static P *rgetto(P *p, P *q)
 {
 	while (p->hdr != q->hdr || p->ofst != q->ofst) {
 		if (!p->ofst)
 			do {
-				if (p->ofst)
-					p->byte -= p->ofst, p->line -= p->hdr->nlines;
+				if (p->ofst) {
+					p->byte -= p->ofst;
+					p->line -= p->hdr->nlines;
+				}
 				pprev(p);
-			}
-			while (p->hdr != q->hdr);
+			} while (p->hdr != q->hdr);
 		--p->ofst;
 		--p->byte;
-		if (p->ofst >= p->hdr->hole) {
-			if (p->ptr[p->ofst + p->hdr->ehole - p->hdr->hole] == '\n')
-				--p->line;
-		} else if (p->ptr[p->ofst] == '\n')
+		if (GCHAR(p) == '\n')
 			--p->line;
 	}
 	return p;
 }
 
-P *prfind(P * p, char *s, int len)
+/* find backward substring s in text pointed by p and set p on the first of found substring */
+P *prfind(P *p, unsigned char *s, int len)
 {
 	P *q = pdup(p);
 
@@ -1122,11 +1137,12 @@ P *prfind(P * p, char *s, int len)
 		return p;
 	} else {
 		prm(q);
-		return 0;
+		return NULL;
 	}
 }
 
-P *prifind(P * p, char *s, int len)
+/* same as prfind() but case insensitive */
+P *prifind(P *p, unsigned char *s, int len)
 {
 	P *q = pdup(p);
 
@@ -1136,11 +1152,12 @@ P *prifind(P * p, char *s, int len)
 		return p;
 	} else {
 		prm(q);
-		return 0;
+		return NULL;
 	}
 }
 
-B *bcpy(P * from, P * to)
+/* copy text between 'from' and 'to' into new buffer */
+B *bcpy(P *from, P *to)
 {
 	H anchor, *l;
 	char *ptr;
@@ -1158,7 +1175,7 @@ B *bcpy(P * from, P * to)
 		if (q->ofst != q->hdr->hole)
 			gstgap(q->hdr, q->ptr, q->ofst);
 		l->nlines = mcnt(q->ptr + q->hdr->ehole, '\n', l->hole = to->ofst - q->ofst);
-		mcpy(ptr, q->ptr + q->hdr->ehole, l->hole);
+		mmove(ptr, q->ptr + q->hdr->ehole, l->hole);
 		vchanged(ptr);
 		vunlock(ptr);
 		enqueb(H, link, &anchor, l);
@@ -1168,7 +1185,7 @@ B *bcpy(P * from, P * to)
 		if (q->ofst != q->hdr->hole)
 			gstgap(q->hdr, q->ptr, q->ofst);
 		l->nlines = mcnt(q->ptr + q->hdr->ehole, '\n', l->hole = SEGSIZ - q->hdr->ehole);
-		mcpy(ptr, q->ptr + q->hdr->ehole, l->hole);
+		mmove(ptr, q->ptr + q->hdr->ehole, l->hole);
 		vchanged(ptr);
 		vunlock(ptr);
 		enqueb(H, link, &anchor, l);
@@ -1177,8 +1194,8 @@ B *bcpy(P * from, P * to)
 			l = halloc();
 			ptr = vlock(vmem, l->seg);
 			l->nlines = q->hdr->nlines;
-			mcpy(ptr, q->ptr, q->hdr->hole);
-			mcpy(ptr + q->hdr->hole, q->ptr + q->hdr->ehole, SEGSIZ - q->hdr->ehole);
+			mmove(ptr, q->ptr, q->hdr->hole);
+			mmove(ptr + q->hdr->hole, q->ptr + q->hdr->ehole, SEGSIZ - q->hdr->ehole);
 			l->hole = GSIZE(q->hdr);
 			vchanged(ptr);
 			vunlock(ptr);
@@ -1191,7 +1208,7 @@ B *bcpy(P * from, P * to)
 			if (to->ofst != to->hdr->hole)
 				gstgap(to->hdr, to->ptr, to->ofst);
 			l->nlines = mcnt(to->ptr, '\n', to->ofst);
-			mcpy(ptr, to->ptr, l->hole = to->ofst);
+			mmove(ptr, to->ptr, l->hole = to->ofst);
 			vchanged(ptr);
 			vunlock(ptr);
 			enqueb(H, link, &anchor, l);
@@ -1206,8 +1223,7 @@ B *bcpy(P * from, P * to)
 }
 
 /* Coalesce small blocks into a single larger one */
-
-void pcoalesce(P * p)
+void pcoalesce(P *p)
 {
 	if (p->hdr != p->b->eof->hdr && GSIZE(p->hdr) + GSIZE(p->hdr->link.next) <= SEGSIZ - SEGSIZ / 4) {
 		H *hdr = p->hdr->link.next;
@@ -1264,7 +1280,6 @@ void pcoalesce(P * p)
  *  vunlock
  *  vchanged
  *  vupcount
- *  mcpy	- to copy deleted text
  *  mcnt	- to count NLs
  *  snip	- queue routines
  *  enqueb
@@ -1274,7 +1289,7 @@ void pcoalesce(P * p)
  */
 
 /* This is only to be used for bdel() */
-static B *bcut(P * from, P * to)
+static B *bcut(P *from, P *to)
 {
 	H *h,			/* The deleted text */
 	*i;
@@ -1286,7 +1301,7 @@ static B *bcut(P * from, P * to)
 	int bofmove = 0;	/* Set if bof got deleted */
 
 	if (!(amnt = to->byte - from->byte))
-		return 0;	/* ...nothing to delete */
+		return NULL;	/* ...nothing to delete */
 
 	nlines = to->line - from->line;
 
@@ -1298,7 +1313,7 @@ static B *bcut(P * from, P * to)
 		/* Store the deleted text */
 		h = halloc();
 		ptr = vlock(vmem, h->seg);
-		mcpy(ptr, from->ptr + from->hdr->ehole, (int) amnt);
+		mmove(ptr, from->ptr + from->hdr->ehole, (int) amnt);
 		h->hole = amnt;
 		h->nlines = nlines;
 		vchanged(ptr);
@@ -1322,7 +1337,7 @@ static B *bcut(P * from, P * to)
 			/* Save deleted text */
 			i = halloc();
 			ptr = vlock(vmem, i->seg);
-			mcpy(ptr, to->ptr, to->hdr->hole);
+			mmove(ptr, to->ptr, to->hdr->hole);
 			i->hole = to->hdr->hole;
 			i->nlines = mcnt(to->ptr, '\n', to->hdr->hole);
 			vchanged(ptr);
@@ -1336,8 +1351,9 @@ static B *bcut(P * from, P * to)
 
 		/* Delete end of from */
 		if (!from->ofst) {
-			/* ..unless from needs to be deleted too */
-			a = from->hdr->link.prev, h = 0;
+			/* ... unless from needs to be deleted too */
+			a = from->hdr->link.prev;
+			h = NULL;
 			if (a == from->b->eof->hdr)
 				bofmove = 1;
 		} else {
@@ -1349,7 +1365,7 @@ static B *bcut(P * from, P * to)
 			/* Save deleted text */
 			h = halloc();
 			ptr = vlock(vmem, h->seg);
-			mcpy(ptr, from->ptr + from->hdr->ehole, SEGSIZ - from->hdr->ehole);
+			mmove(ptr, from->ptr + from->hdr->ehole, SEGSIZ - from->hdr->ehole);
 			h->hole = SEGSIZ - from->hdr->ehole;
 			h->nlines = mcnt(ptr, '\n', h->hole);
 			vchanged(ptr);
@@ -1439,11 +1455,10 @@ static B *bcut(P * from, P * to)
 	pcoalesce(from);
 
 	/* Make buffer out of deleted text and return it */
-
 	return bmkchn(h, from->b, amnt, nlines);
 }
 
-void bdel(P * from, P * to)
+void bdel(P *from, P *to)
 {
 	if (to->byte - from->byte) {
 		B *b = bcut(from, to);
@@ -1460,8 +1475,7 @@ void bdel(P * from, P * to)
 /* p is placed in the new block such that it points to the same text but with
  * p->ofst==0
  */
-
-static void bsplit(P * p)
+static void bsplit(P *p)
 {
 	if (p->ofst) {
 		H *hdr;
@@ -1473,7 +1487,7 @@ static void bsplit(P * p)
 
 		if (p->ofst != p->hdr->hole)
 			gstgap(p->hdr, p->ptr, p->ofst);
-		mcpy(ptr, p->ptr + p->hdr->ehole, SEGSIZ - p->hdr->ehole);
+		mmove(ptr, p->ptr + p->hdr->ehole, SEGSIZ - p->hdr->ehole);
 		hdr->hole = SEGSIZ - p->hdr->ehole;
 		hdr->nlines = mcnt(ptr, '\n', hdr->hole);
 		p->hdr->nlines -= hdr->nlines;
@@ -1501,9 +1515,7 @@ static void bsplit(P * p)
 	}
 }
 
-/* Make a chain out of a block of memory */
-/* The block must not be empty */
-
+/* Make a chain out of a block of memory (the block must not be empty) */
 static H *bldchn(char *blk, int size, long *nlines)
 {
 	H anchor, *l;
@@ -1519,7 +1531,7 @@ static H *bldchn(char *blk, int size, long *nlines)
 			amnt = SEGSIZ;
 		else
 			amnt = size;
-		mcpy(ptr, blk, amnt);
+		mmove(ptr, blk, amnt);
 		l->hole = amnt;
 		l->ehole = SEGSIZ;
 		(*nlines) += (l->nlines = mcnt(ptr, '\n', amnt));
@@ -1528,17 +1540,14 @@ static H *bldchn(char *blk, int size, long *nlines)
 		enqueb(H, link, &anchor, l);
 		blk += amnt;
 		size -= amnt;
-	}
-	while (size);
+	} while (size);
 	l = anchor.link.next;
 	deque(H, link, &anchor);
 	return l;
 }
 
-/* Insert a chain into a buffer */
-/* This does not update pointers */
-
-static void inschn(P * p, H * a)
+/* Insert a chain into a buffer (this does not update pointers) */
+static void inschn(P *p, H *a)
 {
 	if (!p->b->eof->byte) {	/* P's buffer is empty: replace the empty segment in p with a */
 		hfree(p->hdr);
@@ -1574,7 +1583,7 @@ static void inschn(P * p, H * a)
 	}
 }
 
-static void fixupins(P * p, long amnt, long nlines, H * hdr, int hdramnt)
+static void fixupins(P *p, long amnt, long nlines, H *hdr, int hdramnt)
 {
 	P *pp;
 
@@ -1604,10 +1613,8 @@ static void fixupins(P * p, long amnt, long nlines, H * hdr, int hdramnt)
 	p->b->changed = 1;
 }
 
-/* Insert a buffer at pointer position */
-/* The buffer goes away */
-
-P *binsb(P * p, B * b)
+/* Insert a buffer at pointer position (the buffer goes away) */
+P *binsb(P *p, B *b)
 {
 	if (b->eof->byte) {
 		P *q = pdup(p);
@@ -1622,10 +1629,11 @@ P *binsb(P * p, B * b)
 	return p;
 }
 
-P *binsm(P * p, char *blk, int amnt)
+/* insert memory block 'blk' at 'p' */
+P *binsm(P *p, char *blk, int amnt)
 {
 	long nlines;
-	H *h = 0;
+	H *h = NULL;
 	int hdramnt;
 	P *q;
 
@@ -1652,6 +1660,7 @@ P *binsm(P * p, char *blk, int amnt)
 	return p;
 }
 
+/* insert char 'c' at 'p' */
 P *binsc(P *p, char c)
 {
 	if (p->b->o.crlf && c == '\n')
@@ -1660,6 +1669,7 @@ P *binsc(P *p, char c)
 		return binsm(p, &c, 1);
 }
 
+/* insert zero-terminated string 's' at 'p' */
 P *binss(P *p, char *s)
 {
 	return binsm(p, s, strlen(s));
@@ -1669,7 +1679,6 @@ P *binss(P *p, char *s)
  * when requested size has been read or when end of file condition occurs.
  * Returns with -2 in error for read error or 0 in error for success.
  */
-
 static int bkread(int fi, char *buff, int size)
 {
 	int a, b;
@@ -1678,7 +1687,7 @@ static int bkread(int fi, char *buff, int size)
 		error = 0;
 		return 0;
 	}
-	for (a = b = 0; (a < size) && ((b = jread(fi, buff + a, size - a)) > 0); a += b) ;
+	for (a = b = 0; (a < size) && ((b = joe_read(fi, buff + a, size - a)) > 0); a += b) ;
 	if (b < 0)
 		error = -2;
 	else
@@ -1688,7 +1697,6 @@ static int bkread(int fi, char *buff, int size)
 
 /* Read up to 'max' bytes from a file into a buffer */
 /* Returns with 0 in error or -2 in error for read error */
-
 B *bread(int fi, long int max)
 {
 	H anchor, *l;
@@ -1724,7 +1732,6 @@ B *bread(int fi, long int max)
  *
  * Returns new variable length string.
  */
-
 char *parsens(char *s, long int *skip, long int *amnt)
 {
 	char *n = vsncpy(NULL, 0, sz(s));
@@ -1797,7 +1804,6 @@ char *parsens(char *s, long int *skip, long int *amnt)
  * -3 for seek error
  * -4 for open error
  */
-
 B *bload(char *s)
 {
 	char buffer[SEGSIZ];
@@ -1877,7 +1883,7 @@ B *bload(char *s)
 	b->rdonly = b->o.readonly;
 
 	/* Close stream */
-      err:;
+err:
 #ifndef __MSDOS__
 	if (s[0] == '!')
 		pclose(fi);
@@ -1886,20 +1892,26 @@ B *bload(char *s)
 	if (strcmp(n, "-"))
 		fclose(fi);
 
-      opnerr:;
-	if (s[0] == '!')
-		ttopnn(), nreturn(maint->t);
+opnerr:
+	if (s[0] == '!') {
+		ttopnn();
+		nreturn(maint->t);
+	}
 
 	/* Set name */
 	b->name = joesep(strdup(s));
 
 	/* Set flags */
-	if (error || s[0] == '!' || skip || amnt != MAXLONG)
-		b->backup = 1, b->changed = 0;
-	else if (!strcmp(n, "-"))
-		b->backup = 1, b->changed = 1;
-	else
-		b->backup = 0, b->changed = 0;
+	if (error || s[0] == '!' || skip || amnt != MAXLONG) {
+		b->backup = 1;
+		b->changed = 0;
+	} else if (!strcmp(n, "-")) {
+		b->backup = 1;
+		b->changed = 1;
+	} else {
+		b->backup = 0;
+		b->changed = 0;
+	}
 	if (nowrite)
 		b->rdonly = b->o.readonly = 1;
 
@@ -1911,7 +1923,6 @@ B *bload(char *s)
 }
 
 /* Find already loaded buffer or load file into new buffer */
-
 B *bfind(char *s)
 {
 	B *b;
@@ -1952,7 +1963,6 @@ char **getbufs(void)
 }
 
 /* Find an orphaned buffer */
-
 B *borphan(void)
 {
 	B *b;
@@ -1962,7 +1972,7 @@ B *borphan(void)
 			b->orphan = 0;
 			return b;
 		}
-	return 0;
+	return NULL;
 }
 
 /* Write 'size' bytes from file beginning at 'p' to open file 'fd'.
@@ -1970,7 +1980,6 @@ B *borphan(void)
  * error is set to -5 for write error or 0 for success.
  * Don't attempt to write past the end of the file
  */
-
 int bsavefd(P *p, int fd, long int size)
 {
 	P *np = pdup(p);
@@ -1978,11 +1987,11 @@ int bsavefd(P *p, int fd, long int size)
 
 	while (size > (amnt = GSIZE(np->hdr) - np->ofst)) {
 		if (np->ofst < np->hdr->hole) {
-			if (jwrite(fd, np->ptr + np->ofst, np->hdr->hole - np->ofst) < 0)
+			if (joe_write(fd, np->ptr + np->ofst, np->hdr->hole - np->ofst) < 0)
 				goto err;
-			if (jwrite(fd, np->ptr + np->hdr->ehole, SEGSIZ - np->hdr->ehole) < 0)
+			if (joe_write(fd, np->ptr + np->hdr->ehole, SEGSIZ - np->hdr->ehole) < 0)
 				goto err;
-		} else if (jwrite(fd, np->ptr + np->ofst + GGAPSZ(np->hdr), amnt) < 0)
+		} else if (joe_write(fd, np->ptr + np->ofst + GGAPSZ(np->hdr), amnt) < 0)
 			goto err;
 		size -= amnt;
 		pnext(np);
@@ -1990,28 +1999,27 @@ int bsavefd(P *p, int fd, long int size)
 	if (size) {
 		if (np->ofst < np->hdr->hole) {
 			if (size > np->hdr->hole - np->ofst) {
-				if (jwrite(fd, np->ptr + np->ofst, np->hdr->hole - np->ofst) < 0)
+				if (joe_write(fd, np->ptr + np->ofst, np->hdr->hole - np->ofst) < 0)
 					goto err;
-				if (jwrite(fd, np->ptr + np->hdr->ehole, (int) size - np->hdr->hole + np->ofst) < 0)
+				if (joe_write(fd, np->ptr + np->hdr->ehole, (int) size - np->hdr->hole + np->ofst) < 0)
 					goto err;
 			} else {
-				if (jwrite(fd, np->ptr + np->ofst, (int) size) < 0)
+				if (joe_write(fd, np->ptr + np->ofst, (int) size) < 0)
 					goto err;
 			}
 		} else {
-			if (jwrite(fd, np->ptr + np->ofst + GGAPSZ(np->hdr), (int) size) < 0)
+			if (joe_write(fd, np->ptr + np->ofst + GGAPSZ(np->hdr), (int) size) < 0)
 				goto err;
 		}
 	}
 	prm(np);
 	return error = 0;
-      err:;
+err:
 	prm(np);
 	return error = 5;
 }
 
 /* Save 'size' bytes beginning at 'p' in file 's' */
-
 int bsave(P *p, char *s, long int size)
 {
 	FILE *f;
@@ -2059,12 +2067,12 @@ int bsave(P *p, char *s, long int size)
 		char nl = '\n';
 
 		pfwrd(q, size - 1);
-		if (brc(q) != '\n' && jwrite(fileno(f), &nl, 1) < 0)
+		if (brc(q) != '\n' && joe_write(fileno(f), &nl, 1) < 0)
 			error = -5;
 		prm(q);
 	}
 
-      err:;
+err:
 #ifndef __MSDOS__
 	if (s[0] == '!')
 		pclose(f);
@@ -2075,20 +2083,19 @@ int bsave(P *p, char *s, long int size)
 	else
 		fflush(f);
 
-      opnerr:;
-	if (s[0] == '!' || !strcmp(s, "-"))
-		ttopnn(), nreturn(maint->t);
+opnerr:
+	if (s[0] == '!' || !strcmp(s, "-")) {
+		ttopnn();
+		nreturn(maint->t);
+	}
 	return error;
 }
 
 int brc(P *p)
 {
 	if (p->ofst == GSIZE(p->hdr))
-		return MAXINT;
-	if (p->ofst >= p->hdr->hole)
-		return p->ptr[p->ofst + p->hdr->ehole - p->hdr->hole];
-	else
-		return p->ptr[p->ofst];
+		return NO_MORE_DATA;
+	return GCHAR(p);
 }
 
 char *brmem(P *p, char *blk, int size)
@@ -2112,7 +2119,7 @@ char *brmem(P *p, char *blk, int size)
 
 char *brs(P *p, int size)
 {
-	char *s = (char *) malloc(size + 1);
+	char *s = (char *) joe_malloc(size + 1);
 
 	s[size] = 0;
 	return brmem(p, s, size);
@@ -2127,11 +2134,9 @@ char *brvs(P *p, int size)
 
 /* Save edit buffers when editor dies */
 
-extern char *ctime(const time_t *);
-
 RETSIGTYPE ttsig(int sig)
 {
-	long tim = time(0);
+	time_t tim = time(NULL);
 	B *b;
 	FILE *f;
 	int tmpfd;
