@@ -1,13 +1,14 @@
 /*
-	Fast block move/copy subroutines
-	Copyright (C) 1992 Joseph H. Allen
-
-	This file is part of JOE (Joe's Own Editor)
-*/
+ *	Fast block move/copy subroutines
+ *	Copyright
+ *		(C) 1992 Joseph H. Allen
+ *
+ *	This file is part of JOE (Joe's Own Editor)
+ */
+#include "config.h"
 
 /* This module requires ALIGNED and SIZEOF_INT to be defined correctly */
 
-#include "config.h"
 #include "blocks.h"
 
 #define BITS 8
@@ -49,6 +50,36 @@ int *msetI(void *dest, int c, int sz)
 		sz -= 16;
 	}
 	switch (sz) {
+	case 15:	d[14] = c;
+	case 14:	d[13] = c;
+	case 13:	d[12] = c;
+	case 12:	d[11] = c;
+	case 11:	d[10] = c;
+	case 10:	d[9] = c;
+	case 9:		d[8] = c;
+	case 8:		d[7] = c;
+	case 7:		d[6] = c;
+	case 6:		d[5] = c;
+	case 5:		d[4] = c;
+	case 4:		d[3] = c;
+	case 3:		d[2] = c;
+	case 2:		d[1] = c;
+	case 1:		d[0] = c;
+	case 0:		/* do nothing */;
+	}
+	return orgd;
+}
+
+/* Set 'sz' 'char's beginning at 'd' to the value 'c' */
+/* Returns address of block.  Does nothing if 'sz' equals zero */
+
+char *mset(void *dest, unsigned char c, int sz)
+{
+	char	*d = dest;
+	char	*orgd = dest;
+
+	if (sz < 16) {
+		switch (sz) {
 		case 15:	d[14] = c;
 		case 14:	d[13] = c;
 		case 13:	d[12] = c;
@@ -64,29 +95,13 @@ int *msetI(void *dest, int c, int sz)
 		case 3:		d[2] = c;
 		case 2:		d[1] = c;
 		case 1:		d[0] = c;
-		case 0: /* do nothing */;
-	}
-	return orgd;
-}
+		case 0:		/* do nothing */;
+		}
+	} else {
+		unsigned z = SIZEOF_INT - ((unsigned long)d & (SIZEOF_INT - 1));
 
-/* Set 'sz' 'char's beginning at 'd' to the value 'c' */
-/* Returns address of block.  Does nothing if 'sz' equals zero */
-
-char *mset(void *dest, unsigned char c, int sz)
-{
-	char	*d = dest;
-	char	*orgd = dest;
-
-	if (sz < 16)
-		switch (sz) {
-			case 15:	d[14] = c;
-			case 14:	d[13] = c;
-			case 13:	d[12] = c;
-			case 12:	d[11] = c;
-			case 11:	d[10] = c;
-			case 10:	d[9] = c;
-			case 9:		d[8] = c;
-			case 8:		d[7] = c;
+		if (z != SIZEOF_INT) {
+			switch (z) {
 			case 7:		d[6] = c;
 			case 6:		d[5] = c;
 			case 5:		d[4] = c;
@@ -95,22 +110,10 @@ char *mset(void *dest, unsigned char c, int sz)
 			case 2:		d[1] = c;
 			case 1:		d[0] = c;
 			case 0:		/* do nothing */;
-	} else {
-		int z = SIZEOF_INT - ((int) d & (SIZEOF_INT - 1));
-
-		if (z != SIZEOF_INT)
-			switch (z) {
-				case 7:		d[6] = c;
-				case 6:		d[5] = c;
-				case 5:		d[4] = c;
-				case 4:		d[3] = c;
-				case 3:		d[2] = c;
-				case 2:		d[1] = c;
-				case 1:		d[0] = c;
-				case 0:	/* do nothing */;
-				d += z;
-				sz -= z;
 			}
+			d += z;
+			sz -= z;
+		}
 		msetI(d,
 #if SIZEOF_INT >= 8
 		      (c << (BITS * 7)) + (c << (BITS * 6)) + (c << (BITS * 5)) + (c << (BITS * 4)) +
@@ -124,14 +127,14 @@ char *mset(void *dest, unsigned char c, int sz)
 		      c, sz >> SHFT);
 		d += sz & ~(SIZEOF_INT - 1);
 		switch (sz & (SIZEOF_INT - 1)) {
-			case 7:		d[6] = c;
-			case 6:		d[5] = c;
-			case 5:		d[4] = c;
-			case 4:		d[3] = c;
-			case 3:		d[2] = c;
-			case 2:		d[1] = c;
-			case 1:		d[0] = c;
-			case 0:	/* do nothing */;
+		case 7:		d[6] = c;
+		case 6:		d[5] = c;
+		case 5:		d[4] = c;
+		case 4:		d[3] = c;
+		case 3:		d[2] = c;
+		case 2:		d[1] = c;
+		case 1:		d[0] = c;
+		case 0:		/* do nothing */;
 		}
 	}
 	return orgd;
@@ -173,22 +176,22 @@ static int *mbkwdI(void *dest, void *src, int sz)
 	d -= sz;
 	s -= sz;
 	switch (sz) {
-		case 15:	d[14] = s[14];
-		case 14:	d[13] = s[13];
-		case 13:	d[12] = s[12];
-		case 12:	d[11] = s[11];
-		case 11:	d[10] = s[10];
-		case 10:	d[9] = s[9];
-		case 9:		d[8] = s[8];
-		case 8:		d[7] = s[7];
-		case 7:		d[6] = s[6];
-		case 6:		d[5] = s[5];
-		case 5:		d[4] = s[4];
-		case 4:		d[3] = s[3];
-		case 3:		d[2] = s[2];
-		case 2:		d[1] = s[1];
-		case 1:		d[0] = s[0];
-		case 0:		/* do nothing */;
+	case 15:	d[14] = s[14];
+	case 14:	d[13] = s[13];
+	case 13:	d[12] = s[12];
+	case 12:	d[11] = s[11];
+	case 11:	d[10] = s[10];
+	case 10:	d[9] = s[9];
+	case 9:		d[8] = s[8];
+	case 8:		d[7] = s[7];
+	case 7:		d[6] = s[6];
+	case 6:		d[5] = s[5];
+	case 5:		d[4] = s[4];
+	case 4:		d[3] = s[3];
+	case 3:		d[2] = s[2];
+	case 2:		d[1] = s[1];
+	case 1:		d[0] = s[0];
+	case 0:		/* do nothing */;
 	}
 	return d;
 }
@@ -227,22 +230,22 @@ static int *mfwrdI(void *dest, void *src, int sz)
 	s -= 15 - sz;
 	d -= 15 - sz;
 	switch (sz) {
-		case 15:	d[0] = s[0];
-		case 14:	d[1] = s[1];
-		case 13:	d[2] = s[2];
-		case 12:	d[3] = s[3];
-		case 11:	d[4] = s[4];
-		case 10:	d[5] = s[5];
-		case 9:		d[6] = s[6];
-		case 8:		d[7] = s[7];
-		case 7:		d[8] = s[8];
-		case 6:		d[9] = s[9];
-		case 5:		d[10] = s[10];
-		case 4:		d[11] = s[11];
-		case 3:		d[12] = s[12];
-		case 2:		d[13] = s[13];
-		case 1:		d[14] = s[14];
-		case 0:		/* do nothing */;
+	case 15:	d[0] = s[0];
+	case 14:	d[1] = s[1];
+	case 13:	d[2] = s[2];
+	case 12:	d[3] = s[3];
+	case 11:	d[4] = s[4];
+	case 10:	d[5] = s[5];
+	case 9:		d[6] = s[6];
+	case 8:		d[7] = s[7];
+	case 7:		d[8] = s[8];
+	case 6:		d[9] = s[9];
+	case 5:		d[10] = s[10];
+	case 4:		d[11] = s[11];
+	case 3:		d[12] = s[12];
+	case 2:		d[13] = s[13];
+	case 1:		d[14] = s[14];
+	case 0:		/* do nothing */;
 	}
 	return od;
 }
@@ -252,7 +255,7 @@ static int *mfwrdI(void *dest, void *src, int sz)
  * are copied before the ones at the lowest ('s') are.
  */
 
-char *mbkwd(register char *d, register char *s, register int sz)
+static char *mbkwd(register char *d, register char *s, register int sz)
 {
 	if (s == d)
 		return d;
@@ -261,36 +264,36 @@ char *mbkwd(register char *d, register char *s, register int sz)
 #ifdef ALIGNED
 	if (sz >= 16)
 #else
-	if (((int) s & (SIZEOF_INT - 1)) == ((int) d & (SIZEOF_INT - 1)) && sz >= 16)
+	if (((unsigned long)s & (SIZEOF_INT - 1)) == ((unsigned long)d & (SIZEOF_INT - 1)) && sz >= 16)
 #endif
 	{
-		int z = ((int) s & (SIZEOF_INT - 1));
+		unsigned z = ((unsigned long) s & (SIZEOF_INT - 1));
 
 		s -= z;
 		d -= z;
 		switch (z) {
-			case 7:		d[6] = s[6];
-			case 6:		d[5] = s[5];
-			case 5:		d[4] = s[4];
-			case 4:		d[3] = s[3];
-			case 3:		d[2] = s[2];
-			case 2:		d[1] = s[1];
-			case 1:		d[0] = s[0];
-			case 0:		/* do nothing */;
+		case 7:		d[6] = s[6];
+		case 6:		d[5] = s[5];
+		case 5:		d[4] = s[4];
+		case 4:		d[3] = s[3];
+		case 3:		d[2] = s[2];
+		case 2:		d[1] = s[1];
+		case 1:		d[0] = s[0];
+		case 0:		/* do nothing */;
 		}
 		sz -= z;
 		mbkwdI(d - (sz & ~(SIZEOF_INT - 1)), s - (sz & ~(SIZEOF_INT - 1)), sz >> SHFT);
 		d -= sz;
 		s -= sz;
 		switch (sz & (SIZEOF_INT - 1)) {
-			case 7:		d[6] = s[6];
-			case 6:		d[5] = s[5];
-			case 5:		d[4] = s[4];
-			case 4:		d[3] = s[3];
-			case 3:		d[2] = s[2];
-			case 2:		d[1] = s[1];
-			case 1:		d[0] = s[0];
-			case 0:		/* do nothing */;
+		case 7:		d[6] = s[6];
+		case 6:		d[5] = s[5];
+		case 5:		d[4] = s[4];
+		case 4:		d[3] = s[3];
+		case 3:		d[2] = s[2];
+		case 2:		d[1] = s[1];
+		case 1:		d[0] = s[0];
+		case 0:		/* do nothing */;
 		}
 	} else {
 		while (sz >= 16) {
@@ -317,22 +320,22 @@ char *mbkwd(register char *d, register char *s, register int sz)
 		d -= sz;
 		s -= sz;
 		switch (sz) {
-			case 15:	d[14] = s[14];
-			case 14:	d[13] = s[13];
-			case 13:	d[12] = s[12];
-			case 12:	d[11] = s[11];
-			case 11:	d[10] = s[10];
-			case 10:	d[9] = s[9];
-			case 9:		d[8] = s[8];
-			case 8:		d[7] = s[7];
-			case 7:		d[6] = s[6];
-			case 6:		d[5] = s[5];
-			case 5:		d[4] = s[4];
-			case 4:		d[3] = s[3];
-			case 3:		d[2] = s[2];
-			case 2:		d[1] = s[1];
-			case 1:		d[0] = s[0];
-			case 0:		/* do nothing */;
+		case 15:	d[14] = s[14];
+		case 14:	d[13] = s[13];
+		case 13:	d[12] = s[12];
+		case 12:	d[11] = s[11];
+		case 11:	d[10] = s[10];
+		case 10:	d[9] = s[9];
+		case 9:		d[8] = s[8];
+		case 8:		d[7] = s[7];
+		case 7:		d[6] = s[6];
+		case 6:		d[5] = s[5];
+		case 5:		d[4] = s[4];
+		case 4:		d[3] = s[3];
+		case 3:		d[2] = s[2];
+		case 2:		d[1] = s[1];
+		case 1:		d[0] = s[0];
+		case 0:		/* do nothing */;
 		}
 	}
 	return d;
@@ -343,7 +346,7 @@ char *mbkwd(register char *d, register char *s, register int sz)
  * are copied before the ones at the highest ('s'+'sz'-1) are.
  */
 
-char *mfwrd(register char *d, register char *s, register int sz)
+static char *mfwrd(register char *d, register char *s, register int sz)
 {
 	char *od = d;
 
@@ -352,34 +355,34 @@ char *mfwrd(register char *d, register char *s, register int sz)
 #ifdef ALIGNED
 	if (sz >= 16)
 #else
-	if (((int) d & (SIZEOF_INT - 1)) == ((int) s & (SIZEOF_INT - 1)) && sz >= 16)
+	if (((unsigned long)d & (SIZEOF_INT - 1)) == ((unsigned long)s & (SIZEOF_INT - 1)) && sz >= 16)
 #endif
 	{
-		int z = ((int) s & (SIZEOF_INT - 1));
+		unsigned z = ((unsigned long)s & (SIZEOF_INT - 1));
 
 		if (z) {
 			s -= z;
 			d -= z;
 			switch (SIZEOF_INT - z) {
 #if SIZEOF_INT == 8
-				case 7:		d[1] = s[1];
-				case 6:		d[2] = s[2];
-				case 5:		d[3] = s[3];
-				case 4:		d[4] = s[4];
-				case 3:		d[5] = s[5];
-				case 2:		d[6] = s[6];
-				case 1:		d[7] = s[7];
-				case 0:		/* do nothing */;
+			case 7:		d[1] = s[1];
+			case 6:		d[2] = s[2];
+			case 5:		d[3] = s[3];
+			case 4:		d[4] = s[4];
+			case 3:		d[5] = s[5];
+			case 2:		d[6] = s[6];
+			case 1:		d[7] = s[7];
+			case 0:		/* do nothing */;
 #else
 #if SIZEOF_INT == 4
-				case 3:		d[1] = s[1];
-				case 2:		d[2] = s[2];
-				case 1:		d[3] = s[3];
-				case 0:		/* do nothing */;
+			case 3:		d[1] = s[1];
+			case 2:		d[2] = s[2];
+			case 1:		d[3] = s[3];
+			case 0:		/* do nothing */;
 #else
 #if SIZEOF_INT == 2
-				case 1:		d[1] = s[1];
-				case 0:		/* do nothing */;
+			case 1:		d[1] = s[1];
+			case 0:		/* do nothing */;
 #endif
 #endif
 #endif
@@ -393,24 +396,24 @@ char *mfwrd(register char *d, register char *s, register int sz)
 		d += sz - (SIZEOF_INT - 1);
 		switch (sz & (SIZEOF_INT - 1)) {
 #if SIZEOF_INT == 8
-			case 7:		d[0] = s[0];
-			case 6:		d[1] = s[1];
-			case 5:		d[2] = s[2];
-			case 4:		d[3] = s[3];
-			case 3:		d[4] = s[4];
-			case 2:		d[5] = s[5];
-			case 1:		d[6] = s[6];
-			case 0:		/* do nothing */;
+		case 7:		d[0] = s[0];
+		case 6:		d[1] = s[1];
+		case 5:		d[2] = s[2];
+		case 4:		d[3] = s[3];
+		case 3:		d[4] = s[4];
+		case 2:		d[5] = s[5];
+		case 1:		d[6] = s[6];
+		case 0:		/* do nothing */;
 #else
 #if SIZEOF_INT == 4
-			case 3:		d[0] = s[0];
-			case 2:		d[1] = s[1];
-			case 1:		d[2] = s[2];
-			case 0:		/* do nothing */;
+		case 3:		d[0] = s[0];
+		case 2:		d[1] = s[1];
+		case 1:		d[2] = s[2];
+		case 0:		/* do nothing */;
 #else
 #if SIZEOF_INT == 2
-			case 1:		d[0] = s[0];
-			case 0:		/* do nothing */;
+		case 1:		d[0] = s[0];
+		case 0:		/* do nothing */;
 #endif
 #endif
 #endif
@@ -440,22 +443,22 @@ char *mfwrd(register char *d, register char *s, register int sz)
 		s -= 15 - sz;
 		d -= 15 - sz;
 		switch (sz) {
-			case 15:	d[0] = s[0];
-			case 14:	d[1] = s[1];
-			case 13:	d[2] = s[2];
-			case 12:	d[3] = s[3];
-			case 11:	d[4] = s[4];
-			case 10:	d[5] = s[5];
-			case 9:		d[6] = s[6];
-			case 8:		d[7] = s[7];
-			case 7:		d[8] = s[8];
-			case 6:		d[9] = s[9];
-			case 5:		d[10] = s[10];
-			case 4:		d[11] = s[11];
-			case 3:		d[12] = s[12];
-			case 2:		d[13] = s[13];
-			case 1:		d[14] = s[14];
-			case 0:		/* do nothing */;
+		case 15:	d[0] = s[0];
+		case 14:	d[1] = s[1];
+		case 13:	d[2] = s[2];
+		case 12:	d[3] = s[3];
+		case 11:	d[4] = s[4];
+		case 10:	d[5] = s[5];
+		case 9:		d[6] = s[6];
+		case 8:		d[7] = s[7];
+		case 7:		d[8] = s[8];
+		case 6:		d[9] = s[9];
+		case 5:		d[10] = s[10];
+		case 4:		d[11] = s[11];
+		case 3:		d[12] = s[12];
+		case 2:		d[13] = s[13];
+		case 1:		d[14] = s[14];
+		case 0:		/* do nothing */;
 		}
 	}
 	return od;
@@ -497,22 +500,22 @@ int mcnt(register char *blk, register char c, int size)
 		size -= 16;
 	}
 	switch (size) {
-		case 15:	if (blk[14] == c) ++nlines;
-		case 14:	if (blk[13] == c) ++nlines;
-		case 13:	if (blk[12] == c) ++nlines;
-		case 12:	if (blk[11] == c) ++nlines;
-		case 11:	if (blk[10] == c) ++nlines;
-		case 10:	if (blk[9] == c) ++nlines;
-		case 9:		if (blk[8] == c) ++nlines;
-		case 8:		if (blk[7] == c) ++nlines;
-		case 7:		if (blk[6] == c) ++nlines;
-		case 6:		if (blk[5] == c) ++nlines;
-		case 5:		if (blk[4] == c) ++nlines;
-		case 4:		if (blk[3] == c) ++nlines;
-		case 3:		if (blk[2] == c) ++nlines;
-		case 2:		if (blk[1] == c) ++nlines;
-		case 1:		if (blk[0] == c) ++nlines;
-		case 0:		/* do nothing */;
+	case 15:	if (blk[14] == c) ++nlines;
+	case 14:	if (blk[13] == c) ++nlines;
+	case 13:	if (blk[12] == c) ++nlines;
+	case 12:	if (blk[11] == c) ++nlines;
+	case 11:	if (blk[10] == c) ++nlines;
+	case 10:	if (blk[9] == c) ++nlines;
+	case 9:		if (blk[8] == c) ++nlines;
+	case 8:		if (blk[7] == c) ++nlines;
+	case 7:		if (blk[6] == c) ++nlines;
+	case 6:		if (blk[5] == c) ++nlines;
+	case 5:		if (blk[4] == c) ++nlines;
+	case 4:		if (blk[3] == c) ++nlines;
+	case 3:		if (blk[2] == c) ++nlines;
+	case 2:		if (blk[1] == c) ++nlines;
+	case 1:		if (blk[0] == c) ++nlines;
+	case 0:		/* do nothing */;
 	}
 	return nlines;
 }
