@@ -1,27 +1,28 @@
 /*
-    Edit buffer window generation
-    Copyright (C) 1992 Joseph H. Allen
-
-    This file is part of JOE (Joe's Own Editor)
-*/
-
+ *	Edit buffer window generation
+ *	Copyright
+ *		(C) 1992 Joseph H. Allen
+ *
+ *	This file is part of JOE (Joe's Own Editor)
+ */
 #include "config.h"
+#include "types.h"
+
 #include <string.h>
 #include <stdio.h>
 #ifdef HAVE_STDLIB_H
 #include <stdlib.h>
 #endif
-#include "tty.h"
-#include "vfile.h"
-#include "termcap.h"
-#include "kbd.h"
+
+
 #include "b.h"
+#include "blocks.h"
+#include "kbd.h"
+#include "rc.h"
 #include "scrn.h"
-#include "w.h"
 #include "ublock.h"
 #include "utils.h"
-#include "blocks.h"
-#include "bw.h"
+#include "w.h"
 
 /* Display modes */
 int dspasis = 0;
@@ -31,15 +32,22 @@ extern int staen;
 
 static P *getto(P *p, P *cur, P *top, long int line)
 {
-	long dist = MAXLONG;
-	long d;
-	P *best;
 
-	if (!p) {
-		if (d = (line - cur->line >= 0 ? line - cur->line : cur->line - line), d < dist)
-			dist = d, best = cur;
-		if (d = (line - top->line >= 0 ? line - top->line : top->line - line), d < dist)
-			dist = d, best = top;
+	if (p == NULL) {
+		P *best = cur;
+		long dist = MAXLONG;
+		long d;
+
+		d = (line >= cur->line ? line - cur->line : cur->line - line);
+		if (d < dist) {
+			dist = d;
+			best = cur;
+		}
+		d = (line >= top->line ? line - top->line : top->line - line);
+		if (d < dist) {
+			dist = d;
+			best = top;
+		}
 		p = pdup(best);
 		p_goto_bol(p);
 	}
@@ -249,8 +257,7 @@ static int lgen(SCRN *t, int y, int *screen, int x, int w, P *p, long int scr, l
 				--amnt;
 				goto loop;
 			}
-		}
-		while (--amnt);
+		} while (--amnt);
 	if (bp == p->ptr + SEGSIZ) {
 		if (pnext(p)) {
 			bp = p->ptr;
@@ -321,12 +328,11 @@ static int lgen(SCRN *t, int y, int *screen, int x, int w, P *p, long int scr, l
 						goto bye;
 					if (++x == w)
 						goto eosl;
-				}
-				while (--ta);
+				} while (--ta);
 			} else if (bc == '\n')
 				goto eobl;
 			else {
-				xlat(c, bc);
+				xlat(&c, &bc);
 				c ^= c1;
 				outatr(t, screen + x, x, y, bc, c);
 				if (ifhave)
@@ -334,8 +340,7 @@ static int lgen(SCRN *t, int y, int *screen, int x, int w, P *p, long int scr, l
 				if (++x == w)
 					goto eosl;
 			}
-		}
-		while (--amnt);
+		} while (--amnt);
 	if (bp == p->ptr + SEGSIZ) {
 		if (pnext(p)) {
 			bp = p->ptr;
@@ -444,8 +449,7 @@ static int lgena(SCRN *t, int y, int *screen, int x, int w, P *p, long int scr,
 				--amnt;
 				goto loop;
 			}
-		}
-		while (--amnt);
+		} while (--amnt);
 	if (bp == p->ptr + SEGSIZ) {
 		if (pnext(p)) {
 			bp = p->ptr;
@@ -487,19 +491,17 @@ static int lgena(SCRN *t, int y, int *screen, int x, int w, P *p, long int scr,
 					screen[x] = ' ' + c1;
 					if (++x == w)
 						goto eosl;
-				}
-				while (--ta);
+				} while (--ta);
 			} else if (bc == '\n')
 				goto eobl;
 			else {
-				xlat(c, bc);
+				xlat(&c, &bc);
 				c ^= c1;
 				screen[x] = c + bc;
 				if (++x == w)
 					goto eosl;
 			}
-		}
-		while (--amnt);
+		} while (--amnt);
 	if (bp == p->ptr + SEGSIZ) {
 		if (pnext(p)) {
 			bp = p->ptr;
@@ -558,7 +560,7 @@ static void gennum(BW *w, int *screen, SCRN *t, int y, int *comp)
 void bwgen(BW *w, int linums)
 {
 	int *screen;
-	P *p = 0;
+	P *p = NULL;
 	P *q = pdup(w->cursor);
 	int bot = w->h + w->y;
 	int y;
@@ -571,19 +573,26 @@ void bwgen(BW *w, int linums)
 
 	if (markv(0) && markk->b == w->b)
 		if (square) {
-			from = markb->xcol, to = markk->xcol, dosquare = 1;
+			from = markb->xcol;
+			to = markk->xcol;
+			dosquare = 1;
 			fromline = markb->line;
 			toline = markk->line;
-		} else
-			from = markb->byte, to = markk->byte;
+		} else {
+			from = markb->byte;
+			to = markk->byte;
+		}
 	else if (marking && markb && markb->b == w->b && w->cursor->byte != markb->byte && !from) {
 		if (square) {
-			from = long_min(w->cursor->xcol, markb->xcol), to = long_max(w->cursor->xcol, markb->xcol);
+			from = long_min(w->cursor->xcol, markb->xcol);
+			to = long_max(w->cursor->xcol, markb->xcol);
 			fromline = long_min(w->cursor->line, markb->line);
 			toline = long_max(w->cursor->line, markb->line);
 			dosquare = 1;
-		} else
-			from = long_min(w->cursor->byte, markb->byte), to = long_max(w->cursor->byte, markb->byte);
+		} else {
+			from = long_min(w->cursor->byte, markb->byte);
+			to = long_max(w->cursor->byte, markb->byte);
+		}
 	}
 
 	if (marking)
@@ -667,19 +676,26 @@ void bwresz(BW *w, int wi, int he)
 
 BW *bwmk(W *window, B *b, int prompt)
 {
-	BW *w = (BW *) malloc(sizeof(BW));
+	BW *w = (BW *) joe_malloc(sizeof(BW));
 
 	w->parent = window;
 	w->pid = 0;
 	w->out = -1;
 	w->b = b;
-	if (prompt || (!window->y && staen))
-		w->y = window->y, w->h = window->h;
-	else
-		w->y = window->y + 1, w->h = window->h - 1;
+	if (prompt || (!window->y && staen)) {
+		w->y = window->y;
+		w->h = window->h;
+	} else {
+		w->y = window->y + 1;
+		w->h = window->h - 1;
+	}
 	if (b->oldcur) {
-		w->top = b->oldtop, b->oldtop = 0, w->top->owner = 0;
-		w->cursor = b->oldcur, b->oldcur = 0, w->cursor->owner = 0;
+		w->top = b->oldtop;
+		b->oldtop = NULL;
+		w->top->owner = NULL;
+		w->cursor = b->oldcur;
+		b->oldcur = NULL;
+		w->cursor->owner = NULL;
 	} else {
 		w->top = pdup(b->bof);
 		w->cursor = pdup(b->bof);
@@ -688,13 +704,16 @@ BW *bwmk(W *window, B *b, int prompt)
 	w->object = NULL;
 	w->offset = 0;
 	w->o = w->b->o;
-	if (w->o.linums)
-		w->x = window->x + LINCOLS, w->w = window->w - LINCOLS;
-	else
-		w->x = window->x, w->w = window->w;
+	if (w->o.linums) {
+		w->x = window->x + LINCOLS;
+		w->w = window->w - LINCOLS;
+	} else {
+		w->x = window->x;
+		w->w = window->w;
+	}
 	if (window == window->main) {
 		rmkbd(window->kbd);
-		window->kbd = mkkbd(getcontext(w->o.context));
+		window->kbd = mkkbd(kmap_getcontext(w->o.context));
 	}
 	w->top->xcol = 0;
 	w->cursor->xcol = 0;
@@ -706,19 +725,19 @@ void bwrm(BW *w)
 	prm(w->top);
 	prm(w->cursor);
 	brm(w->b);
-	free(w);
+	joe_free(w);
 }
 
 int ustat(BW *bw)
 {
 	static char buf[80];
-	unsigned c = brc(bw->cursor);
+	int c = brc(bw->cursor);
 
-	if (c == MAXINT)
+	if (c == NO_MORE_DATA)
 		snprintf(buf, sizeof(buf), "** Line %ld  Col %ld  Offset %ld(0x%lx) **", bw->cursor->line + 1, piscol(bw->cursor) + 1, bw->cursor->byte, bw->cursor->byte);
 	else
 		snprintf(buf, sizeof(buf), "** Line %ld  Col %ld  Offset %ld(0x%lx)  Ascii %d(0%o/0x%X) **", bw->cursor->line + 1, piscol(bw->cursor) + 1, bw->cursor->byte, bw->cursor->byte, 255 & c, 255 & c, 255 & c);
-	msgnw(bw, buf);
+	msgnw(bw->parent, buf);
 	return 0;
 }
 
@@ -738,8 +757,10 @@ int ucrawll(BW *bw)
 	int amnt = bw->w / 2;
 	int curamnt = bw->w / 2;
 
-	if (amnt > bw->offset)
-		amnt = bw->offset, curamnt = bw->offset;
+	if (amnt > bw->offset) {
+		amnt = bw->offset;
+		curamnt = bw->offset;
+	}
 	if (!bw->offset)
 		curamnt = bw->cursor->xcol;
 	if (!curamnt)
