@@ -1,11 +1,17 @@
-#ifndef _JOEconfig
-#define _JOEconfig 1
+#ifndef _JOE_CONFIG_H
+#define _JOE_CONFIG_H
 
 #include "autoconf.h"
 
 #define MAXINT  ((((unsigned int)-1)/2)-1)
 #define MAXLONG ((((unsigned long)-1L)/2)-1)
 
+#include <stdio.h>
+#ifndef EOF
+#define EOF -1
+#endif
+#define NO_MORE_DATA EOF
+
 #if defined __MSDOS__ && SIZEOF_INT == 2 /* real mode ms-dos compilers */
 #if SIZEOF_VOID_P == 4 /* real mode ms-dos compilers with 'far' memory model or something like that */
 #define physical(a)  (((unsigned long)(a)&0xFFFF)+(((unsigned long)(a)&0xFFFF0000)>>12))
@@ -37,4 +43,4 @@
 
 #endif /* real mode ms-dos */
 
-#endif /* ifndef _JOEconfig */
+#endif /* ifndef _JOE_CONFIG_H */
