@@ -1,12 +1,13 @@
-
 /* 
-	Directory and path functions
-	Copyright (C) 1992 Joseph H. Allen
-
-	This file is part of JOE (Joe's Own Editor)
-*/
-
+ *	Directory and path functions
+ *	Copyright
+ *		(C) 1992 Joseph H. Allen
+ *
+ *	This file is part of JOE (Joe's Own Editor)
+ */
 #include "config.h"
+#include "types.h"
+
 #include <stdio.h>
 #include <sys/types.h>
 #ifdef HAVE_SYS_STAT_H
@@ -19,14 +20,16 @@
 #ifdef HAVE_PATHS_H
 #  include <paths.h>	/* for _PATH_TMP */
 #endif
+#ifdef HAVE_LIMITS_H
+#include <limits.h>
+#endif
 #ifdef HAVE_STDLIB_H
 #include <stdlib.h>
 #endif
 
+#include "path.h"
 #include "vs.h"
 #include "va.h"
-#include "tty.h"
-#include "path.h"
 
 #ifdef HAVE_DIRENT_H
 #  include <dirent.h>
@@ -75,6 +78,11 @@
 #  endif
 #endif
 
+#ifndef PATH_MAX
+#warning What should we include to have PATH_MAX defined?
+#define PATH_MAX	4096
+#endif
+
 /********************************************************************/
 char *joesep(char *path)
 {
@@ -208,7 +216,7 @@ char *mktmp(char *where)
 #ifdef HAVE_MKSTEMP
 	snprintf(name, namesize, "%s/joe.tmp.XXXXXX", where);
 	if((fd = mkstemp(name)) == -1)
-		return 0;       /* FIXME: vflsh() and vflshf() */
+		return NULL;	/* FIXME: vflsh() and vflshf() */
 				/* expect mktmp() always succeed!!! */
 
 	fchmod(fd, 0600);       /* Linux glibc 2.0 mkstemp() creates it with */
@@ -224,8 +232,8 @@ char *mktmp(char *where)
 		close(fd);
 		goto loop;	/* FIXME: possible endless loop --> DoS attack */
 	}
-	if ((fd = creat(name, 0600)) == -1)
-		return 0;	/* FIXME: see above */
+	if ((fd = open(name, O_RDWR | O_CREAT | O_EXCL, 0600)) == -1)
+		return NULL;	/* FIXME: see above */
 	else
 		close(fd);
 #endif
@@ -238,19 +246,20 @@ int rmatch(char *a, char *b)
 
 	for (;;)
 		switch (*a) {
-			case '*':
+		case '*':
 			++a;
-			do
+			do {
 				if (rmatch(a, b))
 					return 1;
-			while (*b++) ;
+			} while (*b++);
 			return 0;
-			case '[':
+		case '[':
 			++a;
 			flag = 0;
-			if (*a == '^')
-				++a, inv = 1;
-			else
+			if (*a == '^') {
+				++a;
+				inv = 1;
+			} else
 				inv = 0;
 			if (*a == ']')
 				if (*b == *a++)
@@ -265,18 +274,18 @@ int rmatch(char *a, char *b)
 				return 0;
 			++b;
 			break;
-			case '?':
+		case '?':
 			++a;
 			if (!*b)
 				return 0;
 			++b;
 			break;
-			case 0:
+		case 0:
 			if (!*b)
 				return 1;
 			else
 				return 0;
-			default:
+		default:
 			if (*a++ != *b++)
 				return 0;
 		}
@@ -301,7 +310,7 @@ struct direct {
 } direc;
 int dirstate = 0;
 struct ffblk ffblk;
-char *dirpath = 0;
+char *dirpath = NULL;
 
 void *opendir(char *path)
 {
@@ -319,10 +328,10 @@ struct direct *readdir()
 
 	if (dirstate) {
 		if (findnext(&ffblk))
-			return 0;
+			return NULL;
 	} else {
 		if (findfirst("*.*", &ffblk, FA_DIREC))
-			return 0;
+			return NULL;
 		dirstate = 1;
 	}
 
@@ -336,7 +345,7 @@ struct direct *readdir()
 char **rexpnd(char *word)
 {
 	void *dir;
-	char **lst = 0;
+	char **lst = NULL;
 
 	struct dirent *de;
 	dir = opendir(".");
@@ -381,3 +390,19 @@ int chpwd(char *path)
 	return chdir(path);
 #endif
 }
+
+/* The pwd function */
+char *pwd(void)
+{
+	static char buf[PATH_MAX];
+	char	*ret;
+
+#ifdef HAVE_GETCWD
+	ret = getcwd(buf, PATH_MAX - 1);
+#else
+	ret = getwd(buf);
+#endif
+	buf[PATH_MAX - 1] = '\0';
+
+	return ret;
+}
