@@ -1,12 +1,12 @@
 /*
-	Directory and path functions
-	Copyright (C) 1992 Joseph H. Allen
-
-	This file is part of JOE (Joe's Own Editor)
-*/
-
-#ifndef _joePATH
-#define _joePATH 1
+ *	Directory and path functions
+ *	Copyright
+ *		(C) 1992 Joseph H. Allen
+ *
+ *	This file is part of JOE (Joe's Own Editor)
+ */
+#ifndef _JOE_PATH_H
+#define _JOE_PATH_H 1
 
 #include "config.h"
 
@@ -98,5 +98,6 @@ int isreg PARAMS((char *s));
 char **rexpnd PARAMS((char *word));
 
 int chpwd PARAMS((char *path));
+char *pwd PARAMS((void));
 
 #endif
