@@ -1,28 +1,24 @@
-/* Device independant TTY interface for JOE
-   Copyright (C) 1992 Joseph H. Allen
-
-This file is part of JOE (Joe's Own Editor)
-
-JOE is free software; you can redistribute it and/or modify it under the 
-terms of the GNU General Public License as published by the Free Software 
-Foundation; either version 1, or (at your option) any later version.  
-
-JOE is distributed in the hope that it will be useful, but WITHOUT ANY 
-WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS 
-FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more 
-details.  
-
-You should have received a copy of the GNU General Public License along with 
-JOE; see the file COPYING.  If not, write to the Free Software Foundation, 
-675 Mass Ave, Cambridge, MA 02139, USA.  */
+/*
+ *	Device independant TTY interface for JOE
+ *	Copyright
+ *		(C) 1992 Joseph H. Allen
+ *
+ *	This file is part of JOE (Joe's Own Editor)
+ */
+#include "config.h"
+#include "types.h"
 
+#include <ctype.h>
 #include <stdio.h>
+#ifdef HAVE_STDLIB_H
 #include <stdlib.h>
+#endif
+
+#include "bw.h"
 #include "blocks.h"
-#include "vs.h"
-#include "termcap.h"
-#include "tty.h"
 #include "scrn.h"
+#include "termcap.h"
+#include "utils.h"
 
 int skiptop = 0;
 int lines = 0;
@@ -30,95 +26,114 @@ int columns = 0;
 
 extern int mid;
 
-/* How to display characters */
-
-unsigned xlata[256] = {
-	UNDERLINE, UNDERLINE, UNDERLINE, UNDERLINE,
-	UNDERLINE, UNDERLINE, UNDERLINE, UNDERLINE,
-	UNDERLINE, UNDERLINE, UNDERLINE, UNDERLINE,
-	UNDERLINE, UNDERLINE, UNDERLINE, UNDERLINE,
-	UNDERLINE, UNDERLINE, UNDERLINE, UNDERLINE,
-	UNDERLINE, UNDERLINE, UNDERLINE, UNDERLINE,
-	UNDERLINE, UNDERLINE, UNDERLINE, UNDERLINE,
-	UNDERLINE, UNDERLINE, UNDERLINE, UNDERLINE,
-	0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
-	0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
-	0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
-	0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
-	0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
-	0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
-	UNDERLINE,
-
-	INVERSE + UNDERLINE, INVERSE + UNDERLINE, INVERSE + UNDERLINE,
-	INVERSE + UNDERLINE,
-	INVERSE + UNDERLINE, INVERSE + UNDERLINE, INVERSE + UNDERLINE,
-	INVERSE + UNDERLINE,
-	INVERSE + UNDERLINE, INVERSE + UNDERLINE, INVERSE + UNDERLINE,
-	INVERSE + UNDERLINE,
-	INVERSE + UNDERLINE, INVERSE + UNDERLINE, INVERSE + UNDERLINE,
-	INVERSE + UNDERLINE,
-	INVERSE + UNDERLINE, INVERSE + UNDERLINE, INVERSE + UNDERLINE,
-	INVERSE + UNDERLINE,
-	INVERSE + UNDERLINE, INVERSE + UNDERLINE, INVERSE + UNDERLINE,
-	INVERSE + UNDERLINE,
-	INVERSE + UNDERLINE, INVERSE + UNDERLINE, INVERSE + UNDERLINE,
-	INVERSE + UNDERLINE,
-	INVERSE + UNDERLINE, INVERSE + UNDERLINE, INVERSE + UNDERLINE,
-	INVERSE + UNDERLINE,
-
-	INVERSE, INVERSE, INVERSE, INVERSE, INVERSE, INVERSE, INVERSE,
-	INVERSE,
-	INVERSE, INVERSE, INVERSE, INVERSE, INVERSE, INVERSE, INVERSE,
-	INVERSE,
-	INVERSE, INVERSE, INVERSE, INVERSE, INVERSE, INVERSE, INVERSE,
-	INVERSE,
-	INVERSE, INVERSE, INVERSE, INVERSE, INVERSE, INVERSE, INVERSE,
-	INVERSE,
-	INVERSE, INVERSE, INVERSE, INVERSE, INVERSE, INVERSE, INVERSE,
-	INVERSE,
-	INVERSE, INVERSE, INVERSE, INVERSE, INVERSE, INVERSE, INVERSE,
-	INVERSE,
-	INVERSE, INVERSE, INVERSE, INVERSE, INVERSE, INVERSE, INVERSE,
-	INVERSE,
-	INVERSE, INVERSE, INVERSE, INVERSE, INVERSE, INVERSE, INVERSE,
-	INVERSE,
-	INVERSE, INVERSE, INVERSE, INVERSE, INVERSE, INVERSE, INVERSE,
-	INVERSE,
-	INVERSE, INVERSE, INVERSE, INVERSE, INVERSE, INVERSE, INVERSE,
-	INVERSE,
-	INVERSE, INVERSE, INVERSE, INVERSE, INVERSE, INVERSE, INVERSE,
-	INVERSE,
-	INVERSE, INVERSE, INVERSE, INVERSE, INVERSE, INVERSE, INVERSE,
-
-	INVERSE + UNDERLINE,
-};
-
+/* How to display characters (especially the control ones) */
+/* here are characters ... */
 unsigned char xlatc[256] = {
-	64, 65, 66, 67, 68, 69, 70, 71, 72, 73, 74, 75, 76, 77, 78, 79,
-	80, 81, 82, 83, 84, 85, 86, 87, 88, 89, 90, 91, 92, 93, 94, 95,
-	32, 33, 34, 35, 36, 37, 38, 39, 40, 41, 42, 43, 44, 45, 46, 47,
-	48, 49, 50, 51, 52, 53, 54, 55, 56, 57, 58, 59, 60, 61, 62, 63,
-	64, 65, 66, 67, 68, 69, 70, 71, 72, 73, 74, 75, 76, 77, 78, 79,
-	80, 81, 82, 83, 84, 85, 86, 87, 88, 89, 90, 91, 92, 93, 94, 95,
-	96, 97, 98, 99, 100, 101, 102, 103, 104, 105, 106, 107, 108, 109,
-	110,
-	111,
-	112, 113, 114, 115, 116, 117, 118, 119, 120, 121, 122, 123, 124,
-	125,
-	126, 63,
-	64, 65, 66, 67, 68, 69, 70, 71, 72, 73, 74, 75, 76, 77, 78, 79,
-	80, 81, 82, 83, 84, 85, 86, 87, 88, 89, 90, 91, 92, 93, 94, 95,
-	32, 33, 34, 35, 36, 37, 38, 39, 40, 41, 42, 43, 44, 45, 46, 47,
-	48, 49, 50, 51, 52, 53, 54, 55, 56, 57, 58, 59, 60, 61, 62, 63,
-	64, 65, 66, 67, 68, 69, 70, 71, 72, 73, 74, 75, 76, 77, 78, 79,
-	80, 81, 82, 83, 84, 85, 86, 87, 88, 89, 90, 91, 92, 93, 94, 95,
-	96, 97, 98, 99, 100, 101, 102, 103, 104, 105, 106, 107, 108, 109,
-	110,
-	111,
-	112, 113, 114, 115, 116, 117, 118, 119, 120, 121, 122, 123, 124,
-	125,
-	126, 63
+	 64,  65,  66,  67,  68,  69,  70,  71,			/*   8 */
+	 72,  73,  74,  75,  76,  77,  78,  79,			/*  16 */
+	 80,  81,  82,  83,  84,  85,  86,  87,			/*  24 */
+	 88,  89,  90,  91,  92,  93,  94,  95,			/*  32 */
+	 32,  33,  34,  35,  36,  37,  38,  39,			/*  40 */
+	 40,  41,  42,  43,  44,  45,  46,  47,			/*  48 */
+	 48,  49,  50,  51,  52,  53,  54,  55,			/*  56 */
+	 56,  57,  58,  59,  60,  61,  62,  63,			/*  64 */
+
+	 64,  65,  66,  67,  68,  69,  70,  71,			/*  72 */
+	 72,  73,  74,  75,  76,  77,  78,  79,			/*  80 */
+	 80,  81,  82,  83,  84,  85,  86,  87,			/*  88 */
+	 88,  89,  90,  91,  92,  93,  94,  95,			/*  96 */
+	 96,  97,  98,  99, 100, 101, 102, 103,			/* 104 */
+	104, 105, 106, 107, 108, 109, 110, 111,			/* 112 */
+	112, 113, 114, 115, 116, 117, 118, 119,			/* 120 */
+	120, 121, 122, 123, 124, 125, 126,  63,			/* 128 */
+
+	 64,  65,  66,  67,  68,  69,  70,  71,			/* 136 */
+	 72,  73,  74,  75,  76,  77,  78,  79,			/* 144 */
+	 80,  81,  82,  83,  84,  85,  86,  87,			/* 152 */
+	 88,  89,  90,  91,  92,  93,  94,  95,			/* 160 */
+	 32,  33,  34,  35,  36,  37,  38,  39,			/* 168 */
+	 40,  41,  42,  43,  44,  45,  46,  47,			/* 176 */
+	 48,  49,  50,  51,  52,  53,  54,  55,			/* 184 */
+	 56,  57,  58,  59,  60,  61,  62,  63,			/* 192 */
+
+	 64,  65,  66,  67,  68,  69,  70,  71,			/* 200 */
+	 72,  73,  74,  75,  76,  77,  78,  79,			/* 208 */
+	 80,  81,  82,  83,  84,  85,  86,  87,			/* 216 */
+	 88,  89,  90,  91,  92,  93,  94,  95,			/* 224 */
+	 96,  97,  98,  99, 100, 101, 102, 103,			/* 232 */
+	104, 105, 106, 107, 108, 109, 110, 111,			/* 240 */
+	112, 113, 114, 115, 116, 117, 118, 119,			/* 248 */
+	120, 121, 122, 123, 124, 125, 126,  63			/* 256 */
 };
+/* ... and here their attributes */ 
+unsigned xlata[256] = {
+	UNDERLINE, UNDERLINE, UNDERLINE, UNDERLINE,		/*   4 */
+	UNDERLINE, UNDERLINE, UNDERLINE, UNDERLINE,		/*   8 */
+	UNDERLINE, UNDERLINE, UNDERLINE, UNDERLINE,		/*  12 */
+	UNDERLINE, UNDERLINE, UNDERLINE, UNDERLINE,		/*  16 */
+	UNDERLINE, UNDERLINE, UNDERLINE, UNDERLINE,		/*  20 */
+	UNDERLINE, UNDERLINE, UNDERLINE, UNDERLINE,		/*  24 */
+	UNDERLINE, UNDERLINE, UNDERLINE, UNDERLINE,		/*  28 */
+	UNDERLINE, UNDERLINE, UNDERLINE, UNDERLINE,		/*  32 */
+	0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,		/*  48 */
+	0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,		/*  64 */
+	0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,		/*  80 */
+	0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,		/*  96 */
+	0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,		/* 112 */
+	0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, UNDERLINE,	/* 128 */
+
+	INVERSE + UNDERLINE, INVERSE + UNDERLINE,		/* 130 */
+	INVERSE + UNDERLINE, INVERSE + UNDERLINE,		/* 132 */
+	INVERSE + UNDERLINE, INVERSE + UNDERLINE,		/* 134 */
+	INVERSE + UNDERLINE, INVERSE + UNDERLINE,		/* 136 */
+	INVERSE + UNDERLINE, INVERSE + UNDERLINE,		/* 138 */
+	INVERSE + UNDERLINE, INVERSE + UNDERLINE,		/* 140 */
+	INVERSE + UNDERLINE, INVERSE + UNDERLINE,		/* 142 */
+	INVERSE + UNDERLINE, INVERSE + UNDERLINE,		/* 144 */
+	INVERSE + UNDERLINE, INVERSE + UNDERLINE,		/* 146 */
+	INVERSE + UNDERLINE, INVERSE + UNDERLINE,		/* 148 */
+	INVERSE + UNDERLINE, INVERSE + UNDERLINE,		/* 150 */
+	INVERSE + UNDERLINE, INVERSE + UNDERLINE,		/* 152 */
+	INVERSE + UNDERLINE, INVERSE + UNDERLINE,		/* 154 */
+	INVERSE + UNDERLINE, INVERSE + UNDERLINE,		/* 156 */
+	INVERSE + UNDERLINE, INVERSE + UNDERLINE,		/* 158 */
+	INVERSE + UNDERLINE, INVERSE + UNDERLINE,		/* 160 */
+
+	INVERSE, INVERSE, INVERSE, INVERSE,			/* 164 */
+	INVERSE, INVERSE, INVERSE, INVERSE,			/* 168 */
+	INVERSE, INVERSE, INVERSE, INVERSE,			/* 172 */
+	INVERSE, INVERSE, INVERSE, INVERSE,			/* 176 */
+	INVERSE, INVERSE, INVERSE, INVERSE,			/* 180 */
+	INVERSE, INVERSE, INVERSE, INVERSE,			/* 184 */
+	INVERSE, INVERSE, INVERSE, INVERSE,			/* 188 */
+	INVERSE, INVERSE, INVERSE, INVERSE,			/* 192 */
+	INVERSE, INVERSE, INVERSE, INVERSE,			/* 196 */
+	INVERSE, INVERSE, INVERSE, INVERSE,			/* 200 */
+	INVERSE, INVERSE, INVERSE, INVERSE,			/* 204 */
+	INVERSE, INVERSE, INVERSE, INVERSE,			/* 208 */
+	INVERSE, INVERSE, INVERSE, INVERSE,			/* 212 */
+	INVERSE, INVERSE, INVERSE, INVERSE,			/* 216 */
+	INVERSE, INVERSE, INVERSE, INVERSE,			/* 220 */
+	INVERSE, INVERSE, INVERSE, INVERSE,			/* 224 */
+	INVERSE, INVERSE, INVERSE, INVERSE,			/* 228 */
+	INVERSE, INVERSE, INVERSE, INVERSE,			/* 232 */
+	INVERSE, INVERSE, INVERSE, INVERSE,			/* 236 */
+	INVERSE, INVERSE, INVERSE, INVERSE,			/* 240 */
+	INVERSE, INVERSE, INVERSE, INVERSE,			/* 244 */
+	INVERSE, INVERSE, INVERSE, INVERSE,			/* 248 */
+	INVERSE, INVERSE, INVERSE, INVERSE,			/* 252 */
+	INVERSE, INVERSE, INVERSE, INVERSE + UNDERLINE		/* 256 */
+};
+
+void xlat(int *attr, unsigned char *c)
+{
+	if(isprint(*c) || (dspasis && *c > 128))
+		*attr = 0;
+	else {
+		*attr = xlata[*c];
+		*c = xlatc[*c];
+	}
+}
 
 /* Set attributes */
 
@@ -212,12 +227,12 @@ int eraeol(SCRN *t, int x, int y)
 		return 0;
 	s = t->scrn + y * t->co + x;
 	ss = s + w;
-	do
+	do {
 		if (*--ss != ' ') {
 			++ss;
 			break;
 		}
-	while (ss != s) ;
+	} while (ss != s);
 	if ((ss - s > 3 || s[w] != ' ') && t->ce) {
 		cpos(t, x, y);
 		attr(t, 0);
@@ -266,7 +281,7 @@ static void out(char *t, char c)
 
 SCRN *nopen(CAP *cap)
 {
-	SCRN *t = (SCRN *) malloc(sizeof(SCRN));
+	SCRN *t = (SCRN *) joe_malloc(sizeof(SCRN));
 	int x, y;
 
 	ttopen();
@@ -282,8 +297,10 @@ SCRN *nopen(CAP *cap)
 		t->co = 80;
 	x = y = 0;
 	ttgtsz(&x, &y);
-	if (x > 7 && y > 3)
-		t->li = y, t->co = x;
+	if (x > 7 && y > 3) {
+		t->li = y;
+		t->co = x;
+	}
 
 	t->haz = getflag(t->cap, "hz");
 	t->os = getflag(t->cap, "os");
@@ -304,10 +321,10 @@ SCRN *nopen(CAP *cap)
 
 	t->te = jgetstr(t->cap, "te");
 
-	t->mb = 0;
-	t->md = 0;
-	t->mh = 0;
-	t->mr = 0;
+	t->mb = NULL;
+	t->md = NULL;
+	t->mh = NULL;
+	t->mr = NULL;
 	t->avattr = 0;
 	if (!(t->me = jgetstr(t->cap, "me")))
 		goto oops;
@@ -321,18 +338,18 @@ SCRN *nopen(CAP *cap)
 		t->avattr |= INVERSE;
       oops:
 
-	t->so = 0;
-	t->se = 0;
+	t->so = NULL;
+	t->se = NULL;
 	if (getnum(t->cap, "sg") <= 0 && !t->mr && jgetstr(t->cap, "se")) {
 		if ((t->so = jgetstr(t->cap, "so")) != NULL)
 			t->avattr |= INVERSE;
 		t->se = jgetstr(t->cap, "se");
 	}
 	if (getflag(t->cap, "xs") || getflag(t->cap, "xt"))
-		t->so = 0;
+		t->so = NULL;
 
-	t->us = 0;
-	t->ue = 0;
+	t->us = NULL;
+	t->ue = NULL;
 	if (getnum(t->cap, "ug") <= 0 && jgetstr(t->cap, "ue")) {
 		if ((t->us = jgetstr(t->cap, "us")) != NULL)
 			t->avattr |= UNDERLINE;
@@ -375,19 +392,19 @@ SCRN *nopen(CAP *cap)
 		t->ip = jgetstr(t->cap, "ip");
 		t->mi = getflag(t->cap, "mi");
 	} else {
-		t->dm = 0;
-		t->dc = 0;
-		t->DC = 0;
-		t->ed = 0;
-		t->im = 0;
-		t->ic = 0;
-		t->IC = 0;
-		t->ip = 0;
-		t->ei = 0;
+		t->dm = NULL;
+		t->dc = NULL;
+		t->DC = NULL;
+		t->ed = NULL;
+		t->im = NULL;
+		t->ic = NULL;
+		t->IC = NULL;
+		t->ip = NULL;
+		t->ei = NULL;
 		t->mi = 1;
 	}
 
-	t->bs = 0;
+	t->bs = NULL;
 	if (jgetstr(t->cap, "bc"))
 		t->bs = jgetstr(t->cap, "bc");
 	else if (jgetstr(t->cap, "le"))
@@ -417,8 +434,10 @@ SCRN *nopen(CAP *cap)
 		if (getflag(t->cap, "pt"))
 			t->ta = "\11";
 	t->bt = jgetstr(t->cap, "bt");
-	if (getflag(t->cap, "xt"))
-		t->ta = 0, t->bt = 0;
+	if (getflag(t->cap, "xt")) {
+		t->ta = NULL;
+		t->bt = NULL;
+	}
 
 	t->cta = tcost(t->cap, t->ta, 1, 2, 2, 0, 0);
 	t->cbt = tcost(t->cap, t->bt, 1, 2, 2, 0, 0);
@@ -432,7 +451,7 @@ SCRN *nopen(CAP *cap)
 	if (jgetstr(t->cap, "cr"))
 		t->cr = jgetstr(t->cap, "cr");
 	if (getflag(t->cap, "nc") || getflag(t->cap, "xr"))
-		t->cr = 0;
+		t->cr = NULL;
 	t->ccr = tcost(t->cap, t->cr, 1, 2, 2, 0, 0);
 
 	t->cRI = tcost(t->cap, t->RI = jgetstr(t->cap, "RI"), 1, 2, 2, 0, 0);
@@ -461,7 +480,7 @@ SCRN *nopen(CAP *cap)
 	ttclose();
 	signrm();
 	fprintf(stderr, "Sorry, your terminal can't do absolute cursor positioning.\nIt's broken\n");
-	return 0;
+	return NULL;
       ok:
 
 /* Determine if we can scroll */
@@ -480,23 +499,25 @@ SCRN *nopen(CAP *cap)
 		t->insdel = 0;
 
 /* Adjust for high baud rates */
-	if (baud >= 38400)
-		t->scroll = 0, t->insdel = 0;
+	if (baud >= 38400) {
+		t->scroll = 0;
+		t->insdel = 0;
+	}
 
 /* Send out terminal initialization string */
 	if (t->ti)
 		texec(t->cap, t->ti, 1, 0, 0, 0, 0);
-	if (t->cl)
+	if (!skiptop && t->cl)
 		texec(t->cap, t->cl, 1, 0, 0, 0, 0);
 
 /* Initialize variable screen size dependant vars */
-	t->scrn = 0;
-	t->sary = 0;
-	t->updtab = 0;
-	t->compose = 0;
-	t->ofst = 0;
-	t->ary = 0;
-	t->htab = (struct hentry *) malloc(256 * sizeof(struct hentry));
+	t->scrn = NULL;
+	t->sary = NULL;
+	t->updtab = NULL;
+	t->compose = NULL;
+	t->ofst = NULL;
+	t->ary = NULL;
+	t->htab = (struct hentry *) joe_malloc(256 * sizeof(struct hentry));
 
 	nresize(t, t->co, t->li);
 
@@ -514,23 +535,23 @@ void nresize(SCRN *t, int w, int h)
 	t->li = h;
 	t->co = w;
 	if (t->sary)
-		free(t->sary);
+		joe_free(t->sary);
 	if (t->updtab)
-		free(t->updtab);
+		joe_free(t->updtab);
 	if (t->scrn)
-		free(t->scrn);
+		joe_free(t->scrn);
 	if (t->compose)
-		free(t->compose);
+		joe_free(t->compose);
 	if (t->ofst)
-		free(t->ofst);
+		joe_free(t->ofst);
 	if (t->ary)
-		free(t->ary);
-	t->scrn = (int *) malloc(t->li * t->co * sizeof(int));
-	t->sary = (int *) calloc(t->li, sizeof(int));
-	t->updtab = (int *) malloc(t->li * sizeof(int));
-	t->compose = (int *) malloc(t->co * sizeof(int));
-	t->ofst = (int *) malloc(t->co * sizeof(int));
-	t->ary = (struct hentry *) malloc(t->co * sizeof(struct hentry));
+		joe_free(t->ary);
+	t->scrn = (int *) joe_malloc(t->li * t->co * sizeof(int));
+	t->sary = (int *) joe_calloc(t->li, sizeof(int));
+	t->updtab = (int *) joe_malloc(t->li * sizeof(int));
+	t->compose = (int *) joe_malloc(t->co * sizeof(int));
+	t->ofst = (int *) joe_malloc(t->co * sizeof(int));
+	t->ary = (struct hentry *) joe_malloc(t->co * sizeof(struct hentry));
 
 	nredraw(t);
 }
@@ -599,7 +620,8 @@ static int relcost(register SCRN *t, register int x, register int y, register in
 	if (x > ox && t->ta) {
 		int dist = x - ox;
 		int ntabs = (dist + ox % t->tw) / t->tw;
-		int cstunder = x % t->tw + t->cta * ntabs, cstover;
+		int cstunder = x % t->tw + t->cta * ntabs;
+		int cstover;
 
 		if (x + t->tw < t->co && t->bs)
 			cstover = t->cbs * (t->tw - x % t->tw) + t->cta * (ntabs + 1);
@@ -687,10 +709,13 @@ static void cposs(register SCRN *t, register int x, register int y)
 /* Home y position is usually 0, but it is 'top' if we have scrolling region
  * relative addressing
  */
-	if (t->rr)
-		hy = t->top, hl = t->bot - 1;
-	else
-		hy = 0, hl = t->li - 1;
+	if (t->rr) {
+		hy = t->top;
+		hl = t->bot - 1;
+	} else {
+		hy = 0;
+		hl = t->li - 1;
+	}
 
 /* Assume best way is with only using relative cursor positioning */
 
@@ -703,68 +728,94 @@ static void cposs(register SCRN *t, register int x, register int y)
 
 	if (t->ccm < bestcost) {
 		cost = tcost(t->cap, t->cm, 1, y, x, 0, 0);
-		if (cost < bestcost)
-			bestcost = cost, bestway = 6;
+		if (cost < bestcost) {
+			bestcost = cost;
+			bestway = 6;
+		}
 	}
 	if (t->ccr < bestcost) {
 		cost = relcost(t, x, y, 0, t->y) + t->ccr;
-		if (cost < bestcost)
-			bestcost = cost, bestway = 1;
+		if (cost < bestcost) {
+			bestcost = cost;
+			bestway = 1;
+		}
 	}
 	if (t->cho < bestcost) {
 		cost = relcost(t, x, y, 0, hy) + t->cho;
-		if (cost < bestcost)
-			bestcost = cost, bestway = 2;
+		if (cost < bestcost) {
+			bestcost = cost;
+			bestway = 2;
+		}
 	}
 	if (t->cll < bestcost) {
 		cost = relcost(t, x, y, 0, hl) + t->cll;
-		if (cost < bestcost)
-			bestcost = cost, bestway = 3;
+		if (cost < bestcost) {
+			bestcost = cost;
+			bestway = 3;
+		}
 	}
 	if (t->cch < bestcost && x != t->x) {
 		cost = relcost(t, x, y, x, t->y) + tcost(t->cap, t->ch, 1, x, 0, 0, 0);
-		if (cost < bestcost)
-			bestcost = cost, bestway = 4;
+		if (cost < bestcost) {
+			bestcost = cost;
+			bestway = 4;
+		}
 	}
 	if (t->ccv < bestcost && y != t->y) {
 		cost = relcost(t, x, y, t->x, y) + tcost(t->cap, t->cv, 1, y, 0, 0, 0);
-		if (cost < bestcost)
-			bestcost = cost, bestway = 5;
+		if (cost < bestcost) {
+			bestcost = cost;
+			bestway = 5;
+		}
 	}
 	if (t->ccV < bestcost) {
 		cost = relcost(t, x, y, 0, y) + tcost(t->cap, t->cV, 1, y, 0, 0, 0);
-		if (cost < bestcost)
-			bestcost = cost, bestway = 13;
+		if (cost < bestcost) {
+			bestcost = cost;
+			bestway = 13;
+		}
 	}
 	if (t->cch + t->ccv < bestcost && x != t->x && y != t->y) {
 		cost = tcost(t->cap, t->cv, 1, y - hy, 0, 0, 0) + tcost(t->cap, t->ch, 1, x, 0, 0, 0);
-		if (cost < bestcost)
-			bestcost = cost, bestway = 7;
+		if (cost < bestcost) {
+			bestcost = cost;
+			bestway = 7;
+		}
 	}
 	if (t->ccv + t->ccr < bestcost && y != t->y) {
 		cost = tcost(t->cap, t->cv, 1, y, 0, 0, 0) + tcost(t->cap, t->cr, 1, 0, 0, 0, 0) + relcost(t, x, y, 0, y);
-		if (cost < bestcost)
-			bestcost = cost, bestway = 8;
+		if (cost < bestcost) {
+			bestcost = cost;
+			bestway = 8;
+		}
 	}
 	if (t->cll + t->cch < bestcost) {
 		cost = tcost(t->cap, t->ll, 1, 0, 0, 0, 0) + tcost(t->cap, t->ch, 1, x, 0, 0, 0) + relcost(t, x, y, x, hl);
-		if (cost < bestcost)
-			bestcost = cost, bestway = 9;
+		if (cost < bestcost) {
+			bestcost = cost;
+			bestway = 9;
+		}
 	}
 	if (t->cll + t->ccv < bestcost) {
 		cost = tcost(t->cap, t->ll, 1, 0, 0, 0, 0) + tcost(t->cap, t->cv, 1, y, 0, 0, 0) + relcost(t, x, y, 0, y);
-		if (cost < bestcost)
-			bestcost = cost, bestway = 10;
+		if (cost < bestcost) {
+			bestcost = cost;
+			bestway = 10;
+		}
 	}
 	if (t->cho + t->cch < bestcost) {
 		cost = tcost(t->cap, t->ho, 1, 0, 0, 0, 0) + tcost(t->cap, t->ch, 1, x, 0, 0, 0) + relcost(t, x, y, x, hy);
-		if (cost < bestcost)
-			bestcost = cost, bestway = 11;
+		if (cost < bestcost) {
+			bestcost = cost;
+			bestway = 11;
+		}
 	}
 	if (t->cho + t->ccv < bestcost) {
 		cost = tcost(t->cap, t->ho, 1, 0, 0, 0, 0) + tcost(t->cap, t->cv, 1, y, 0, 0, 0) + relcost(t, x, y, 0, y);
-		if (cost < bestcost)
-			bestcost = cost, bestway = 12;
+		if (cost < bestcost) {
+			bestcost = cost;
+			bestway = 12;
+		}
 	}
 
 /* Do absolute cursor positioning if we don't know the cursor position or
@@ -772,63 +823,64 @@ static void cposs(register SCRN *t, register int x, register int y)
  */
 
 	switch (bestway) {
-		case 1:
+	case 1:
 		texec(t->cap, t->cr, 1, 0, 0, 0, 0);
 		t->x = 0;
 		break;
-		case 2:
+	case 2:
 		texec(t->cap, t->ho, 1, 0, 0, 0, 0);
 		t->x = 0;
 		t->y = hy;
 		break;
-		case 3:
+	case 3:
 		texec(t->cap, t->ll, 1, 0, 0, 0, 0);
 		t->x = 0;
 		t->y = hl;
 		break;
-		case 9:
+	case 9:
 		texec(t->cap, t->ll, 1, 0, 0, 0, 0);
 		t->x = 0;
 		t->y = hl;
 		goto doch;
-		case 11:
+	case 11:
 		texec(t->cap, t->ho, 1, 0, 0, 0, 0);
 		t->x = 0;
 		t->y = hy;
-	      doch:
-		case 4:
+doch:
+	case 4:
 		texec(t->cap, t->ch, 1, x, 0, 0, 0);
 		t->x = x;
 		break;
-		case 10:
+	case 10:
 		texec(t->cap, t->ll, 1, 0, 0, 0, 0);
 		t->x = 0;
 		t->y = hl;
 		goto docv;
-		case 12:
+	case 12:
 		texec(t->cap, t->ho, 1, 0, 0, 0, 0);
 		t->x = 0;
 		t->y = hy;
 		goto docv;
-		case 8:
+	case 8:
 		texec(t->cap, t->cr, 1, 0, 0, 0, 0);
 		t->x = 0;
-	      docv:
-		case 5:
+docv:
+	case 5:
 		texec(t->cap, t->cv, 1, y, 0, 0, 0);
 		t->y = y;
 		break;
-		case 6:
+	case 6:
 		texec(t->cap, t->cm, 1, y, x, 0, 0);
-		t->y = y, t->x = x;
+		t->y = y;
+		t->x = x;
 		break;
-		case 7:
+	case 7:
 		texec(t->cap, t->cv, 1, y, 0, 0, 0);
 		t->y = y;
 		texec(t->cap, t->ch, 1, x, 0, 0, 0);
 		t->x = x;
 		break;
-		case 13:
+	case 13:
 		texec(t->cap, t->cV, 1, y, 0, 0, 0);
 		t->y = y;
 		t->x = 0;
@@ -840,24 +892,31 @@ static void cposs(register SCRN *t, register int x, register int y)
 /* First adjust row */
 	if (y > t->y) {
 		/* Have to go down */
-		if (!t->lf || t->cDO < (y - t->y) * t->clf)
-			texec(t->cap, t->DO, 1, y - t->y, 0, 0, 0), t->y = y;
-		else
-			while (y > t->y)
-				texec(t->cap, t->lf, 1, 0, 0, 0, 0), ++t->y;
+		if (!t->lf || t->cDO < (y - t->y) * t->clf) {
+			texec(t->cap, t->DO, 1, y - t->y, 0, 0, 0);
+			t->y = y;
+		} else
+			while (y > t->y) {
+				texec(t->cap, t->lf, 1, 0, 0, 0, 0);
+				++t->y;
+			}
 	} else if (y < t->y) {
 		/* Have to go up */
-		if (!t->up || t->cUP < (t->y - y) * t->cup)
-			texec(t->cap, t->UP, 1, t->y - y, 0, 0, 0), t->y = y;
-		else
-			while (y < t->y)
-				texec(t->cap, t->up, 1, 0, 0, 0, 0), --t->y;
+		if (!t->up || t->cUP < (t->y - y) * t->cup) {
+			texec(t->cap, t->UP, 1, t->y - y, 0, 0, 0);
+			t->y = y;
+		} else
+			while (y < t->y) {
+				texec(t->cap, t->up, 1, 0, 0, 0, 0);
+				--t->y;
+			}
 	}
 
 /* Use tabs */
 	if (x > t->x && t->ta) {
 		int ntabs = (x - t->x + t->x % t->tw) / t->tw;
-		int cstunder = x % t->tw + t->cta * ntabs, cstover;
+		int cstunder = x % t->tw + t->cta * ntabs;
+		int cstover;
 
 		if (x + t->tw < t->co && t->bs)
 			cstover = t->cbs * (t->tw - x % t->tw) + t->cta * (ntabs + 1);
@@ -866,16 +925,16 @@ static void cposs(register SCRN *t, register int x, register int y)
 		if (cstunder < t->cRI && cstunder < x - t->x && cstover > cstunder) {
 			if (ntabs) {
 				t->x = x - x % t->tw;
-				do
+				do {
 					texec(t->cap, t->ta, 1, 0, 0, 0, 0);
-				while (--ntabs);
+				} while (--ntabs);
 			}
 		} else if (cstover < t->cRI && cstover < x - t->x) {
 			t->x = t->tw + x - x % t->tw;
 			++ntabs;
-			do
+			do {
 				texec(t->cap, t->ta, 1, 0, 0, 0, 0);
-			while (--ntabs);
+			} while (--ntabs);
 		}
 	} else if (x < t->x && t->bt) {
 		int ntabs = ((t->x + t->tw - 1) - (t->x + t->tw - 1) % t->tw - ((x + t->tw - 1) - (x + t->tw - 1) % t->tw)) / t->tw;
@@ -892,34 +951,38 @@ static void cposs(register SCRN *t, register int x, register int y)
 		if (cstunder < t->cLE && (t->bs ? cstunder < (t->x - x) * t->cbs : 1)
 		    && cstover > cstunder) {
 			if (ntabs) {
-				do
+				do {
 					texec(t->cap, t->bt, 1, 0, 0, 0, 0);
-				while (--ntabs);
+				} while (--ntabs);
 				t->x = x + t->tw - x % t->tw;
 			}
 		} else if (cstover < t->cRI && (t->bs ? cstover < (t->x - x) * t->cbs : 1)) {
 			t->x = x - x % t->tw;
 			++ntabs;
-			do
+			do {
 				texec(t->cap, t->bt, 1, 0, 0, 0, 0);
-			while (--ntabs);
+			} while (--ntabs);
 		}
 	}
 
 /* Now adjust column */
 	if (x < t->x) {
 		/* Have to go left */
-		if (!t->bs || t->cLE < (t->x - x) * t->cbs)
-			texec(t->cap, t->LE, 1, t->x - x, 0, 0, 0), t->x = x;
-		else
-			while (x < t->x)
-				texec(t->cap, t->bs, 1, 0, 0, 0, 0), --t->x;
+		if (!t->bs || t->cLE < (t->x - x) * t->cbs) {
+			texec(t->cap, t->LE, 1, t->x - x, 0, 0, 0);
+			t->x = x;
+		} else
+			while (x < t->x) {
+				texec(t->cap, t->bs, 1, 0, 0, 0, 0);
+				--t->x;
+			}
 	} else if (x > t->x) {
 		/* Have to go right */
 		/* Hmm.. this should take into account possible attribute changes */
-		if (t->cRI < x - t->x)
-			texec(t->cap, t->RI, 1, x - t->x, 0, 0, 0), t->x = x;
-		else {
+		if (t->cRI < x - t->x) {
+			texec(t->cap, t->RI, 1, x - t->x, 0, 0, 0);
+			t->x = x;
+		} else {
 			int *s = t->scrn + t->x + t->y * t->co;
 
 			if (t->ins)
@@ -957,8 +1020,7 @@ int cpos(register SCRN *t, register int x, register int y)
 				ttputc(c);
 				++cs;
 				++t->x;
-			}
-			while (x != t->x);
+			} while (x != t->x);
 			return 0;
 		}
 	}
@@ -974,8 +1036,10 @@ static void doinschr(SCRN *t, int x, int y, int *s, int n)
 {
 	int a;
 
-	if (x < 0)
-		s -= x, x = 0;
+	if (x < 0) {
+		s -= x;
+		x = 0;
+	}
 	if (x >= t->co - 1 || n <= 0)
 		return;
 	if (t->im || t->ic || t->IC) {
@@ -997,7 +1061,7 @@ static void doinschr(SCRN *t, int x, int y, int *s, int n)
 		}
 	}
 	mmove(t->scrn + x + t->co * y + n, t->scrn + x + t->co * y, (t->co - (x + n)) * sizeof(int));
-	mcpy(t->scrn + x + t->co * y, s, n * sizeof(int));
+	mmove(t->scrn + x + t->co * y, s, n * sizeof(int));
 }
 
 static void dodelchr(SCRN *t, int x, int y, int n)
@@ -1019,7 +1083,6 @@ static void dodelchr(SCRN *t, int x, int y, int n)
 		texec(t->cap, t->ed, 1, x, 0, 0, 0);	/* Exit delete mode */
 	}
 	mmove(t->scrn + t->co * y + x, t->scrn + t->co * y + x + n, (t->co - (x + n)) * sizeof(int));
-
 	msetI(t->scrn + t->co * y + t->co - n, ' ', n);
 }
 
@@ -1039,8 +1102,12 @@ void magic(SCRN *t, int y, int *cs, int *s, int placex)
 	msetI(ofst, 0, t->co);
 
 /* Build hash table */
-	for (x = 0; x != t->co - 1; ++x)
-		t->ary[aryx].next = htab[cs[x] & 255].next, t->ary[aryx].loc = x, ++htab[cs[x] & 255].loc, htab[cs[x] & 255].next = aryx++;
+	for (x = 0; x != t->co - 1; ++x) {
+		t->ary[aryx].next = htab[cs[x] & 255].next;
+		t->ary[aryx].loc = x;
+		++htab[cs[x] & 255].loc;
+		htab[cs[x] & 255].next = aryx++;
+	}
 
 /* Build offset table */
 	for (x = 0; x < t->co - 1;)
@@ -1075,12 +1142,17 @@ void magic(SCRN *t, int y, int *cs, int *s, int placex)
 						++cst;
 					pre = s[x + back - 1] & 255;
 				}
-				if (cst > best)
-					maxaryy = aryy, maxlen = amnt, best = cst, bestback = back;
+				if (cst > best) {
+					maxaryy = aryy;
+					maxlen = amnt;
+					best = cst;
+					bestback = back;
+				}
 			}
-			if (!maxlen)
-				ofst[x] = t->co - 1, maxlen = 1;
-			else if (best < 2)
+			if (!maxlen) {
+				ofst[x] = t->co - 1;
+				maxlen = 1;
+			} else if (best < 2)
 				for (z = 0; z != maxlen; ++z)
 					ofst[x + z] = t->co - 1;
 			else
@@ -1110,8 +1182,7 @@ void magic(SCRN *t, int y, int *cs, int *s, int placex)
 				int z, fu;
 
 				for (z = x; z != t->co - 1 && ofst[z] == q; ++z) ;
-				while (s[x + q] == cs[x + q]
-				       && x - q < placex)
+				while (s[x + q] == cs[x + q] && x - q < placex)
 					++x;
 				doinschr(t, x + q, y, s + x + q, -q);
 				for (fu = x; fu != t->co - 1; ++fu)
@@ -1180,7 +1251,7 @@ static void doupscrl(SCRN *t, int top, int bot, int amnt)
 	return;
 
       done:
-	mfwrd(t->scrn + top * t->co, t->scrn + (top + amnt) * t->co, (bot - top - amnt) * t->co * sizeof(int));
+	mmove(t->scrn + top * t->co, t->scrn + (top + amnt) * t->co, (bot - top - amnt) * t->co * sizeof(int));
 
 	if (bot == t->li && t->db) {
 		msetI(t->scrn + (t->li - amnt) * t->co, -1, amnt * t->co);
@@ -1245,7 +1316,7 @@ static void dodnscrl(SCRN *t, int top, int bot, int amnt)
 	msetI(t->updtab + top, 1, bot - top);
 	return;
       done:
-	mbkwd(t->scrn + (top + amnt) * t->co, t->scrn + top * t->co, (bot - top - amnt) * t->co * sizeof(int));
+	mmove(t->scrn + (top + amnt) * t->co, t->scrn + top * t->co, (bot - top - amnt) * t->co * sizeof(int));
 
 	if (!top && t->da) {
 		msetI(t->scrn, -1, amnt * t->co);
@@ -1266,20 +1337,19 @@ void nscroll(SCRN *t)
 			if (q > 0) {
 				for (z = y; z != t->li && t->sary[z] == q; ++z)
 					t->sary[z] = 0;
-				doupscrl(t, y, z + q, q), y = z - 1;
+				doupscrl(t, y, z + q, q);
+				y = z - 1;
 			} else {
 				for (r = y; r != t->li && (t->sary[r] < 0 || t->sary[r] == t->li); ++r) ;
 				p = r - 1;
 				do {
 					q = t->sary[p];
 					if (q && q != t->li) {
-						for (z = p; t->sary[z] = 0, (z && t->sary[z - 1]
-									     == q); --z) ;
+						for (z = p; t->sary[z] = 0, (z && t->sary[z - 1] == q); --z) ;
 						dodnscrl(t, z + q, p + 1, -q);
 						p = z + 1;
 					}
-				}
-				while (p-- != y);
+				} while (p-- != y);
 				y = r - 1;
 			}
 		}
@@ -1307,7 +1377,7 @@ void nreturn(SCRN *t)
 {
 	if (t->ti)
 		texec(t->cap, t->ti, 1, 0, 0, 0, 0);
-	if (t->cl)
+	if (!skiptop && t->cl)
 		texec(t->cap, t->cl, 1, 0, 0, 0, 0);
 	nredraw(t);
 }
@@ -1323,12 +1393,12 @@ void nclose(SCRN *t)
 		texec(t->cap, t->te, 1, 0, 0, 0, 0);
 	ttclose();
 	rmcap(t->cap);
-	free(t->scrn);
-	free(t->sary);
-	free(t->ofst);
-	free(t->htab);
-	free(t->ary);
-	free(t);
+	joe_free(t->scrn);
+	joe_free(t->sary);
+	joe_free(t->ofst);
+	joe_free(t->htab);
+	joe_free(t->ary);
+	joe_free(t);
 }
 
 void nscrldn(SCRN *t, int top, int bot, int amnt)
@@ -1340,8 +1410,10 @@ void nscrldn(SCRN *t, int top, int bot, int amnt)
 	if ((amnt < bot - top && bot - top - amnt < amnt / 2) || !t->scroll)
 		amnt = bot - top;
 	if (amnt < bot - top) {
-		for (x = bot; x != top + amnt; --x)
-			t->sary[x - 1] = (t->sary[x - amnt - 1] == t->li ? t->li : t->sary[x - amnt - 1] - amnt), t->updtab[x - 1] = t->updtab[x - amnt - 1];
+		for (x = bot; x != top + amnt; --x) {
+			t->sary[x - 1] = (t->sary[x - amnt - 1] == t->li ? t->li : t->sary[x - amnt - 1] - amnt);
+			t->updtab[x - 1] = t->updtab[x - amnt - 1];
+		}
 		for (x = top; x != top + amnt; ++x)
 			t->updtab[x] = 1;
 	}
@@ -1361,8 +1433,10 @@ void nscrlup(SCRN *t, int top, int bot, int amnt)
 	if ((amnt < bot - top && bot - top - amnt < amnt / 2) || !t->scroll)
 		amnt = bot - top;
 	if (amnt < bot - top) {
-		for (x = top + amnt; x != bot; ++x)
-			t->sary[x - amnt] = (t->sary[x] == t->li ? t->li : t->sary[x] + amnt), t->updtab[x - amnt] = t->updtab[x];
+		for (x = top + amnt; x != bot; ++x) {
+			t->sary[x - amnt] = (t->sary[x] == t->li ? t->li : t->sary[x] + amnt);
+			t->updtab[x - amnt] = t->updtab[x];
+		}
 		for (x = bot - amnt; x != bot; ++x)
 			t->updtab[x] = 1;
 	}
