@@ -1,14 +1,12 @@
-
 /*
-	UNIX Tty and Process interface
-	Copyright (C) 1992 Joseph H. Allen
-
-	This file is part of JOE (Joe's Own Editor)
-*/
-
-/** System include files **/
-
+ *	UNIX Tty and Process interface
+ *	Copyright
+ *		(C) 1992 Joseph H. Allen
+ *
+ *	This file is part of JOE (Joe's Own Editor)
+ */
 #include "config.h"
+#include "types.h"
 
 #include <sys/types.h>
 #ifdef HAVE_SYS_STAT_H
@@ -20,7 +18,6 @@
 #endif
 #endif
 #include <stdio.h>
-#include <signal.h>
 #ifdef HAVE_FCNTL_H
 #include <fcntl.h>
 #endif
@@ -35,8 +32,6 @@
 #include <stdlib.h>
 #include <unistd.h>
 
-extern int errno;
-
 int idleout = 1;
 
 #include "config.h"
@@ -90,27 +85,7 @@ int idleout = 1;
 #include "main.h"
 #include "path.h"
 #include "tty.h"
-
-/* The pwd function */
-#ifdef HAVE_GETCWD
-char *pwd(void)
-{
-	static char buf[1024];
-
-	return getcwd(buf, 1024);
-}
-#else
-#ifdef HAVE_GETWD
-char *pwd(void)
-{
-	static char buf[1024];
-
-	return getwd(buf);
-}
-#else
-#error "Don't know how to get current directory"
-#endif
-#endif
+#include "utils.h"
 
 /** Aliased defines **/
 
@@ -143,8 +118,8 @@ int Baud = 0;			/* Baud rate from joerc, cmd line or environment */
 
 /* The terminal */
 
-FILE *termin = 0;
-FILE *termout = 0;
+FILE *termin = NULL;
+FILE *termout = NULL;
 
 /* Original state of tty */
 
@@ -162,7 +137,7 @@ static struct ltchars oltarg;
 
 /* Output buffer, index and size */
 
-char *obuf = 0;
+char *obuf = NULL;
 int obufp = 0;
 int obufsiz;
 
@@ -200,11 +175,9 @@ static unsigned char havec;	/* Character read in during pending input check */
 int leave = 0;			/* When set, typeahead checking is disabled */
 
 /* TTY mode flag.  1 for open, 0 for closed */
-
 static int ttymode = 0;
 
 /* Signal state flag.  1 for joe, 0 for normal */
-
 static int ttysig = 0;
 
 /* Stuff for shell windows */
@@ -217,6 +190,7 @@ static int mpxsfd;		/* Clients send packets to this fd */
 
 static int nmpx = 0;
 static int accept = MAXINT;	/* =MAXINT if we have last packet */
+				/* FIXME: overloaded meaning of MAXINT */
 
 struct packet {
 	MPX *who;
@@ -227,53 +201,28 @@ struct packet {
 
 MPX asyncs[NPROC];
 
-/* Versions of 'read' and 'write' which automatically retry during signals
- * (yuck, yuck, yuck... we the #$%#$@ did they have to do this?) */
-
-int jread(int fd, void *buf, int siz)
-{
-	int rt;
-
-	do
-		rt = read(fd, buf, siz);
-	while (rt < 0 && errno == EINTR);
-	return rt;
-}
-
-int jwrite(int fd, void *buf, int siz)
-{
-	int rt;
-
-	do
-		rt = write(fd, buf, siz);
-	while (rt < 0 && errno == EINTR);
-	return rt;
-}
-
 /* Set signals for JOE */
-
 void sigjoe(void)
 {
 	if (ttysig)
 		return;
 	ttysig = 1;
-	signal(SIGHUP, ttsig);
-	signal(SIGTERM, ttsig);
-	signal(SIGINT, SIG_IGN);
-	signal(SIGPIPE, SIG_IGN);
+	joe_set_signal(SIGHUP, ttsig);
+	joe_set_signal(SIGTERM, ttsig);
+	joe_set_signal(SIGINT, SIG_IGN);
+	joe_set_signal(SIGPIPE, SIG_IGN);
 }
 
 /* Restore signals for exiting */
-
 void signrm(void)
 {
 	if (!ttysig)
 		return;
 	ttysig = 0;
-	signal(SIGHUP, SIG_DFL);
-	signal(SIGTERM, SIG_DFL);
-	signal(SIGINT, SIG_DFL);
-	signal(SIGPIPE, SIG_DFL);
+	joe_set_signal(SIGHUP, SIG_DFL);
+	joe_set_signal(SIGTERM, SIG_DFL);
+	joe_set_signal(SIGINT, SIG_DFL);
+	joe_set_signal(SIGPIPE, SIG_DFL);
 }
 
 /* Open terminal and set signals */
@@ -292,18 +241,15 @@ void ttclose(void)
 	signrm();
 }
 
-/* Window size interrupt handler */
-
 static int winched = 0;
-
+#ifdef SIGWINCH
+/* Window size interrupt handler */
 static RETSIGTYPE winchd(int unused)
 {
 	++winched;
-#ifdef SIGWINCH
-	signal(SIGWINCH, winchd);
-	siginterrupt(SIGWINCH, 1);
-#endif
+	REINSTALL_SIGHANDLER(SIGWINCH, winchd);
 }
+#endif
 
 /* Second ticker */
 
@@ -320,24 +266,10 @@ void tickoff(void)
 	alarm(0);
 }
 
-struct sigaction vnew;
-
 void tickon(void)
 {
 	ticked = 0;
-	vnew.sa_handler = dotick;
-
-#ifdef SA_INTERRUPT
-	vnew.sa_flags = SA_INTERRUPT;
-	sigaction(SIGALRM, &vnew, NULL);
-#else
-#ifdef SV_INTERRUPT
-	vnew.sa_flags = SV_INTERRUPT;
-	sigvec(SIGALRM, &vnew, NULL);
-#else
-	signal(SIGALRM, dotick);
-#endif
-#endif
+	joe_set_signal(SIGALRM, dotick);
 	alarm(1);
 }
 
@@ -365,8 +297,7 @@ void ttopnn(void)
 			exit(1);
 		} else {
 #ifdef SIGWINCH
-			signal(SIGWINCH, winchd);
-			siginterrupt(SIGWINCH, 1);
+			joe_set_signal(SIGWINCH, winchd);
 #endif
 			tickon();
 		}
@@ -412,8 +343,10 @@ void ttopnn(void)
 	otarg = targ;
 	oltarg = ltarg;
 	arg.sg_flags = ((arg.sg_flags & ~(ECHO | CRMOD | XTABS | ALLDELAY | TILDE)) | CBREAK);
-	if (noxon)
-		targ.t_startc = -1, targ.t_stopc = -1;
+	if (noxon) {
+		targ.t_startc = -1;
+		targ.t_stopc = -1;
+	}
 	targ.t_intrc = -1;
 	targ.t_quitc = -1;
 	targ.t_eofc = -1;
@@ -442,7 +375,7 @@ void ttopnn(void)
 		baud = Baud;
 	upc = DIVIDEND / baud;
 	if (obuf)
-		free(obuf);
+		joe_free(obuf);
 	if (!(TIMES * upc))
 		obufsiz = 4096;
 	else {
@@ -452,7 +385,7 @@ void ttopnn(void)
 	}
 	if (!obufsiz)
 		obufsiz = 1;
-	obuf = (char *) malloc(obufsiz);
+	obuf = (char *) joe_malloc(obufsiz);
 }
 
 /* Close terminal */
@@ -557,21 +490,21 @@ int ttflsh(void)
 			a.it_interval.tv_usec = 0;
 			a.it_interval.tv_sec = 0;
 			alarm(0);
-			signal(SIGALRM, dosig);
+			joe_set_signal(SIGALRM, dosig);
 			yep = 0;
 			maskit();
 			setitimer(ITIMER_REAL, &a, &b);
-			jwrite(fileno(termout), obuf, obufp);
+			joe_write(fileno(termout), obuf, obufp);
 			while (!yep)
 				pauseit();
 			unmaskit();
 			tickon();
 		} else
-			jwrite(fileno(termout), obuf, obufp);
+			joe_write(fileno(termout), obuf, obufp);
 
 #else
 
-		jwrite(fileno(termout), obuf, obufp);
+		joe_write(fileno(termout), obuf, obufp);
 
 #ifdef FIORDCHK
 		if (baud < 9600 && usec / 1000)
@@ -588,9 +521,9 @@ int ttflsh(void)
 		char c = 0;
 
 		if (pack.who && pack.who->func)
-			jwrite(pack.who->ackfd, &c, 1);
+			joe_write(pack.who->ackfd, &c, 1);
 		else
-			jwrite(ackkbd, &c, 1);
+			joe_write(ackkbd, &c, 1);
 		accept = MAXINT;
 	}
 
@@ -601,8 +534,9 @@ int ttflsh(void)
 			fcntl(mpxfd, F_SETFL, O_NDELAY);
 			if (read(mpxfd, &pack, sizeof(struct packet) - 1024) > 0) {
 				fcntl(mpxfd, F_SETFL, 0);
-				jread(mpxfd, pack.data, pack.size);
-				have = 1, accept = pack.ch;
+				joe_read(mpxfd, pack.data, pack.size);
+				have = 1;
+				accept = pack.ch;
 			} else
 				fcntl(mpxfd, F_SETFL, 0);
 		} else {
@@ -630,16 +564,22 @@ int ttgetc(void)
 
       loop:
 	ttflsh();
-	while (winched)
-		winched = 0, edupd(1), ttflsh();
-	if (ticked)
-		edupd(0), ttflsh(), tickon();
+	while (winched) {
+		winched = 0;
+		edupd(1);
+		ttflsh();
+	}
+	if (ticked) {
+		edupd(0);
+		ttflsh();
+		tickon();
+	}
 	if (ackkbd != -1) {
 		if (!have) {	/* Wait for input */
 			stat = read(mpxfd, &pack, sizeof(struct packet) - 1024);
 
 			if (pack.size && stat > 0) {
-				jread(mpxfd, pack.data, pack.size);
+				joe_read(mpxfd, pack.data, pack.size);
 			} else if (stat < 1) {
 				if (winched || ticked)
 					goto loop;
@@ -651,8 +591,10 @@ int ttgetc(void)
 		have = 0;
 		if (pack.who) {	/* Got bknd input */
 			if (accept != MAXINT) {
-				if (pack.who->func)
-					pack.who->func(pack.who->object, pack.data, pack.size), edupd(1);
+				if (pack.who->func) {
+					pack.who->func(pack.who->object, pack.data, pack.size);
+					edupd(1);
+				}
 			} else
 				mpxdied(pack.who);
 			goto loop;
@@ -781,15 +723,14 @@ static void mpxstart(void)
 			int sta;
 
 			pack.who = 0;
-			sta = jread(fileno(termin), &c, 1);
+			sta = joe_read(fileno(termin), &c, 1);
 			if (sta == 0)
 				pack.ch = MAXINT;
 			else
 				pack.ch = c;
 			pack.size = 0;
-			jwrite(mpxsfd, &pack, sizeof(struct packet) - 1024);
-		}
-		while (jread(fds[0], &pack, 1) == 1);
+			joe_write(mpxsfd, &pack, sizeof(struct packet) - 1024);
+		} while (joe_read(fds[0], &pack, 1) == 1);
 		_exit(0);
 	}
 	close(fds[0]);
@@ -799,7 +740,8 @@ static void mpxstart(void)
 static void mpxend(void)
 {
 	kill(kbdpid, 9);
-	while (wait(NULL) < 0 && errno == EINTR) ;
+	while (wait(NULL) < 0 && errno == EINTR)
+		/* do nothing */;
 	close(ackkbd);
 	ackkbd = -1;
 	close(mpxfd);
@@ -859,7 +801,7 @@ static char *getpty(int *ptyfd)
 {
 	int x, fd;
 	char *orgpwd = pwd();
-	static char **ptys = 0;
+	static char **ptys = NULL;
 	static char *ttydir;
 	static char *ptydir;
 	static char ttyname[32];
@@ -901,7 +843,7 @@ static char *getpty(int *ptyfd)
 					close(*ptyfd);
 			}
 		}
-	return 0;
+	return NULL;
 }
 
 #endif
@@ -919,8 +861,6 @@ static RETSIGTYPE death(int unused)
 #define SIGCHLD SIGCLD
 #endif
 
-struct sigaction inew;
-
 /* Build a new environment */
 
 extern char **mainenv;
@@ -931,7 +871,7 @@ static char **newenv(char **old, char *s)
 	int x, y, z;
 
 	for (x = 0; old[x]; ++x) ;
-	new = (char **) malloc((x + 2) * sizeof(char *));
+	new = (char **) joe_malloc((x + 2) * sizeof(char *));
 
 	for (x = 0, y = 0; old[x]; ++x) {
 		for (z = 0; s[z] != '='; ++z)
@@ -959,13 +899,13 @@ MPX *mpxmk(int *ptyfd, char *cmd, char **args, void (*func) (/* ??? */), void *o
 	char *name;
 
 	if (!(name = getpty(ptyfd)))
-		return 0;
+		return NULL;
 	for (x = 0; x != NPROC; ++x)
 		if (!asyncs[x].func) {
 			m = asyncs + x;
 			goto ok;
 		}
-	return 0;
+	return NULL;
       ok:
 	ttflsh();
 	++nmpx;
@@ -982,18 +922,7 @@ MPX *mpxmk(int *ptyfd, char *cmd, char **args, void (*func) (/* ??? */), void *o
 		close(fds[1]);
 		close(comm[0]);
 		dead = 0;
-		inew.sa_handler = death;
-#ifdef SA_INTERRUPT
-		inew.sa_flags = SA_INTERRUPT;
-		sigaction(SIGCHLD, &inew, (struct sigaction *) 0);
-#else
-#ifdef SV_INTERRUPT
-		inew.sa_flags = SV_INTERRUPT;
-		sigvec(SIGCHLD, &inew, (struct sigvec *) 0);
-#else
-		signal(SIGCHLD, death);
-#endif
-#endif
+		joe_set_signal(SIGCHLD, death);
 
 		if (!(pid = fork())) {
 			signrm();
@@ -1046,7 +975,7 @@ MPX *mpxmk(int *ptyfd, char *cmd, char **args, void (*func) (/* ??? */), void *o
 
 			_exit(0);
 		}
-		jwrite(comm[1], &pid, sizeof(pid));
+		joe_write(comm[1], &pid, sizeof(pid));
 
 	      loop:
 		pack.who = m;
@@ -1056,19 +985,19 @@ MPX *mpxmk(int *ptyfd, char *cmd, char **args, void (*func) (/* ??? */), void *o
 		else
 			pack.size = read(*ptyfd, pack.data, 1024);
 		if (pack.size > 0) {
-			jwrite(mpxsfd, &pack, sizeof(struct packet) - 1024 + pack.size);
+			joe_write(mpxsfd, &pack, sizeof(struct packet) - 1024 + pack.size);
 
-			jread(fds[0], &pack, 1);
+			joe_read(fds[0], &pack, 1);
 			goto loop;
 		} else {
 			pack.ch = MAXINT;
 			pack.size = 0;
-			jwrite(mpxsfd, &pack, sizeof(struct packet) - 1024);
+			joe_write(mpxsfd, &pack, sizeof(struct packet) - 1024);
 
 			_exit(0);
 		}
 	}
-	jread(comm[0], &m->pid, sizeof(m->pid));
+	joe_read(comm[0], &m->pid, sizeof(m->pid));
 
 	close(comm[0]);
 	close(comm[1]);
@@ -1080,9 +1009,10 @@ void mpxdied(MPX *m)
 {
 	if (!--nmpx)
 		mpxend();
-	while (wait(NULL) < 0 && errno == EINTR) ;
+	while (wait(NULL) < 0 && errno == EINTR)
+		/* do nothing */;
 	if (m->die)
 		m->die(m->dieobj);
-	m->func = 0;
+	m->func = NULL;
 	edupd(1);
 }
