@@ -1,27 +1,31 @@
 /*
-	Highlighted block functions
-	Copyright (C) 1992 Joseph H. Allen
-
-	This file is part of JOE (Joe's Own Editor)
-*/
-
+ * 	Highlighted block functions
+ *	Copyright
+ *		(C) 1992 Joseph H. Allen
+ *
+ *	This file is part of JOE (Joe's Own Editor)
+ */
 #include "config.h"
+#include "types.h"
 
 #include <unistd.h>
+#ifdef HAVE_SYS_TYPES_H
 #include <sys/types.h>
+#endif
 #ifdef HAVE_SYS_WAIT_H
 #include <sys/wait.h>
 #endif
+
 #include "b.h"
-#include "bw.h"
-#include "scrn.h"
-#include "w.h"
 #include "pw.h"
-#include "qw.h"
+#include "queue.h"
+#include "scrn.h"
+#include "tty.h"
+#include "ublock.h"
 #include "uedit.h"
 #include "utils.h"
 #include "vs.h"
-#include "ublock.h"
+#include "w.h"
 
 /* Global options */
 
@@ -33,8 +37,8 @@ extern int marking;
 
 /* Global variables */
 
-P *markb = 0;			/* Beginning and end of block */
-P *markk = 0;
+P *markb = NULL;		/* Beginning and end of block */
+P *markk = NULL;
 
 /* Push markb & markk */
 
@@ -42,11 +46,7 @@ typedef struct marksav MARKSAV;
 struct marksav {
 	LINK(MARKSAV) link;
 	P *markb, *markk;
-} markstack = {
-
-	{
-	&markstack, &markstack}
-};
+} markstack = { { &markstack, &markstack} };
 MARKSAV markfree = { {&markfree, &markfree} };
 int nstack = 0;
 
@@ -94,15 +94,7 @@ int markv(int r)
 {
 	if (markb && markk && markb->b == markk->b && markk->byte > markb->byte && (!square || markk->xcol > markb->xcol))
 		return 1;
-	else if (r && markb && markk && markb->b == markk->b && markk->byte < markb->byte && (!square || markk->xcol < markb->xcol)) {
-		P *t = markb;
-
-		markb = markk;
-		markk = t;
-		markb->owner = &markb;
-		markk->owner = &markk;
-		return 1;
-	} else
+	else
 		return 0;
 }
 
@@ -184,7 +176,7 @@ void pclrrect(P *org, long int height, long int right, int usetabs)
 }
 
 /* int ptabrect(P *org,long height,long right)
- * Check if there are any TABs in a rectange
+ * Check if there are any TABs in a rectangle
  */
 
 int ptabrect(P *org, long int height, long int right)
@@ -195,12 +187,13 @@ int ptabrect(P *org, long int height, long int right)
 		int c;
 
 		pcol(p, org->xcol);
-		while (c = pgetc(p), c != MAXINT && c != '\n')
+		while ((c = pgetc(p)) != NO_MORE_DATA && c != '\n') {
 			if (c == '\t') {
 				prm(p);
 				return 1;
 			} else if (piscol(p) > right)
 				break;
+		}
 		if (c != '\n')
 			pnextl(p);
 	}
@@ -227,9 +220,11 @@ void pinsrect(P *cur, B *tmp, long int width, int usetabs)
 				pfill(p, cur->xcol + width, usetabs);
 			if (piseol(p))
 				pbackws(p);
-			if (!pnextl(p))
-				binsc(p, '\n'), pgetc(p);
-			if (pgetc(q) == MAXINT)
+			if (!pnextl(p)) {
+				binsc(p, '\n');
+				pgetc(p);
+			}
+			if (pgetc(q) == NO_MORE_DATA)
 				break;
 		}
 	prm(p);
@@ -356,7 +351,7 @@ int ublkdel(BW *bw)
 		if (lightoff)
 			unmark(bw);
 	} else {
-		msgnw(bw, "No block");
+		msgnw(bw->parent, "No block");
 		return -1;
 	}
 	return 0;
@@ -393,7 +388,7 @@ int ublkmove(BW *bw)
 {
 	if (markv(1)) {
 		if (markb->b->rdonly) {
-			msgnw(bw, "Read only");
+			msgnw(bw->parent, "Read only");
 			return -1;
 		}
 		if (square) {
@@ -402,15 +397,18 @@ int ublkmove(BW *bw)
 			int usetabs = ptabrect(markb, height, markk->xcol);
 			long ocol = piscol(bw->cursor);
 			B *tmp = pextrect(markb, height, markk->xcol);
+			int update_xcol = (bw->cursor->xcol >= markk->xcol && bw->cursor->line >= markb->line && bw->cursor->line <= markk->line);
 
 			ublkdel(bw);
+			/* now we can't use markb and markk until we set them again */
+			/* ublkdel() frees them */
 			if (bw->o.overtype) {
 				/* If cursor was in block, blkdel moves it to left edge of block, so fix it
 				 * back to its original place here */
 				pcol(bw->cursor, ocol);
 				pfill(bw->cursor, ocol, 0);
 				pdelrect(bw->cursor, height, piscol(bw->cursor) + width);
-			} else if (bw->cursor->xcol >= markk->xcol && bw->cursor->line >= markb->line && bw->cursor->line <= markk->line)
+			} else if (update_xcol)
 				/* If cursor was to right of block, xcol was not properly updated */
 				bw->cursor->xcol -= width;
 			pinsrect(bw->cursor, tmp, width, usetabs);
@@ -441,7 +439,7 @@ int ublkmove(BW *bw)
 			return 0;
 		}
 	}
-	msgnw(bw, "No block");
+	msgnw(bw->parent, "No block");
 	return -1;
 }
 
@@ -449,7 +447,7 @@ int ublkmove(BW *bw)
 
 int ublkcpy(BW *bw)
 {
-	if (markv(1))
+	if (markv(1)) {
 		if (square) {
 			long height = markk->line - markb->line + 1;
 			long width = markk->xcol - markb->xcol;
@@ -483,8 +481,9 @@ int ublkcpy(BW *bw)
 			}
 			updall();
 			return 0;
+		}
 	} else {
-		msgnw(bw, "No block");
+		msgnw(bw->parent, "No block");
 		return -1;
 	}
 }
@@ -496,7 +495,7 @@ int dowrite(BW *bw, char *s, void *object, int *notify)
 {
 	if (notify)
 		*notify = 1;
-	if (markv(1))
+	if (markv(1)) {
 		if (square) {
 			int fl;
 			int ret = 0;
@@ -504,8 +503,10 @@ int dowrite(BW *bw, char *s, void *object, int *notify)
 					  markk->line - markb->line + 1,
 					  markk->xcol);
 
-			if ((fl = bsave(tmp->bof, s, tmp->eof->byte)) != 0)
-				msgnw(bw, msgs[5 + fl]), ret = -1;
+			if ((fl = bsave(tmp->bof, s, tmp->eof->byte)) != 0) {
+				msgnw(bw->parent, msgs[5 + fl]);
+				ret = -1;
+			}
 			brm(tmp);
 			if (lightoff)
 				unmark(bw);
@@ -515,15 +516,18 @@ int dowrite(BW *bw, char *s, void *object, int *notify)
 			int fl;
 			int ret = 0;
 
-			if ((fl = bsave(markb, s, markk->byte - markb->byte)) != 0)
-				msgnw(bw, msgs[5 + fl]), ret = -1;
+			if ((fl = bsave(markb, s, markk->byte - markb->byte)) != 0) {
+				msgnw(bw->parent, msgs[5 + fl]);
+				ret = -1;
+			}
 			if (lightoff)
 				unmark(bw);
 			vsrm(s);
 			return ret;
+		}
 	} else {
 		vsrm(s);
-		msgnw(bw, "No block");
+		msgnw(bw->parent, "No block");
 		return -1;
 	}
 }
@@ -542,12 +546,12 @@ void setindent(BW *bw)
 	q = pdup(p);
 	indent = pisindent(p);
 
-	do
+	do {
 		if (!pprevl(p))
 			goto done;
 		else
 			p_goto_bol(p);
-	while (pisindent(p) >= indent && !pisblank(p));
+	} while (pisindent(p) >= indent && !pisblank(p));
 	pnextl(p);
       done:
 	p_goto_bol(p);
@@ -557,10 +561,10 @@ void setindent(BW *bw)
 	markb = p;
 	p->owner = &markb;
 
-	do
+	do {
 		if (!pnextl(q))
 			break;
-	while (pisindent(q) >= indent && !pisblank(q)) ;
+	} while (pisindent(q) >= indent && !pisblank(q));
 
 	if (markk)
 		prm(markk);
@@ -582,8 +586,7 @@ int urindent(BW *bw)
 			do {
 				pcol(p, markb->xcol);
 				pfill(p, markb->xcol + bw->o.istep, bw->o.indentc == '\t' ? 1 : 0);
-			}
-			while (pnextl(p) && p->line <= markk->line);
+			} while (pnextl(p) && p->line <= markk->line);
 			prm(p);
 		}
 	} else {
@@ -595,8 +598,10 @@ int urindent(BW *bw)
 			while (p->byte < markk->byte) {
 				p_goto_bol(p);
 				if (!piseol(p))
-					while (piscol(p) < bw->o.istep)
-						binsc(p, bw->o.indentc), pgetc(p);
+					while (piscol(p) < bw->o.istep) {
+						binsc(p, bw->o.indentc);
+						pgetc(p);
+					}
 				pnextl(p);
 			}
 			prm(p);
@@ -625,16 +630,14 @@ int ulindent(BW *bw)
 						return -1;
 					}
 				}
-			}
-			while (pnextl(p) && p->line <= markk->line);
+			} while (pnextl(p) && p->line <= markk->line);
 			pset(p, markb);
 			do {
 				pcol(p, markb->xcol);
 				pset(q, p);
 				pcol(q, markb->xcol + bw->o.istep);
 				bdel(p, q);
-			}
-			while (pnextl(p) && p->line <= markk->line);
+			} while (pnextl(p) && p->line <= markk->line);
 			prm(p);
 			prm(q);
 		}
@@ -694,7 +697,7 @@ int doinsf(BW *bw, char *s, void *object, int *notify)
 
 			tmp = bload(s);
 			if (error) {
-				msgnw(bw, msgs[error + 5]);
+				msgnw(bw->parent, msgs[error + 5]);
 				brm(tmp);
 				return -1;
 			}
@@ -718,15 +721,16 @@ int doinsf(BW *bw, char *s, void *object, int *notify)
 			updall();
 			return 0;
 		} else {
-			msgnw(bw, "No block");
+			msgnw(bw->parent, "No block");
 			return -1;
 	} else {
 		int ret = 0;
 		B *tmp = bload(s);
 
-		if (error)
-			msgnw(bw, msgs[error + 5]), brm(tmp), ret = -1;
-		else
+		if (error) {
+			msgnw(bw->parent, msgs[error + 5]), brm(tmp);
+			ret = -1;
+		} else
 			binsb(bw->cursor, tmp);
 		vsrm(s);
 		bw->cursor->xcol = piscol(bw->cursor);
@@ -749,7 +753,7 @@ static int dofilt(BW *bw, char *s, void *object, int *notify)
 	if (markb && markk && !square && markb->b == bw->b && markk->b == bw->b && markb->byte == markk->byte)
 		goto ok;
 	if (!markv(1)) {
-		msgnw(bw, "No block");
+		msgnw(bw->parent, "No block");
 		return -1;
 	}
       ok:
@@ -840,7 +844,7 @@ static int dofilt(BW *bw, char *s, void *object, int *notify)
 	return 0;
 }
 
-static B *filthist = 0;
+static B *filthist = NULL;
 
 static void markall(BW *bw)
 {
@@ -857,7 +861,8 @@ static int checkmark(BW *bw)
 		if (square)
 			return 2;
 		else {
-			markall(bw), filtflg = 1;
+			markall(bw);
+			filtflg = 1;
 			return 1;
 	} else {
 		filtflg = 0;
@@ -868,25 +873,23 @@ static int checkmark(BW *bw)
 int ufilt(BW *bw)
 {
 #ifdef __MSDOS__
-	msgnw(bw, "Sorry, no sub-processes in DOS (yet)");
+	msgnw(bw->parent, "Sorry, no sub-processes in DOS (yet)");
 	return -1;
 #else
 	switch (checkmark(bw)) {
-		case 0:
+	case 0:
 		if (wmkpw(bw->parent, "Command to filter block through (^C to abort): ", &filthist, dofilt, NULL, NULL, utypebw, NULL, NULL))
 			return 0;
 		else
 			return -1;
-
-		case 1:
+	case 1:
 		if (wmkpw(bw->parent, "Command to filter file through (^C to abort): ", &filthist, dofilt, NULL, NULL, utypebw, NULL, NULL))
 			return 0;
 		else
 			return -1;
-
-		case 2:
-		default:
-		msgnw(bw, "No block");
+	case 2:
+	default:
+		msgnw(bw->parent, "No block");
 		return -1;
 	}
 #endif
