@@ -1,22 +1,23 @@
 /*
-	User text formatting functions
-	Copyright (C) 1992 Joseph H. Allen
-
-	This file is part of JOE (Joe's Own Editor)
-*/
-
+ *	User text formatting functions
+ *	Copyright
+ *		(C) 1992 Joseph H. Allen
+ *
+ *	This file is part of JOE (Joe's Own Editor)
+ */
 #include "config.h"
+#include "types.h"
 
 #include <ctype.h>
-#include <string.h>
 #ifdef HAVE_STDLIB_H
 #include <stdlib.h>
 #endif
+#ifdef HAVE_STRING_H
+#include <string.h>
+#endif
 
 #include "b.h"
-#include "bw.h"
 #include "ublock.h"
-#include "uformat.h"
 #include "utils.h"
 
 /* Center line cursor is on and move cursor to beginning of next line */
@@ -28,23 +29,25 @@ int ucenter(BW *bw)
 	int c;
 
 	p_goto_eol(p);
-	while (isblank(c = prgetc(p))) ;
+	while (isblank(c = prgetc(p)))
+		/* do nothing */;
 	if (c == '\n') {
 		pgetc(p);
 		goto done;
 	}
-	if (c == MAXINT)
+	if (c == NO_MORE_DATA)
 		goto done;
 	pgetc(p);
 	endcol = piscol(p);
 
 	p_goto_bol(p);
-	while (isblank(c = pgetc(p))) ;
+	while (isblank(c = pgetc(p)))
+		/* do nothing */;
 	if (c == '\n') {
 		prgetc(p);
 		goto done;
 	}
-	if (c == MAXINT)
+	if (c == NO_MORE_DATA)
 		goto done;
 	prgetc(p);
 	begcol = piscol(p);
@@ -74,8 +77,11 @@ int ucenter(BW *bw)
 static int cpara(int c)
 {
 	if (c == ' ' || c == '\t' || c == '\\' ||
-	    c == '>' || c == '|' || c == ':' || c == '*' || c == '/'
-	    || c == ',' || c == '.' || c == '?' || c == ';' || c == ']' || c == '}' || c == '=' || c == '+' || c == '-' || c == '_' || c == ')' || c == '&' || c == '^' || c == '%' || c == '$' || c == '#' || c == '@' || c == '!' || c == '~')
+	    c == '>' || c == '|' || c == ':' || c == '*' || c == '/' ||
+	    c == ',' || c == '.' || c == '?' || c == ';' || c == ']' ||
+	    c == '}' || c == '=' || c == '+' || c == '-' || c == '_' ||
+	    c == ')' || c == '&' || c == '^' || c == '%' || c == '$' ||
+	    c == '#' || c == '@' || c == '!' || c == '~')
 		return 1;
 	else
 		return 0;
@@ -94,7 +100,8 @@ static int pisnpara(P *p)
 
 	q = pdup(p);
 	p_goto_bol(q);
-	while (cpara(c = pgetc(q))) ;
+	while (cpara(c = pgetc(q)))
+		/* do nothing */;
 	prm(q);
 	if (c == '.' || c == '\r' || c == '\n')
 		return 1;
@@ -110,9 +117,9 @@ static long nindent(P *p)
 	long col;
 
 	p_goto_bol(q);
-	do
+	do {
 		col = q->col;
-	while (cpara(pgetc(q)));
+	} while (cpara(pgetc(q)));
 	prm(q);
 	return col;
 }
@@ -225,7 +232,8 @@ int ubop(BW *bw)
 {
 	P *q = pdup(bw->cursor);
 
-      up:while (pisnpara(q) && !pisbof(q) && (!within || !markb || q->byte > markb->byte))
+      up:
+	while (pisnpara(q) && !pisbof(q) && (!within || !markb || q->byte > markb->byte))
 		pprevl(q);
 	pbop(q);
 	if (q->byte != bw->cursor->byte) {
@@ -245,7 +253,8 @@ int ueop(BW *bw)
 {
 	P *q = pdup(bw->cursor);
 
-      up:while (pisnpara(q) && !piseof(q))
+      up:
+	while (pisnpara(q) && !piseof(q))
 		pnextl(q);
 	pbop(q);
 	peop(q);
@@ -275,28 +284,31 @@ void wrapword(P *p, long int indent, int french, char *indents)
 
 	/* Get indentation prefix from beginning of line */
 /*
- if(!indents)
-  {
-  int f=0;
-  P *r=pdup(p);
-  p_goto_bol(r);
-  q=pdup(r);
-  while(cpara(c=brc(q)))
-   {
-   if(!isblank(c)) f=1;
-   pgetc(q);
-   }
-  if(f)
-   {
-   indents=brs(r,q->byte-r->byte); rmf=1;
-   if(indents[0]=='/' && indents[1]=='*') indents[0]=' ';
-   }
-  prm(r); prm(q);
-  }
+	if(!indents) {
+		int f = 0;
+		P *r = pdup(p);
+
+		p_goto_bol(r);
+		q = pdup(r);
+		while(cpara(c = brc(q))) {
+			if(!isblank(c))
+				f = 1;
+			pgetc(q);
+		}
+		if(f) {
+			indents = brs(r, q->byte-r->byte);
+			rmf = 1;
+			if(indents[0] == '/' && indents[1] == '*')
+				indents[0] = ' ';
+		}
+		prm(r);
+		prm(q);
+	}
 */
 
 	/* Get to beginning of word */
-	while (!pisbol(p) && piscol(p) > indent && !isblank(prgetc(p))) ;
+	while (!pisbol(p) && piscol(p) > indent && !isblank(prgetc(p)))
+		/* do nothing */;
 
 	/* If we found the beginning of a word... */
 	if (!pisbol(p) && piscol(p) > indent) {
@@ -319,20 +331,24 @@ void wrapword(P *p, long int indent, int french, char *indents)
 		prm(q);
 
 		/* Move word to beginning of next line */
-		binsc(p, '\n'), ++to;
+		binsc(p, '\n');
+		++to;
 		if (p->b->o.crlf)
 			++to;
 		pgetc(p);
 
 		/* Indent to left margin */
-		if (indents)
-			binss(p, indents), to += strlen(indents);
-		else
-			while (indent--)
-				binsc(p, ' '), ++to;
+		if (indents) {
+			binss(p, indents);
+			to += strlen(indents);
+		} else
+			while (indent--) {
+				binsc(p, ' ');
+				++to;
+			}
 
 		if (rmf)
-			free(indents);
+			joe_free(indents);
 	}
 
 	/* Move cursor back to original position */
@@ -397,7 +413,7 @@ int uformat(BW *bw)
 		indent = bw->o.lmargin;
 
 	/* Cut paragraph into memory buffer */
-	buf = (char *) malloc(len = (bw->cursor->byte - p->byte));
+	buf = (char *) joe_malloc(len = (bw->cursor->byte - p->byte));
 	brmem(p, buf, len);
 	bdel(p, bw->cursor);
 
@@ -454,7 +470,7 @@ int uformat(BW *bw)
 	/* Do rest */
 
 	while (len > 0)
-		if (isspace(*b) || *b == '\r') {
+		if (isblank(*b) || *b == '\n' || *b == '\r') {
 			int f = 0;
 
 			/* Set f if there are two spaces after . ? or ! instead of one */
@@ -468,24 +484,28 @@ int uformat(BW *bw)
 			if (*b == '\r' && len) {
 				if (b - buf == curoff)
 					pset(bw->cursor, p);
-				++b, --len;
+				++b;
+				--len;
 			}
 
 			if (*b == '\n' && len) {
 				if (b - buf == curoff)
 					pset(bw->cursor, p);
-				++b, --len;
+				++b;
+				--len;
 				while (cpara(*b) && len) {
 					if (b - buf == curoff)
 						pset(bw->cursor, p);
-					++b, --len;
+					++b;
+					--len;
 				}
 			}
 
 			if (len && isblank(*b)) {
 				if (b - buf == curoff)
 					pset(bw->cursor, p);
-				++b, --len;
+				++b;
+				--len;
 				goto loop;
 			}
 
@@ -509,8 +529,8 @@ int uformat(BW *bw)
 
 	binsc(p, '\n');
 	prm(p);
-	free(buf);
-	free(indents);
+	joe_free(buf);
+	joe_free(indents);
 	return 0;
 }
 
@@ -524,9 +544,9 @@ int ufmtblk(BW *bw)
 		markk->end = 1;
 		utomarkk(bw);
 		within = 1;
-		do
+		do {
 			ubop(bw), uformat(bw);
-		while (bw->cursor->byte > markb->byte);
+		} while (bw->cursor->byte > markb->byte);
 		within = 0;
 		markk->end = 0;
 		if (lightoff)
