@@ -1,28 +1,38 @@
-/* Search & Replace system */
-
+/*
+ *	Search & Replace system
+ *	Copyright
+ *		(C) 1992 Joseph H. Allen
+ *
+ *	This file is part of JOE (Joe's Own Editor)
+ */
 #include "config.h"
+#include "types.h"
+
 #include <ctype.h>
 #include <stdio.h>
-#ifdef HAVE_STDLIB_H
-#include <stdlib.h>
-#endif
+
+#include "b.h"
+#include "bw.h"
+#include "main.h"
 #include "pw.h"
+#include "queue.h"
 #include "qw.h"
-#include "vs.h"
 #include "regex.h"
 #include "ublock.h"
 #include "uedit.h"
-#include "main.h"
 #include "undo.h"
 #include "usearch.h"
+#include "utils.h"
+#include "vs.h"
+#include "w.h"
 
 int smode = 0;			/* Decremented to zero by execmd */
 int csmode = 0;			/* Set for continued search mode */
 
-B *findhist = 0;		/* Search string history */
-B *replhist = 0;		/* Replacement string history */
+B *findhist = NULL;		/* Search string history */
+B *replhist = NULL;		/* Replacement string history */
 
-SRCH *globalsrch = 0;		/* Most recent completed search data */
+SRCH *globalsrch = NULL;	/* Most recent completed search data */
 
 SRCHREC fsr = { {&fsr, &fsr} };
 
@@ -60,12 +70,12 @@ static P *searchf(SRCH *srch, P *p)
 			prm(end);
 			return p;
 		}
-		if (pgetc(start) == MAXINT)
+		if (pgetc(start) == NO_MORE_DATA)
 			break;
 	}
 	prm(start);
 	prm(end);
-	return 0;
+	return NULL;
 }
 
 /* Search backwards.
@@ -107,7 +117,7 @@ static P *searchb(SRCH *srch, P *p)
 	}
 	prm(start);
 	prm(end);
-	return 0;
+	return NULL;
 }
 
 /* Make a search stucture */
@@ -120,19 +130,19 @@ static SRCH *setmark(SRCH *srch)
 	srch->markb = markb;
 	if (srch->markb)
 		srch->markb->owner = &srch->markb;
-	markb = 0;
+	markb = NULL;
 
 	srch->markk = markk;
 	if (srch->markk)
 		srch->markk->owner = &srch->markk;
-	markk = 0;
+	markk = NULL;
 
 	return srch;
 }
 
 SRCH *mksrch(char *pattern, char *replacement, int ignore, int backwards, int repeat, int replace, int rest)
 {
-	SRCH *srch = (SRCH *) malloc(sizeof(SRCH));
+	SRCH *srch = (SRCH *) joe_malloc(sizeof(SRCH));
 	int x;
 
 	srch->pattern = pattern;
@@ -142,16 +152,16 @@ SRCH *mksrch(char *pattern, char *replacement, int ignore, int backwards, int re
 	srch->repeat = repeat;
 	srch->replace = replace;
 	srch->rest = rest;
-	srch->entire = 0;
+	srch->entire = NULL;
 	srch->flg = 0;
 	srch->addr = -1;
-	srch->markb = 0;
-	srch->markk = 0;
+	srch->markb = NULL;
+	srch->markk = NULL;
 	srch->valid = 0;
-	srch->restrict = 0;
+	srch->block_restrict = 0;
 	izque(SRCHREC, link, &srch->recs);
 	for (x = 0; x != 26; ++x)
-		srch->pieces[x] = 0;
+		srch->pieces[x] = NULL;
 	return srch;
 }
 
@@ -179,7 +189,7 @@ void rmsrch(SRCH *srch)
 	vsrm(srch->pattern);
 	vsrm(srch->replacement);
 	vsrm(srch->entire);
-	free(srch);
+	joe_free(srch);
 	updall();
 }
 
@@ -199,11 +209,13 @@ static P *insert(SRCH *srch, P *p, char *s, int len)
 			len -= x;
 			s += x;
 		} else if (len >= 2) {
-			if (s[1] == '\\')
-				binsc(p, '\\'), pgetc(p);
-			else if (s[1] == 'n')
-				binsc(p, '\n'), pgetc(p);
-			else if (((s[1] >= 'a' && s[1] <= 'z') || (s[1] >= 'A' && s[1] <= 'Z'))
+			if (s[1] == '\\') {
+				binsc(p, '\\');
+				pgetc(p);
+			} else if (s[1] == 'n') {
+				binsc(p, '\n');
+				pgetc(p);
+			} else if (((s[1] >= 'a' && s[1] <= 'z') || (s[1] >= 'A' && s[1] <= 'Z'))
 				 && srch->pieces[(s[1] & 0x1f) - 1]) {
 				binsm(p, sv(srch->pieces[(s[1] & 0x1f) - 1]));
 				pfwrd(p, (long) sLEN(srch->pieces[(s[1] & 0x1f) - 1]));
@@ -258,8 +270,8 @@ static int pfsave(BW *bw, SRCH *srch)
 			markk->owner = &markk;
 			markk->xcol = piscol(markk);
 		}
-		srch->markb = 0;
-		srch->markk = 0;
+		srch->markb = NULL;
+		srch->markk = NULL;
 
 		updall();
 	}
@@ -276,39 +288,40 @@ static int set_options(BW *bw, char *s, SRCH *srch, int *notify)
 {
 	int x;
 
-	for (x = 0; s[x]; ++x)
+	for (x = 0; s[x]; ++x) {
 		switch (s[x]) {
-			case 'r':
-			case 'R':
+		case 'r':
+		case 'R':
 			srch->replace = 1;
 			break;
-			case 'b':
-			case 'B':
+		case 'b':
+		case 'B':
 			srch->backwards = 1;
 			break;
-			case 'i':
-			case 'I':
+		case 'i':
+		case 'I':
 			srch->ignore = 1;
 			break;
-			case 'k':
-			case 'K':
-			srch->restrict = 1;
+		case 'k':
+		case 'K':
+			srch->block_restrict = 1;
 			break;
-			case '0':
-			case '1':
-			case '2':
-			case '3':
-			case '4':
-			case '5':
-			case '6':
-			case '7':
-			case '8':
-			case '9':
+		case '0':
+		case '1':
+		case '2':
+		case '3':
+		case '4':
+		case '5':
+		case '6':
+		case '7':
+		case '8':
+		case '9':
 			if (srch->repeat == -1)
 				srch->repeat = 0;
 			srch->repeat = srch->repeat * 10 + s[x] - '0';
 			break;
 		}
+	}
 	vsrm(s);
 	if (srch->replace) {
 		if (wmkpw(bw->parent, "Replace with (^C to abort): ", &replhist, set_replace, srchstr, pfabort, utypebw, srch, notify))
@@ -335,7 +348,7 @@ static int set_pattern(BW *bw, char *s, SRCH *srch, int *notify)
 		if (srch->backwards)
 			binsc(pbw->cursor, 'b');
 		if (srch->repeat >= 0)
-			snprintf(buf, MSGBUFSIZE, "%d", srch->repeat), binss(pbw->cursor, buf);
+			snprintf(buf, JOE_MSGBUFSIZE, "%d", srch->repeat), binss(pbw->cursor, buf);
 		pset(pbw->cursor, pbw->b->eof);
 		pbw->cursor->xcol = piscol(pbw->cursor);
 		srch->ignore = 0;
@@ -366,7 +379,7 @@ static int dofirst(BW *bw, int back, int repl)
 		p_goto_bol(bw->cursor);
 		if (byte == bw->cursor->byte)
 			prgetc(bw->cursor);
-		return urtn(bw, MAXINT);
+		return urtn((BASE *)bw, MAXINT);
 	}
 	srch = setmark(mksrch(NULL, NULL, 0, back, -1, repl, 0));
 	srch->addr = bw->cursor->byte;
@@ -400,7 +413,7 @@ static int doreplace(BW *bw, SRCH *srch)
 	P *q;
 
 	if (bw->b->rdonly) {
-		msgnw(bw, "Read only");
+		msgnw(bw->parent, "Read only");
 		return -1;
 	}
 	if (markk)
@@ -469,14 +482,14 @@ static int dopfrepl(BW *bw, int c, SRCH *srch, int *notify)
 		goback(srch, bw);
 		goback(srch, bw);
 		return dopfnext(bw, srch, notify);
-	} else if (c != MAXINT) {
+	} else if (c != MAXINT) {	/* FIXME: what's the meaning of MAXINT here? */
 		if (notify)
 			*notify = 1;
 		pfsave(bw, srch);
 		nungetc(c);
 		return 0;
 	}
-	if (mkqwnsr(bw, sc("Replace (Y)es (N)o (R)est (B)ackup (^C to abort)?"), dopfrepl, pfsave, srch, notify))
+	if (mkqwnsr(bw->parent, sc("Replace (Y)es (N)o (R)est (B)ackup (^C to abort)?"), dopfrepl, pfsave, srch, notify))
 		return 0;
 	else
 		return pfsave(bw, srch);
@@ -488,9 +501,9 @@ static int dopfrepl(BW *bw, int c, SRCH *srch, int *notify)
  * 1 if we're done
  */
 
-static int restrict(BW *bw, SRCH *srch)
+static int restrict_to_block(BW *bw, SRCH *srch)
 {
-	if (!srch->valid || !srch->restrict)
+	if (!srch->valid || !srch->block_restrict)
 		return 0;
 	bw->cursor->xcol = piscol(bw->cursor);
 	if (srch->backwards)
@@ -548,10 +561,10 @@ static int fnext(BW *bw, SRCH *srch)
 		return 1;
 	} else if (srch->rest || (srch->repeat != -1 && srch->replace)) {
 		if (srch->valid)
-			switch (restrict(bw, srch)) {
-				case -1:
+			switch (restrict_to_block(bw, srch)) {
+			case -1:
 				goto again;
-				case 1:
+			case 1:
 				if (srch->addr >= 0)
 					pgoto(bw->cursor, srch->addr);
 				return !srch->rest;
@@ -561,10 +574,10 @@ static int fnext(BW *bw, SRCH *srch)
 		goto next;
 	} else if (srch->repeat != -1) {
 		if (srch->valid)
-			switch (restrict(bw, srch)) {
-				case -1:
+			switch (restrict_to_block(bw, srch)) {
+			case -1:
 				goto again;
-				case 1:
+			case 1:
 				if (srch->addr >= 0)
 					pgoto(bw->cursor, srch->addr);
 				return 1;
@@ -585,26 +598,24 @@ int dopfnext(BW *bw, SRCH *srch, int *notify)
 		smode = 2;	/* We have started a search mode */
 	if (srch->replace)
 		visit(srch, bw, 0);
-      again:switch (fnext(bw, srch)) {
-		case 0:
+again:	switch (fnext(bw, srch)) {
+	case 0:
 		break;
-
-		case 1:
-	      bye:if (!srch->flg && !srch->rest) {
-			if (srch->valid && srch->restrict)
-				msgnw(bw, "Not found (search restricted to marked block)");
+	case 1:
+bye:		if (!srch->flg && !srch->rest) {
+			if (srch->valid && srch->block_restrict)
+				msgnw(bw->parent, "Not found (search restricted to marked block)");
 			else
-				msgnw(bw, "Not found");
+				msgnw(bw->parent, "Not found");
 			ret = -1;
 		}
 		break;
-
-		case 2:
+	case 2:
 		if (srch->valid)
-			switch (restrict(bw, srch)) {
-				case -1:
+			switch (restrict_to_block(bw, srch)) {
+			case -1:
 				goto again;
-				case 1:
+			case 1:
 				if (srch->addr >= 0)
 					pgoto(bw->cursor, srch->addr);
 				goto bye;
