@@ -1,42 +1,39 @@
-/* Software virtual memory system
-   Copyright (C) 1992 Joseph H. Allen
-
-This file is part of JOE (Joe's Own Editor)
-
-JOE is free software; you can redistribute it and/or modify it under the 
-terms of the GNU General Public License as published by the Free Software 
-Foundation; either version 1, or (at your option) any later version.  
-
-JOE is distributed in the hope that it will be useful, but WITHOUT ANY 
-WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS 
-FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more 
-details.  
-
-You should have received a copy of the GNU General Public License along with 
-JOE; see the file COPYING.  If not, write to the Free Software Foundation, 
-675 Mass Ave, Cambridge, MA 02139, USA.  */
-
-#include <stdio.h>
-#include <sys/types.h>
-#include <sys/stat.h>
-#include <fcntl.h>
-#include <stdlib.h>
-#include <unistd.h>
+/*
+ *	Software virtual memory system
+ *	Copyright
+ *		(C) 1992 Joseph H. Allen
+ *
+ *	This file is part of JOE (Joe's Own Editor)
+ */
 #include "config.h"
-#include "vs.h"
+#include "types.h"
+
+#ifdef HAVE_SYS_TYPES_H
+#include <sys/types.h>
+#endif
+#ifdef HAVE_SYS_STAT_H
+#include <sys/stat.h>
+#endif
+#include <fcntl.h>
+#ifdef HAVE_STDLIB_H
+#include <stdlib.h>
+#endif
+#include <unistd.h>
+
 #include "blocks.h"
 #include "queue.h"
 #include "path.h"
-#include "tty.h"
+#include "utils.h"
 #include "vfile.h"
+#include "vs.h"
 
 static VFILE vfiles = { {&vfiles, &vfiles} };	/* Known vfiles */
-static VPAGE *freepages = 0;	/* Linked list of free pages */
+static VPAGE *freepages = NULL;	/* Linked list of free pages */
 static VPAGE *htab[HTSIZE];	/* Hash table of page headers */
 static long curvalloc = 0;	/* Amount of memory in use */
 static long maxvalloc = ILIMIT;	/* Maximum allowed */
 char *vbase;			/* Data first entry in vheader refers to */
-VPAGE **vheaders = 0;		/* Array of header addresses */
+VPAGE **vheaders = NULL;	/* Array of header addresses */
 static int vheadsz = 0;		/* No. entries allocated to vheaders */
 
 void vflsh(void)
@@ -52,11 +49,13 @@ void vflsh(void)
 		last = -1;
 	      loop:
 		addr = MAXLONG;
-		vlowest = 0;
+		vlowest = NULL;
 		for (x = 0; x != HTSIZE; x++)
 			for (vp = htab[x]; vp; vp = vp->next)
-				if (vp->addr < addr && vp->addr > last && vp->vfile == vfile && (vp->addr >= vfile->size || (vp->dirty && !vp->count)))
-					addr = vp->addr, vlowest = vp;
+				if (vp->addr < addr && vp->addr > last && vp->vfile == vfile && (vp->addr >= vfile->size || (vp->dirty && !vp->count))) {
+					addr = vp->addr;
+					vlowest = vp;
+				}
 		if (vlowest) {
 			if (!vfile->name)
 				vfile->name = mktmp(NULL);
@@ -64,10 +63,10 @@ void vflsh(void)
 				vfile->fd = open(vfile->name, O_RDWR);
 			lseek(vfile->fd, addr, 0);
 			if (addr + PGSIZE > vsize(vfile)) {
-				jwrite(vfile->fd, vlowest->data, (int) (vsize(vfile) - addr));
+				joe_write(vfile->fd, vlowest->data, (int) (vsize(vfile) - addr));
 				vfile->size = vsize(vfile);
 			} else {
-				jwrite(vfile->fd, vlowest->data, PGSIZE);
+				joe_write(vfile->fd, vlowest->data, PGSIZE);
 				if (addr + PGSIZE > vfile->size)
 					vfile->size = addr + PGSIZE;
 			}
@@ -87,11 +86,13 @@ void vflshf(VFILE *vfile)
 
       loop:
 	addr = MAXLONG;
-	vlowest = 0;
+	vlowest = NULL;
 	for (x = 0; x != HTSIZE; x++)
 		for (vp = htab[x]; vp; vp = vp->next)
-			if (vp->addr < addr && vp->dirty && vp->vfile == vfile && !vp->count)
-				addr = vp->addr, vlowest = vp;
+			if (vp->addr < addr && vp->dirty && vp->vfile == vfile && !vp->count) {
+				addr = vp->addr;
+				vlowest = vp;
+			}
 	if (vlowest) {
 		if (!vfile->name)
 			vfile->name = mktmp(NULL);
@@ -100,10 +101,10 @@ void vflshf(VFILE *vfile)
 		}
 		lseek(vfile->fd, addr, 0);
 		if (addr + PGSIZE > vsize(vfile)) {
-			jwrite(vfile->fd, vlowest->data, (int) (vsize(vfile) - addr));
+			joe_write(vfile->fd, vlowest->data, (int) (vsize(vfile) - addr));
 			vfile->size = vsize(vfile);
 		} else {
-			jwrite(vfile->fd, vlowest->data, PGSIZE);
+			joe_write(vfile->fd, vlowest->data, PGSIZE);
 			if (addr + PGSIZE > vfile->size)
 				vfile->size = addr + PGSIZE;
 		}
@@ -114,12 +115,12 @@ void vflshf(VFILE *vfile)
 
 static char *mema(int align, int size)
 {
-	char *z = (char *) malloc(align + size);
+	char *z = (char *) joe_malloc(align + size);
 
 	return z + align - physical(z) % align;
 }
 
-char *vlock(VFILE *vfile, long int addr)
+char *vlock(VFILE *vfile, unsigned long addr)
 {
 	VPAGE *vp, *pp;
 	int x, y;
@@ -127,9 +128,11 @@ char *vlock(VFILE *vfile, long int addr)
 
 	addr -= ofst;
 
-	for (vp = htab[((addr >> LPGSIZE) + (int) vfile) & (HTSIZE - 1)]; vp; vp = vp->next)
-		if (vp->vfile == vfile && vp->addr == addr)
-			return ++vp->count, vp->data + ofst;
+	for (vp = htab[((addr >> LPGSIZE) + (unsigned long) vfile) & (HTSIZE - 1)]; vp; vp = vp->next)
+		if (vp->vfile == vfile && vp->addr == addr) {
+			++vp->count;
+			return vp->data + ofst;
+		}
 
 	if (freepages) {
 		vp = freepages;
@@ -138,30 +141,28 @@ char *vlock(VFILE *vfile, long int addr)
 	}
 
 	if (curvalloc + PGSIZE <= maxvalloc) {
-		vp = (VPAGE *) malloc(sizeof(VPAGE) * INC);
+		vp = (VPAGE *) joe_malloc(sizeof(VPAGE) * INC);
 		if (vp) {
 			vp->data = (char *) mema(PGSIZE, PGSIZE * INC);
 			if (vp->data) {
 				int q;
 
 				curvalloc += PGSIZE * INC;
-				if (!vheaders)
-					vheaders = (VPAGE **)
-					    malloc((vheadsz = INC)
-						   * sizeof(VPAGE *)), vbase = vp->data;
-				else if (physical(vp->data) < physical(vbase)) {
+				if (!vheaders) {
+					vheaders = (VPAGE **) joe_malloc((vheadsz = INC) * sizeof(VPAGE *));
+					vbase = vp->data;
+				} else if (physical(vp->data) < physical(vbase)) {
 					VPAGE **t = vheaders;
 					int amnt = (physical(vbase) - physical(vp->data)) >> LPGSIZE;
 
-					vheaders = (VPAGE **)
-					    malloc((amnt + vheadsz) * sizeof(VPAGE *));
-					mcpy(vheaders + amnt, t, vheadsz * sizeof(VPAGE *));
+					vheaders = (VPAGE **) joe_malloc((amnt + vheadsz) * sizeof(VPAGE *));
+					mmove(vheaders + amnt, t, vheadsz * sizeof(VPAGE *));
 					vheadsz += amnt;
 					vbase = vp->data;
-					free(t);
+					joe_free(t);
 				} else if (((physical(vp->data + PGSIZE * INC) - physical(vbase)) >> LPGSIZE) > vheadsz) {
 					vheaders = (VPAGE **)
-					    realloc(vheaders, (vheadsz = (((physical(vp->data + PGSIZE * INC) - physical(vbase)) >> LPGSIZE))) * sizeof(VPAGE *));
+					    joe_realloc(vheaders, (vheadsz = (((physical(vp->data + PGSIZE * INC) - physical(vbase)) >> LPGSIZE))) * sizeof(VPAGE *));
 				}
 				for (q = 1; q != INC; ++q) {
 					vp[q].next = freepages;
@@ -172,8 +173,8 @@ char *vlock(VFILE *vfile, long int addr)
 				vheader(vp->data) = vp;
 				goto gotit;
 			}
-			free(vp);
-			vp = 0;
+			joe_free(vp);
+			vp = NULL;
 		}
 	}
 
@@ -198,8 +199,8 @@ char *vlock(VFILE *vfile, long int addr)
 	vp->vfile = vfile;
 	vp->dirty = 0;
 	vp->count = 1;
-	vp->next = htab[((addr >> LPGSIZE) + (int) vfile) & (HTSIZE - 1)];
-	htab[((addr >> LPGSIZE) + (int) vfile) & (HTSIZE - 1)] = vp;
+	vp->next = htab[((addr >> LPGSIZE) + (unsigned long)vfile) & (HTSIZE - 1)];
+	htab[((addr >> LPGSIZE) + (unsigned long)vfile) & (HTSIZE - 1)] = vp;
 
 	if (addr < vfile->size) {
 		if (!vfile->fd) {
@@ -207,10 +208,10 @@ char *vlock(VFILE *vfile, long int addr)
 		}
 		lseek(vfile->fd, addr, 0);
 		if (addr + PGSIZE > vfile->size) {
-			jread(vfile->fd, vp->data, (int) (vfile->size - addr));
+			joe_read(vfile->fd, vp->data, (int) (vfile->size - addr));
 			mset(vp->data + vfile->size - addr, 0, PGSIZE - (int) (vfile->size - addr));
 		} else
-			jread(vfile->fd, vp->data, PGSIZE);
+			joe_read(vfile->fd, vp->data, PGSIZE);
 	} else
 		mset(vp->data, 0, PGSIZE);
 
@@ -219,17 +220,17 @@ char *vlock(VFILE *vfile, long int addr)
 
 VFILE *vtmp(void)
 {
-	VFILE *new = (VFILE *) malloc(sizeof(VFILE));
+	VFILE *new = (VFILE *) joe_malloc(sizeof(VFILE));
 
 	new->fd = 0;
-	new->name = 0;
+	new->name = NULL;
 	new->alloc = 0;
 	new->size = 0;
 	new->left = 0;
 	new->lv = 0;
-	new->vpage = 0;
+	new->vpage = NULL;
 	new->flags = 1;
-	new->vpage1 = 0;
+	new->vpage1 = NULL;
 	new->addr = -1;
 	return enqueb_f(VFILE, link, &vfiles, new);
 }
@@ -240,23 +241,23 @@ VFILE *vopen(name)
 char *name;
 {
 	struct stat buf;
-	VFILE *new = (VFILE *) malloc(sizeof(VFILE));
+	VFILE *new = (VFILE *) joe_malloc(sizeof(VFILE));
 
 	new->name = vsncpy(NULL, 0, sz(name));
 	new->fd = open(name, O_RDWR);
 	if (!new->fd) {
 		fprintf(stderr, "Couldn\'t open file \'%s\'\n", name);
-		free(new);
-		return 0;
+		joe_free(new);
+		return NULL;
 	}
 	fstat(new->fd, &buf);
 	new->size = buf.st_size;
 	new->alloc = new->size;
 	new->left = 0;
 	new->lv = 0;
-	new->vpage = 0;
+	new->vpage = NULL;
 	new->flags = 0;
-	new->vpage1 = 0;
+	new->vpage1 = NULL;
 	new->addr = -1;
 	return enqueb_f(VFILE, link, &vfiles, new);
 }
@@ -281,7 +282,7 @@ void vclose(VFILE *vfile)
 	}
 	if (vfile->fd)
 		close(vfile->fd);
-	free(deque_f(VFILE, link, vfile));
+	joe_free(deque_f(VFILE, link, vfile));
 	for (x = 0; x != HTSIZE; x++)
 		for (pp = (VPAGE *) (htab + x), vp = pp->next; vp;)
 			if (vp->vfile == vfile) {
@@ -289,8 +290,10 @@ void vclose(VFILE *vfile)
 				vp->next = freepages;
 				freepages = vp;
 				vp = pp->next;
-			} else
-				pp = vp, vp = vp->next;
+			} else {
+				pp = vp;
+				vp = vp->next;
+			}
 }
 
 #ifdef junk
@@ -306,8 +309,8 @@ long amount;
 		if (freepages) {
 			vp = freepages;
 			freepages = vp->next;
-			free(vp->data);
-			free(vp);
+			joe_free(vp->data);
+			joe_free(vp);
 			curvalloc -= PGSIZE;
 		} else {
 		      again:
@@ -315,8 +318,8 @@ long amount;
 				for (pp = (VPAGE *) (htab + x), vp = pp->next; vp; pp = vp, vp = vp->next)
 					if (!vp->count && !vp->dirty) {
 						pp->next = vp->next;
-						free(vp->data);
-						free(vp);
+						joe_free(vp->data);
+						joe_free(vp);
 						if ((curvalloc -= PGSIZE)
 						    <= maxvalloc)
 							return;
@@ -328,8 +331,8 @@ long amount;
 				for (pp = (VPAGE *) (htab + x), vp = pp->next; vp; pp = vp, vp = vp->next)
 					if (!vp->count && !vp->dirty) {
 						pp->next = vp->next;
-						free(vp->data);
-						free(vp);
+						joe_free(vp->data);
+						joe_free(vp);
 						if ((curvalloc -= PGSIZE)
 						    <= maxvalloc)
 							return;
@@ -381,7 +384,7 @@ int _vrgetc(vfile)
 VFILE *vfile;
 {
 	if (vtell(vfile) == 0)
-		return MAXINT;
+		return NO_MORE_DATA;
 	vseek(vfile, vtell(vfile) - 1);
 	++vfile->bufp;
 	--vfile->left;
@@ -392,7 +395,7 @@ int _vgetc(vfile)
 VFILE *vfile;
 {
 	if (vtell(vfile) == vsize(vfile))
-		return MAXINT;
+		return NO_MORE_DATA;
 	vseek(vfile, vtell(vfile));
 	return vgetc(vfile);
 }
@@ -598,8 +601,10 @@ void vputs(v, s)
 VFILE *v;
 char *s;
 {
-	while (*s)
-		vputc(v, *s), ++s;
+	while (*s) {
+		vputc(v, *s);
+		++s;
+	}
 }
 
 /* Read a line from a file.  Remove '\n' if there was any */
@@ -614,7 +619,7 @@ char *s;
 	/* Return with NULL if at end of file */
 	if (vtell(v) == vsize(v)) {
 		vsrm(s);
-		return 0;
+		return NULL;
 	}
 
 	/* Create string if it doesn't exist */
@@ -731,38 +736,42 @@ char *s;
 				cnt -= 15;
 				goto ovr;
 			}
-		}
-		while (a += 16, b += 16, (cnt -= 16) >= 16);
+		} while (a += 16, b += 16, (cnt -= 16) >= 16);
 
 /*
- x=a, y=b; a+=cnt-15; b+=cnt-15;
- switch(cnt)
-  {
-  case 15: if((b[0]=a[0])=='\n') { a+=1; b+=1; goto zif; }
-  case 14: if((b[1]=a[1])=='\n') { a+=2; b+=2; goto zif; }
-  case 13: if((b[2]=a[2])=='\n') { a+=3; b+=3; goto zif; }
-  case 12: if((b[3]=a[3])=='\n') { a+=4; b+=4; goto zif; }
-  case 11: if((b[4]=a[4])=='\n') { a+=5; b+=5; goto zif; }
-  case 10: if((b[5]=a[5])=='\n') { a+=6; b+=6; goto zif; }
-  case 9: if((b[6]=a[6])=='\n')  { a+=7; b+=7; goto zif; }
-  case 8: if((b[7]=a[7])=='\n')  { a+=8; b+=8; goto zif; }
-  case 7: if((b[8]=a[8])=='\n')  { a+=9; b+=9; goto zif; }
-  case 6: if((b[9]=a[9])=='\n')  { a+=10; b+=10; goto zif; }
-  case 5: if((b[10]=a[10])=='\n'){ a+=11; b+=11; goto zif; }
-  case 4: if((b[11]=a[11])=='\n'){ a+=12; b+=12; goto zif; }
-  case 3: if((b[12]=a[12])=='\n'){ a+=13; b+=13; goto zif; }
-  case 2: if((b[13]=a[13])=='\n'){ a+=14; b+=14; goto zif; }
-  case 1: if((b[14]=a[14])=='\n'){ a+=15; b+=15; goto zif; }
-  }
- a=x+cnt, b=y+cnt; cnt=0; goto ovr;
- zif: cnt-=a-x-1;
+	x = a;
+	y = b;
+	a += cnt - 15;
+	b += cnt - 15;
+	switch(cnt) {
+	case 15:	if((b[0]=a[0])=='\n') { a+=1; b+=1; goto zif; }
+	case 14:	if((b[1]=a[1])=='\n') { a+=2; b+=2; goto zif; }
+	case 13:	if((b[2]=a[2])=='\n') { a+=3; b+=3; goto zif; }
+	case 12:	if((b[3]=a[3])=='\n') { a+=4; b+=4; goto zif; }
+	case 11:	if((b[4]=a[4])=='\n') { a+=5; b+=5; goto zif; }
+	case 10:	if((b[5]=a[5])=='\n') { a+=6; b+=6; goto zif; }
+	case 9:		if((b[6]=a[6])=='\n')  { a+=7; b+=7; goto zif; }
+	case 8:		if((b[7]=a[7])=='\n')  { a+=8; b+=8; goto zif; }
+	case 7:		if((b[8]=a[8])=='\n')  { a+=9; b+=9; goto zif; }
+	case 6:		if((b[9]=a[9])=='\n')  { a+=10; b+=10; goto zif; }
+	case 5:		if((b[10]=a[10])=='\n'){ a+=11; b+=11; goto zif; }
+	case 4:		if((b[11]=a[11])=='\n'){ a+=12; b+=12; goto zif; }
+	case 3:		if((b[12]=a[12])=='\n'){ a+=13; b+=13; goto zif; }
+	case 2:		if((b[13]=a[13])=='\n'){ a+=14; b+=14; goto zif; }
+	case 1:		if((b[14]=a[14])=='\n'){ a+=15; b+=15; goto zif; }
+	}
+	a = x + cnt;
+	b = y + cnt;
+	cnt=0;
+	goto ovr;
+zif:	cnt -= a - x - 1;
 */
 
 	if (cnt)
-		do
+		do {
 			if ((*b++ = *a++) == '\n')
 				break;
-		while (--cnt) ;
+		} while (--cnt);
 
       ovr:
 
