@@ -1,20 +1,23 @@
 /*
-	Window system
-	Copyright (C) 1992 Joseph H. Allen
-
-	This file is part of JOE (Joe's Own Editor)
-*/
-
+ *	Window system
+ *	Copyright
+ *		(C) 1992 Joseph H. Allen
+ *
+ *	This file is part of JOE (Joe's Own Editor)
+ */
 #include "config.h"
+#include "types.h"
+
 #ifdef HAVE_STDLIB_H
 #include <stdlib.h>
 #endif
-#include "b.h"
-#include "scrn.h"
-#include "queue.h"
-#include "main.h"
-#include "poshist.h"
+
 #include "blocks.h"
+#include "kbd.h"
+#include "poshist.h"
+#include "queue.h"
+#include "rc.h"
+#include "scrn.h"
 #include "utils.h"
 #include "w.h"
 
@@ -23,7 +26,7 @@ extern int staen;		/* 0 if top-most status line not displayed */
 
 /* Count no. of main windows */
 
-int countmain(SCREEN * t)
+int countmain(SCREEN *t)
 {
 	int nmain = 1;
 	W *m = t->curwin->main;
@@ -39,14 +42,14 @@ int countmain(SCREEN * t)
 
 /* Redraw a window */
 
-void wredraw(W * w)
+void wredraw(W *w)
 {
 	msetI(w->t->t->updtab + w->y, 1, w->h);
 }
 
 /* Find first window in a group */
 
-W *findtopw(W * w)
+W *findtopw(W *w)
 {
 	W *x;
 
@@ -57,7 +60,7 @@ W *findtopw(W * w)
 /* Determine height of a window.  Returns reqh if it is set, otherwise
  * used fixed or hh scaled to the current screen size */
 
-static int geth(W * w)
+static int geth(W *w)
 {
 	if (w->reqh)
 		return w->reqh;
@@ -69,7 +72,7 @@ static int geth(W * w)
 
 /* Set the height of a window */
 
-static void seth(W * w, int h)
+static void seth(W *w, int h)
 {
 	long tmp;
 
@@ -80,7 +83,7 @@ static void seth(W * w, int h)
 
 /* Determine height of a family of windows.  Uses 'reqh' if it's set */
 
-int getgrouph(W * w)
+int getgrouph(W *w)
 {
 	W *x;
 	int h;
@@ -96,7 +99,7 @@ int getgrouph(W * w)
 
 /* Determine minimum height of a family */
 
-static int getminh(W * w)
+static int getminh(W *w)
 {
 	W *x;
 	int h;
@@ -109,7 +112,7 @@ static int getminh(W * w)
 
 /* Find last window in a group */
 
-W *findbotw(W * w)
+W *findbotw(W *w)
 {
 	W *x;
 
@@ -120,7 +123,7 @@ W *findbotw(W * w)
 /* Demote group of window to end of window list.  Returns true if top window
    was demoted */
 
-int demotegroup(W * w)
+int demotegroup(W *w)
 {
 	W *top = findtopw(w);
 	W *bot = findbotw(w);
@@ -129,9 +132,10 @@ int demotegroup(W * w)
 
 	for (w = top; w != bot; w = next) {
 		next = w->link.next;
-		if (w == w->t->topwin)
-			flg = 1, w->t->topwin = next;
-		else
+		if (w == w->t->topwin) {
+			flg = 1;
+			w->t->topwin = next;
+		} else
 			demote(W, link, w->t->topwin, w);
 		w->y = -1;
 	}
@@ -145,7 +149,7 @@ int demotegroup(W * w)
 
 /* Find last window on the screen */
 
-W *lastw(SCREEN * t)
+W *lastw(SCREEN *t)
 {
 	W *x;
 
@@ -157,21 +161,21 @@ W *lastw(SCREEN * t)
 
 SCREEN *scr;
 
-SCREEN *screate(SCRN * scrn)
+SCREEN *screate(SCRN *scrn)
 {
-	SCREEN *t = (SCREEN *) malloc(sizeof(SCREEN));
+	SCREEN *t = (SCREEN *) joe_malloc(sizeof(SCREEN));
 
 	t->t = scrn;
 	t->w = scrn->co;
 	t->h = scrn->li;
-	t->topwin = 0;
-	t->curwin = 0;
+	t->topwin = NULL;
+	t->curwin = NULL;
 	t->wind = skiptop;
 	scr = t;
 	return t;
 }
 
-void sresize(SCREEN * t)
+void sresize(SCREEN *t)
 {
 	SCRN *scrn = t->t;
 	W *w;
@@ -183,9 +187,11 @@ void sresize(SCREEN * t)
 	if (t->wind < 0)
 		t->wind = 0;
 	w = t->topwin;
-	do
-		w->y = -1, w->w = t->w - 1;
-	while (w = w->link.next, w != t->topwin);
+	do {
+		w->y = -1;
+		w->w = t->w - 1;
+		w = w->link.next;
+	} while (w != t->topwin);
 	wfit(t);
 	updall();
 }
@@ -198,7 +204,7 @@ void updall(void)
 		scr->t->updtab[y] = 1;
 }
 
-void scrins(B * b, long l, long n, int flg)
+void scrins(B *b, long l, long n, int flg)
 {
 	W *w;
 
@@ -208,11 +214,12 @@ void scrins(B * b, long l, long n, int flg)
 				if (w->object && w->watom->ins)
 					w->watom->ins(w->object, b, l, n, flg);
 			}
-		} while (w = w->link.next, w != scr->topwin);
+		w = w->link.next;
+		} while (w != scr->topwin);
 	}
 }
 
-void scrdel(B * b, long l, long n, int flg)
+void scrdel(B *b, long l, long n, int flg)
 {
 	W *w;
 
@@ -222,7 +229,8 @@ void scrdel(B * b, long l, long n, int flg)
 				if (w->object && w->watom->del)
 					w->watom->del(w->object, b, l, n, flg);
 			}
-		} while (w = w->link.next, w != scr->topwin);
+		w = w->link.next;
+		} while (w != scr->topwin);
 	}
 }
 
@@ -236,7 +244,7 @@ void scrdel(B * b, long l, long n, int flg)
 static int doabort(W *w, int *ret);
 extern volatile int dostaupd;
 
-void wfit(SCREEN * t)
+void wfit(SCREEN *t)
 {
 	int y;			/* Where next window goes */
 	int left;		/* Lines left on screen */
@@ -252,14 +260,14 @@ void wfit(SCREEN * t)
       tryagain:
 	y = t->wind;
 	left = t->h - y;
-	pw = 0;
+	pw = NULL;
 
 	w = t->topwin;
 	do {
 		w->ny = -1;
 		w->nh = geth(w);
-	}
-	while ((w = w->link.next) != t->topwin);
+		w = w->link.next;
+	} while (w != t->topwin);
 
 	/* Fit a group of windows on the screen */
 	w = t->topwin;
@@ -273,8 +281,10 @@ void wfit(SCREEN * t)
 		/* Fit a family of windows on the screen */
 		do {
 			w->ny = y;	/* Set window's y position */
-			if (!w->win)
-				pw = w, w->nh -= adj;	/* Adjust main window of the group */
+			if (!w->win) {
+				pw = w;
+				w->nh -= adj;	/* Adjust main window of the group */
+			}
 			if (!w->win && w->nh < 2)
 				while (w->nh < 2)
 					w->nh += doabort(w->link.next, &ret);
@@ -283,10 +293,8 @@ void wfit(SCREEN * t)
 			y += w->nh;
 			left -= w->nh;	/* Increment y value by height of window */
 			w = w->link.next;	/* Next window */
-		}
-		while (w != t->topwin && w->main == w->link.prev->main);
-	}
-	while (w != t->topwin && left >= FITHEIGHT);
+		} while (w != t->topwin && w->main == w->link.prev->main);
+	} while (w != t->topwin && left >= FITHEIGHT);
 
 	/* We can't use extra space to fit a new family on, so give space to parent of
 	 * previous family */
@@ -306,7 +314,7 @@ void wfit(SCREEN * t)
 	 * happened
 	 */
 	w = t->topwin;
-	do
+	do {
 		if (w->y >= 0 && w->ny >= 0)
 			if (w->ny > w->y) {
 				W *l = pw = w;
@@ -346,7 +354,7 @@ void wfit(SCREEN * t)
 				w = w->link.next;
 		else
 			w = w->link.next;
-	while (w != t->topwin);
+	} while (w != t->topwin);
 
 	/* Update current height and position values */
 	w = t->topwin;
@@ -365,13 +373,13 @@ void wfit(SCREEN * t)
 			w->y = -1;
 		w->h = w->nh;
 		w->reqh = 0;
-	}
-	while (w = w->link.next, w != t->topwin);
+		w = w->link.next;
+	} while (w != t->topwin);
 }
 
 /* Goto next window */
 
-int wnext(SCREEN * t)
+int wnext(SCREEN *t)
 {
 	if (t->curwin->link.next != t->curwin) {
 		t->curwin = t->curwin->link.next;
@@ -384,7 +392,7 @@ int wnext(SCREEN * t)
 
 /* Goto previous window */
 
-int wprev(SCREEN * t)
+int wprev(SCREEN *t)
 {
 	if (t->curwin->link.prev != t->curwin) {
 		t->curwin = t->curwin->link.prev;
@@ -399,7 +407,7 @@ int wprev(SCREEN * t)
 
 /* Grow window */
 
-int wgrow(W * w)
+int wgrow(W *w)
 {
 	W *nextw;
 
@@ -428,7 +436,7 @@ int wgrow(W * w)
 
 /* Shrink window */
 
-int wshrink(W * w)
+int wshrink(W *w)
 {
 	W *nextw;
 
@@ -459,7 +467,7 @@ int wshrink(W * w)
 
 /* Show all windows */
 
-void wshowall(SCREEN * t)
+void wshowall(SCREEN *t)
 {
 	int n = 0;
 	int set;
@@ -467,10 +475,11 @@ void wshowall(SCREEN * t)
 
 	/* Count no. of main windows */
 	w = t->topwin;
-	do
+	do {
 		if (!w->win)
 			++n;
-	while (w = w->link.next, w != t->topwin) ;
+		w = w->link.next;
+	} while (w != t->topwin);
 
 	/* Compute size to set each window */
 	if ((set = (t->h - t->wind) / n) < FITHEIGHT)
@@ -478,7 +487,7 @@ void wshowall(SCREEN * t)
 
 	/* Set size of each variable size window */
 	w = t->topwin;
-	do
+	do {
 		if (!w->win) {
 			int h = getminh(w);
 
@@ -486,23 +495,25 @@ void wshowall(SCREEN * t)
 				seth(w, 2);
 			else
 				seth(w, set - (h - 2));
-			w->orgwin = 0;
+			w->orgwin = NULL;
 		}
-	while (w = w->link.next, w != t->topwin) ;
+		w = w->link.next;
+	} while (w != t->topwin);
 
 	/* Do it */
 	wfit(t);
 }
 
-static void wspread(SCREEN * t)
+static void wspread(SCREEN *t)
 {
 	int n = 0;
 	W *w = t->topwin;
 
-	do
+	do {
 		if (w->y >= 0 && !w->win)
 			++n;
-	while (w = w->link.next, w != t->topwin) ;
+		w = w->link.next;
+	} while (w != t->topwin);
 	if (!n) {
 		wfit(t);
 		return;
@@ -512,7 +523,7 @@ static void wspread(SCREEN * t)
 	else
 		n = FITHEIGHT;
 	w = t->topwin;
-	do
+	do {
 		if (!w->win) {
 			int h = getminh(w);
 
@@ -520,38 +531,40 @@ static void wspread(SCREEN * t)
 				seth(w, 2);
 			else
 				seth(w, n - (h - 2));
-			w->orgwin = 0;
+			w->orgwin = NULL;
 		}
-	while (w = w->link.next, w != t->topwin) ;
+		w = w->link.next;
+	} while (w != t->topwin);
 	wfit(t);
 }
 
 /* Show just one family of windows */
 
-void wshowone(W * w)
+void wshowone(W *w)
 {
 	W *q = w->t->topwin;
 
-	do
+	do {
 		if (!q->win) {
 			seth(q, w->t->h - w->t->wind - (getminh(q) - 2));
-			q->orgwin = 0;
+			q->orgwin = NULL;
 		}
-	while (q = q->link.next, q != w->t->topwin) ;
+		q = q->link.next;
+	} while (q != w->t->topwin);
 	wfit(w->t);
 }
 
 /* Create a window */
 
-W *wcreate(SCREEN * t, WATOM * watom, W * where, W * target, W * original, int height, char *huh, int *notify)
+W *wcreate(SCREEN *t, WATOM *watom, W *where, W *target, W *original, int height, char *huh, int *notify)
 {
 	W *new;
 
 	if (height < 1)
-		return 0;
+		return NULL;
 
 	/* Create the window */
-	new = (W *) malloc(sizeof(W));
+	new = (W *) joe_malloc(sizeof(W));
 	new->notify = notify;
 	new->t = t;
 	new->w = t->w - 1;
@@ -564,9 +577,9 @@ W *wcreate(SCREEN * t, WATOM * watom, W * where, W * target, W * original, int h
 	new->huh = huh;
 	new->orgwin = original;
 	new->watom = watom;
-	new->object = 0;
-	new->msgb = 0;
-	new->msgt = 0;
+	new->object = NULL;
+	new->msgb = NULL;
+	new->msgt = NULL;
 	/* Set window's target and family */
 /* was:	if (new->win = target) {	which may be mistyped == */
 	if ((new->win = target) != NULL) {	/* A subwindow */
@@ -581,17 +594,17 @@ W *wcreate(SCREEN * t, WATOM * watom, W * where, W * target, W * original, int h
 	if (original) {
 		if (original->h - height <= 2) {
 			/* Not enough space for window */
-			free(new);
-			return 0;
+			joe_free(new);
+			return NULL;
 		} else
 			seth(original, original->h - height);
 	}
 
 	/* Create new keyboard handler for window */
 	if (watom->context)
-		new->kbd = mkkbd(getcontext(watom->context));
+		new->kbd = mkkbd(kmap_getcontext(watom->context));
 	else
-		new->kbd = 0;
+		new->kbd = NULL;
 
 	/* Put window on the screen */
 	if (where)
@@ -599,8 +612,10 @@ W *wcreate(SCREEN * t, WATOM * watom, W * where, W * target, W * original, int h
 	else {
 		if (t->topwin)
 			enqueb(W, link, t->topwin, new);
-		else
-			izque(W, link, new), t->curwin = t->topwin = new;
+		else {
+			izque(W, link, new);
+			t->curwin = t->topwin = new;
+		}
 	}
 
 	return new;
@@ -608,7 +623,7 @@ W *wcreate(SCREEN * t, WATOM * watom, W * where, W * target, W * original, int h
 
 /* Abort group of windows */
 
-static int doabort(W * w, int *ret)
+static int doabort(W *w, int *ret)
 {
 	int amnt = geth(w);
 	W *z;
@@ -620,13 +635,12 @@ static int doabort(W * w, int *ret)
 	z = w->t->topwin;
 	do {
 		if (z->orgwin == w)
-			z->orgwin = 0;
+			z->orgwin = NULL;
 		if ((z->win == w || z->main == w) && z->y != -2) {
 			amnt += doabort(z, ret);
 			goto loop;
 		}
-	}
-	while (z = z->link.next, z != w->t->topwin);
+	} while (z = z->link.next, z != w->t->topwin);
 	if (w->orgwin)
 		seth(w->orgwin, geth(w->orgwin) + geth(w));
 	if (w->t->curwin == w) {
@@ -652,14 +666,14 @@ static int doabort(W * w, int *ret)
 			*w->notify = 1;
 	}
 	rmkbd(w->kbd);
-	free(w);
+	joe_free(w);
 	windie(w);
 	return amnt;
 }
 
 /* Abort a window and its children */
 
-int wabort(W * w)
+int wabort(W *w)
 {
 	SCREEN *t = w->t;
 	int ret;
@@ -682,7 +696,7 @@ int wabort(W * w)
 
 /* Generate text with formatting escape sequences */
 
-void genfmt(SCRN * t, int x, int y, int ofst, char *s, int flg)
+void genfmt(SCRN *t, int x, int y, int ofst, char *s, int flg)
 {
 	int *scrn = t->scrn + y * t->co + x;
 	int atr = 0;
@@ -690,44 +704,39 @@ void genfmt(SCRN * t, int x, int y, int ofst, char *s, int flg)
 	int c;
 
 	while ((c = *s++) != '\0')
-		if (c == '\\')
+		if (c == '\\') {
 			switch (c = *s++) {
-				case 'u':
-				case 'U':
+			case 'u':
+			case 'U':
 				atr ^= UNDERLINE;
 				break;
-
-				case 'i':
-				case 'I':
+			case 'i':
+			case 'I':
 				atr ^= INVERSE;
 				break;
-
-				case 'b':
-				case 'B':
+			case 'b':
+			case 'B':
 				atr ^= BOLD;
 				break;
-
-				case 'd':
-				case 'D':
+			case 'd':
+			case 'D':
 				atr ^= DIM;
 				break;
-
-				case 'f':
-				case 'F':
+			case 'f':
+			case 'F':
 				atr ^= BLINK;
 				break;
-
-				case 0:
+			case 0:
 				--s;
 				break;
-
-				default:
+			default:
 				if (col++ >= ofst) {
 					outatr(t, scrn, x, y, c, atr);
 					++scrn;
 					++x;
 				}
 				break;
+			}
 		} else if (col++ >= ofst) {
 			if (c == '\t')
 				c = ' ';
@@ -741,19 +750,19 @@ void genfmt(SCRN * t, int x, int y, int ofst, char *s, int flg)
 
 /* Generate text: no formatting */
 
-void gentxt(SCRN * t, int x, int y, int ofst, char *s, int len, int flg)
+void gentxt(SCRN *t, int x, int y, int ofst, char *s, int len, int flg)
 {
 	int *scrn = t->scrn + y * t->co + x;
 	int col;
-	int c;
+	unsigned char c;
 	int a;
 
 	for (col = 0; col != len; ++col)
 		if (col >= ofst) {
-			c = (unsigned) s[col];
+			c = s[col];
 			if (c == '\t')
 				c = ' ';
-			xlat(a, c);
+			xlat(&a, &c);
 			outatr(t, scrn, x, y, c, a);
 			++scrn;
 			++x;
@@ -769,27 +778,26 @@ int fmtlen(char *s)
 	int col = 0;
 
 	while (*s) {
-		if (*s == '\\')
+		if (*s == '\\') {
 			switch (*++s) {
-				case 'u':
-				case 'i':
-				case 'd':
-				case 'f':
-				case 'b':
-				case 'U':
-				case 'I':
-				case 'D':
-				case 'F':
-				case 'B':
+			case 'u':
+			case 'i':
+			case 'd':
+			case 'f':
+			case 'b':
+			case 'U':
+			case 'I':
+			case 'D':
+			case 'F':
+			case 'B':
 				++s;
-				goto cont;
-
-				case 0:
+				continue;
+			case 0:
 				--s;
 			}
+		}
 		++col;
 		++s;
-	      cont:;
 	}
 	return col;
 }
@@ -805,32 +813,30 @@ int fmtpos(char *s, int goal)
 	while (*s && col != goal) {
 		if (*s == '\\')
 			switch (*++s) {
-				case 'u':
-				case 'i':
-				case 'd':
-				case 'f':
-				case 'b':
-				case 'U':
-				case 'I':
-				case 'D':
-				case 'F':
-				case 'B':
+			case 'u':
+			case 'i':
+			case 'd':
+			case 'f':
+			case 'b':
+			case 'U':
+			case 'I':
+			case 'D':
+			case 'F':
+			case 'B':
 				++s;
-				goto cont;
-
-				case 0:
+				continue;
+			case 0:
 				--s;
 			}
 		++col;
 		++s;
-	      cont:;
 	}
 	return s - org + goal - col;
 }
 
 /* Display a message and skip the next key */
 
-static void mdisp(SCRN * t, int y, char *s)
+static void mdisp(SCRN *t, int y, char *s)
 {
 	int ofst;
 	int len;
@@ -844,7 +850,7 @@ static void mdisp(SCRN * t, int y, char *s)
 	t->updtab[y] = 1;
 }
 
-void msgout(W * w)
+void msgout(W *w)
 {
 	SCRN *t = w->t->t;
 
@@ -860,20 +866,20 @@ void msgout(W * w)
 
 /* Set temporary message */
 
-char msgbuf[MSGBUFSIZE];
+char msgbuf[JOE_MSGBUFSIZE];
 
 /* display message on bottom line of window */
-void msgnw(BASE *w, char *s)
+void msgnw(W *w, char *s)
 {
-	w->parent->msgb = s;
+	w->msgb = s;
 }
 
-void msgnwt(BASE *w, char *s)
+void msgnwt(W *w, char *s)
 {
-	w->parent->msgt = s;
+	w->msgt = s;
 }
 
-int urtn(BASE * b, int k)
+int urtn(BASE *b, int k)
 {
 	if (b->parent->watom->rtn)
 		return b->parent->watom->rtn(b, k);
@@ -881,7 +887,7 @@ int urtn(BASE * b, int k)
 		return -1;
 }
 
-int utype(BASE * b, int k)
+int utype(BASE *b, int k)
 {
 	if (b->parent->watom->type)
 		return b->parent->watom->type(b, k);
@@ -891,27 +897,27 @@ int utype(BASE * b, int k)
 
 /* Window user commands */
 
-int uprevw(BASE * bw)
+int uprevw(BASE *bw)
 {
 	return wprev(bw->parent->t);
 }
 
-int unextw(BASE * bw)
+int unextw(BASE *bw)
 {
 	return wnext(bw->parent->t);
 }
 
-int ugroww(BASE * bw)
+int ugroww(BASE *bw)
 {
 	return wgrow(bw->parent);
 }
 
-int ushrnk(BASE * bw)
+int ushrnk(BASE *bw)
 {
 	return wshrink(bw->parent);
 }
 
-int uexpld(BASE * bw)
+int uexpld(BASE *bw)
 {
 	if (bw->parent->t->h - bw->parent->t->wind == getgrouph(bw->parent))
 		wshowall(bw->parent->t);
@@ -920,7 +926,7 @@ int uexpld(BASE * bw)
 	return 0;
 }
 
-int uretyp(BASE * bw)
+int uretyp(BASE *bw)
 {
 	nredraw(bw->parent->t->t);
 	return 0;
