@@ -1,98 +1,15 @@
 /*
-	Window management
-	Copyright (C) 1992 Joseph H. Allen
-
-	This file is part of JOE (Joe's Own Editor)
-*/
-
-#ifndef _Iw
-#define _Iw 1
+ *	Window management
+ *	Copyright
+ *		(C) 1992 Joseph H. Allen
+ *
+ *	This file is part of JOE (Joe's Own Editor)
+ */
+#ifndef _JOE_W_H
+#define _JOE_W_H 1
 
 #include "config.h"
-#include "queue.h"
-#include "scrn.h"
-#include "kbd.h"
-#include "b.h"
-
-typedef struct watom WATOM;
-typedef struct screen SCREEN;
-typedef struct window W;
-typedef struct base BASE;
-
-struct watom {
-	char *context;		/* Context name */
-	void (*disp) ();	/* Display window */
-	void (*follow) ();	/* Called to have window follow cursor */
-	int (*abort) ();	/* Common user functions */
-	int (*rtn) ();
-	int (*type) ();
-	void (*resize) ();	/* Called when window changed size */
-	void (*move) ();	/* Called when window moved */
-	void (*ins) ();		/* Called on line insertions */
-	void (*del) ();		/* Called on line deletions */
-	int what;		/* Type of this thing */
-};
-
-struct screen {
-	SCRN *t;		/* Screen data on this screen is output to */
-
-	int wind;		/* Number of help lines on this screen */
-
-	W *topwin;		/* Top-most window showing on screen */
-	W *curwin;		/* Window cursor is in */
-
-	int w, h;		/* Width and height of this screen */
-};
-
-struct window {
-	LINK(W) link;		/* Linked list of windows in order they
-				   appear on the screen */
-
-	SCREEN *t;		/* Screen this thing is on */
-
-	int x, y, w, h;		/* Position and size of window */
-	/* Currently, x=0, w=width of screen. */
-	/* y== -1 if window is not on screen */
-
-	int ny, nh;		/* Temporary values for wfit */
-
-	int reqh;		/* Requested new height or 0 for same */
-	/* This is an argument for wfit */
-
-	int fixed;		/* If this is zero, use 'hh'.  If not, this
-				   is a fixed size window and this variable
-				   gives its height */
-
-	int hh;			/* Height window would be on a screen with
-				   1000 lines.  When the screen size changes
-				   this is used to calculate the window's
-				   real height */
-
-	W *win;			/* Window this one operates on */
-	W *main;		/* Main window of this family */
-	W *orgwin;		/* Window where space from this window came */
-	int curx, cury;		/* Cursor position within window */
-	KBD *kbd;		/* Keyboard handler for this window */
-	WATOM *watom;		/* The type of this window */
-	void *object;		/* Object which inherits this */
-
-	char *msgt;		/* Message at top of window */
-
-	char *msgb;		/* Message at bottom of window */
-
-	char *huh;		/* Name of window for context sensitive hlp */
-
-	int *notify;		/* Address of kill notification flag */
-};
-
-/* Anything which goes in window.object must start like this: */
-
-struct base {
-	W *parent;
-};
-
-/* Minimum text window height */
-#define FITHEIGHT 4
+#include "types.h"
 
 /***************/
 /* Subroutines */
@@ -231,16 +148,16 @@ void gentxt PARAMS((SCRN *t, int x, int y, int ofst, char *s, int len, int flg))
 int fmtlen PARAMS((char *s));
 int fmtpos PARAMS((char *s, int goal));
 
-/* void msgnw[t](W *w,char *text);
+/* void msgnw[t](W *w, char *s);
  * Display a message which will be eliminated on the next keypress.
  * msgnw displays message on bottom line of window
  * msgnwt displays message on top line of window
  */
-void msgnw PARAMS((BASE *w, char *s));
-void msgnwt PARAMS((BASE *w, char *s));
+void msgnw PARAMS((W *w, char *s));
+void msgnwt PARAMS((W *w, char *s));
 
-#define MSGBUFSIZE 300
-extern char msgbuf[MSGBUFSIZE];	/* Message composition buffer for msgnw/msgnwt */
+#define JOE_MSGBUFSIZE 300
+extern char msgbuf[JOE_MSGBUFSIZE];	/* Message composition buffer for msgnw/msgnwt */
 
 void msgout PARAMS((W *w));			/* Output msgnw/msgnwt messages */
 
